package com.mints.street

import android.content.Context
import android.os.StrictMode
import android.os.StrictMode.VmPolicy
import com.baidu.lbsapi.BMapManager
import com.baidu.mapapi.CoordType
import com.baidu.mapapi.SDKInitializer
import com.cuieney.rxpay_annotation.WX
import com.fry.base.base.BaseApp
import com.mints.street.ad.CsjGroMoreManager
import com.mints.street.main.vr.PanoramicActivity
import com.mints.street.manager.UmengManager
import com.orhanobut.logger.AndroidLogAdapter
import com.orhanobut.logger.FormatStrategy
import com.orhanobut.logger.Logger
import com.orhanobut.logger.PrettyFormatStrategy
import com.tencent.bugly.crashreport.CrashReport
import me.goldze.mvvmhabit.utils.KLog

/**
 * Created by 冯瑞雨 on 2021/7/1.
 */
@WX(packageName = "com.mints.street")
class AppApplication : BaseApp() {

    override fun onCreate() {
        super.onCreate()
        mContext = this

        thirdConfig()

        val builder = VmPolicy.Builder()
        StrictMode.setVmPolicy(builder.build())
        builder.detectFileUriExposure()

        // 解决java.util.concurrent.TimeoutException https://www.jianshu.com/p/844ef9c84f15
//        val defaultUncaughtExceptionHandler = Thread.getDefaultUncaughtExceptionHandler()
//        Thread.setDefaultUncaughtExceptionHandler { t, e ->
//            if (t.name == "FinalizerWatchdogDaemon" && e is TimeoutException) {
//                Log.e("ignore", "ignore")
//            } else {
//                defaultUncaughtExceptionHandler?.uncaughtException(t, e)
//            }
//        }
    }

    /*
     * 三方配置
     */
    private fun thirdConfig() {
        // 友盟预初始化
        UmengManager.preInit()

        // bugly
        CrashReport.initCrashReport(this, "df5783aab4", BuildConfig.DEBUG)

        // GroMore
        CsjGroMoreManager.init(this)


        //百度地图
        // 在使用 SDK 各组间之前初始化 context 信息，传入 ApplicationContext
        // 默认本地个性化地图初始化方法
        SDKInitializer.initialize(this)
        //自4.3.0起，百度地图SDK所有接口均支持百度坐标和国测局坐标，用此方法设置您使用的坐标类型.
        //包括BD09LL和GCJ02两种坐标，默认是BD09LL坐标。
        SDKInitializer.setCoordType(CoordType.BD09LL)
        //百度全景
        initEngineManager(this)

        bindLogger()
    }
    /**
     * 初始化百度全景
     */
    private fun initEngineManager(context: Context) {
        if (mBMapManager == null) {
            mBMapManager = BMapManager(context)
        }
        if (mBMapManager?.init(PanoramicActivity.MyGeneralListener())==false) {
            KLog.e("BMapManager","BMapManager  初始化错误!")
        }
    }

    /**
     * 自定义logger
     */
    private fun bindLogger() {
        //自定义logger
        val formatStrategy: FormatStrategy = PrettyFormatStrategy.newBuilder()
                .showThreadInfo(false) //是否选择显示线程信息，默认为true
                .methodCount(0) //方法数显示多少行，默认2行
                .methodOffset(7) //隐藏方法内部调用到偏移量，默认5
                // .logStrategy(customLog) //打印日志的策略，默认LogCat
                .tag("mints---network") //自定义TAG全部标签，默认PRETTY_LOGGER
                .build()
        Logger.addLogAdapter(AndroidLogAdapter(formatStrategy))
    }

    companion object {
        var mContext: AppApplication? = null
        var mBMapManager: BMapManager? = null

        fun getContext(): AppApplication? {
            return mContext
        }
    }


}