package com.mints.street.adapter

import android.content.Intent
import android.text.TextUtils
import android.view.View
import androidx.databinding.ViewDataBinding
import com.alibaba.android.vlayout.layout.LinearLayoutHelper
import com.fry.base.adapter.AbstractVLayoutBaseAdapter
import com.fry.base.adapter.BindingViewHolder
import com.fry.base.recycler_view.VHMAdapter
import com.fry.base.recycler_view.VHModel
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.bean.HistoryBean
import com.mints.street.databinding.ItemHistoryRecordAdapterBinding
import com.mints.street.main.home.SearchMapActivity
import com.mints.street.main.home.SearchMapViewModel
import me.goldze.mvvmhabit.binding.command.BindingAction
import me.goldze.mvvmhabit.binding.command.BindingCommand
import me.goldze.mvvmhabit.utils.KLog
import me.goldze.mvvmhabit.utils.RxUtils
import me.goldze.mvvmhabit.utils.SPUtils
import java.lang.reflect.Type

/**
 * Created by 冯瑞雨 on 2021/7/14.
 * 历史记录的adapter
 */
class HistoryRecordAdapter(val activity: SearchMapActivity,val viewModel: SearchMapViewModel
    , var type: Int=0,var name: String? = null) :
    AbstractVLayoutBaseAdapter<ItemHistoryRecordAdapterBinding, String>(activity, name, 1) {

    var listName: MutableList<HistoryBean>?=null

    var binding:ItemHistoryRecordAdapterBinding?= null
    override fun onBindViewHolder(
        holder: BindingViewHolder<ItemHistoryRecordAdapterBinding>,
        position: Int
    ) {
        binding = holder.binding
        listName  = getHistoryData()
        if (listName!=null && listName!!.size>0){
            binding?.historyBg?.visibility = View.VISIBLE
        }else{
            binding?.historyBg?.visibility = View.GONE
        }
        val adapter = VHMAdapter<HistoryRecordItemVHM>()
        listName?.let {
            for (index in it.size - 1 downTo 0) {
                adapter.add(HistoryRecordItemVHM(activity,viewModel,it[index]))
            }
        }
        holder.binding.listView.adapter = adapter
        //清空
        RxUtils.onMultiClick(binding?.tvClear){
            listName?.clear()
            SPUtils.getInstance(SearchMapActivity.HISTORY_NAME)
                .removeKey(if (type==0){"Territory"}else{"Abroad"})
            binding?.historyBg?.visibility = View.GONE
            notifyDataSetChanged()
        }
    }

    private fun getHistoryData(): MutableList<HistoryBean>? {
        val instance = SPUtils.getInstance(SearchMapActivity.HISTORY_NAME)
        val string = instance.decodeString(if (type==0){"Territory"}else{"Abroad"})
        if (TextUtils.isEmpty(string)) {
            return null
        }
        return try {
            val listType: Type = object : TypeToken<List<HistoryBean?>?>() {}.type
            Gson().fromJson(string, listType)
        }catch (e:Exception){
            null
        }
    }

    fun addHistoryData(addName: HistoryBean) {
        if (listName == null) {
            listName = ArrayList<HistoryBean>()
        }
        if (listName?.size ?: 0 >= 15) {
            val size = listName!!.size - 15
            for (index in 0..size) {
                listName?.removeAt(index)
            }
        }
        for (index in listName!!.indices) {
            if (listName!![index].name == addName.name) {
                listName?.removeAt(index)
                break
            }
        }
        if (listName!=null && listName!!.size>0 && binding?.historyBg?.visibility == View.GONE){
            binding?.historyBg?.visibility = View.VISIBLE
        }
        listName?.add(addName)
        val instance = SPUtils.getInstance(SearchMapActivity.HISTORY_NAME)
        instance?.encode(if (type==0){"Territory"}else{"Abroad"}, Gson().toJson(listName).toString())
    }

    override fun getItemCount() = 1

    override fun onCreateLayoutHelper() = LinearLayoutHelper()

    override fun getLayoutId(viewType: Int) = R.layout.item_history_record_adapter

}

class HistoryRecordItemVHM(val activity: SearchMapActivity,val viewModel: SearchMapViewModel,
    var historyBean: HistoryBean
) : VHModel() {
    override fun getLayoutId() = R.layout.item_history_record_name

    override fun getVariableId() = BR.vhm

    override fun onBind(binding: ViewDataBinding) {
        super.onBind(binding)

    }
    val onClick = BindingCommand<Any>(BindingAction {
        viewModel.searchName.value = HistoryBean(name = historyBean.name,
            latitude = historyBean.latitude,longitude = historyBean.longitude)
        val intent = Intent()
        intent.putExtra("latitude",historyBean.latitude)
        intent.putExtra("longitude",historyBean.longitude)
        viewModel.finishData(SearchMapActivity.REQUEST_CODE,intent)
    })
}