package com.mints.street.login

import android.Manifest
import android.graphics.Color
import android.os.Bundle
import android.view.View
import androidx.lifecycle.Observer
import com.fry.base.base.BaseActivity
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.databinding.ActivityLoginBinding
import com.mints.street.main.MainActivity
import com.mints.street.manager.UserManager
import com.mints.street.utils.BackInputUtil
import com.mints.street.widget.dialog.DialogUtils
import com.tbruyelle.rxpermissions2.RxPermissions
import kotlinx.android.synthetic.main.activity_bindmobile.*
import kotlinx.android.synthetic.main.activity_login.*
import kotlinx.android.synthetic.main.include_header.*
import me.goldze.mvvmhabit.utils.AppUtils
import me.goldze.mvvmhabit.utils.ToastUtils
import me.goldze.mvvmhabit.utils.systembar.StatusBarUtil

/**
 * Created by 冯瑞雨 on 2021/7/2.
 */
class LoginActivity : BaseActivity<ActivityLoginBinding, LoginViewModel>(), View.OnClickListener {

    private val userManager by lazy { UserManager.INSTANCE }
    override fun getStatusBarHeightView(): View? {
        return binding.tvName
    }
    override fun initContentView(savedInstanceState: Bundle?) = R.layout.activity_login

    override fun initVariableId() = BR.viewModel

    override fun initData() {
        super.initData()
        initView()
        initListener()
    }

    private fun initListener() {

        binding.tvSendSms.setOnClickListener {

//            ToastUtils.showLong("点击获取验证码")
            var mobile = et_phone.text.toString().trim()

            if (mobile.length < 13) {
                ToastUtils.showLong("请输入手机号")
                return@setOnClickListener
            }

            ToastUtils.showLong("验证码已发送")
            sendCodeThread()

            mobile = mobile.replace(" ".toRegex(), "")
            viewModel.sendMobileCode(mobile)

        }

        binding.button.setOnClickListener {

            var mobile = et_phone.text.toString().trim()
            if (mobile.length < 13) {
                ToastUtils.showLong("请输入手机号")
                return@setOnClickListener
            }

            val code = et_sms.text.toString().trim()
            if (code.length < 4) {
                ToastUtils.showLong("请输入验证码")
                return@setOnClickListener
            }

            RxPermissions(this@LoginActivity)
                    .request(Manifest.permission.READ_PHONE_STATE)
                    .subscribe { granted: Boolean ->
                        if (granted) {

                            mobile = mobile.replace(" ".toRegex(), "")
                            // 登录
                            viewModel.mobilelogin(mobile, code)
                        } else {
                            DialogUtils.showPermissionDialog(this@LoginActivity,"设备")
                        }
                    }

        }
        iv_left_icon.setOnClickListener(this)
        iv_left_icon.setOnClickListener(this)
    }

    private fun initView() {

        tv_title.text = "手机号登录"
        tv_title.setTextColor(Color.WHITE)
        iv_left_icon.setImageResource(R.mipmap.ic_arrow_back_white)
        iv_left_icon.setPadding(AppUtils.dp2Px(this, 15F), AppUtils.dp2Px(this, 15F), AppUtils.dp2Px(this, 15F), AppUtils.dp2Px(this, 15F))
        header_line.visibility = View.GONE
        iv_left_icon.visibility = View.VISIBLE

        //手机号设置格式
        BackInputUtil.phoneNumAddSpace(et_phone)
        /*mobile = UserManager.INSTANCE.getMobile()
        if (!TextUtils.isEmpty(mobile)) {
            et_sms.setText(mobile)
            et_sms.setSelection(mobile!!.length + 2)
        }*/

    }

    override fun initViewObservable() {
        super.initViewObservable()
        viewModel.data.observe(this, Observer<String> {
            ToastUtils.showLong("登录成功")
            finish()
        })

    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.iv_left_icon -> finish()
        }
    }

    /**
     * 发送手机验证码，按钮动态改变状态
     */
    var num = 0
    var run: Runnable? = null
    fun sendCodeThread() {
        num = 60
        tv_send_sms?.let {
            it.isEnabled = false
            it.text = "（$num）重新获取"

            run = object : Runnable {
                override fun run() {
                    num--
                    if (num == 0) {
                        it.text = "重新获取"
                        it.isEnabled = true
                    } else {
                        it.text = "（$num）重新获取"
                        it.postDelayed(this, 1000)
                    }
                }
            }
            it.postDelayed(run, 1000)
        }
    }
}