package com.mints.street.login

import android.app.Application
import androidx.lifecycle.MutableLiveData
import com.mints.street.bean.BaseResponse
import com.mints.street.bean.UserBean
import com.mints.street.common.DeviceInfo
import com.mints.street.manager.UserManager
import com.mints.street.manager.oaid.OaidManager
import com.mints.street.model.ApiModel
import com.mints.street.netwrok.base.HttpSubscribeImpl
import com.mints.street.utils.DeviceUuidFactory
import me.goldze.mvvmhabit.base.BaseViewModel
import me.goldze.mvvmhabit.utils.KLog
import java.util.HashMap

/**
 * Created by mcg
 */
class LoginViewModel(application: Application) : BaseViewModel(application) {

    val data :MutableLiveData<String>  = MutableLiveData()

    val deviceInfo: DeviceInfo
        get() {
            return DeviceInfo.instance
        }

    /**
     * 获取手机验证码
     *
     * @param mobile
     */
    fun sendMobileCode(mobile: String) {
        val vo = HashMap<String, Any>()
        vo["mobile"] = mobile
//        vo["type"] = 1

        ApiModel.sendMobileCode(lifecycleProvider,vo).subscribe(
                object : HttpSubscribeImpl<BaseResponse<Any>>(
                        this@LoginViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<Any>) {
                       // KLog.e("sendMobileCode", response.result.toString())
                    }
                    override fun onError(e: Throwable) {
                        KLog.e("sendMobileCode", "发送验证码错误")
                    }
                })
    }

    /**
     * 调用登录接口
     */
    fun mobilelogin(mobile: String, smsCode: String) {
//        showDialog()
        val vo = HashMap<String, Any>()
        vo["mobile"] = mobile
        vo["smsCode"] = smsCode
//        vo["device"] = DeviceUuidFactory().deviceUuid.toString()

        ApiModel.login(lifecycleProvider,vo).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<UserBean>>(
                        this@LoginViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<UserBean>) {

                        //提交设备信息【注：这里海哥的意思是说在测试的环境下，进行当前的注册是利用手机号进行注册，由于开通了微信登录，就不需要再手机号登录的时候进行设备信息的提交】
                        this@LoginViewModel.saveTerminalInfo()

                        //保存用户信息到Usermanager
                        UserManager.INSTANCE.saveUserInfo(response.result)

                        data.value  = response.message

                        KLog.e("login", response.result.toString())
//                        ToastUtils.showShort("登录信息="+response.result.consumer?.idcode)

                    }

                    override fun onError(e: Throwable) {
                        KLog.e("LoginViewModel", "登录错误")

                    }
                })
    }

    /**
     *  提交设备信息
     */
    fun saveTerminalInfo() {
        val vo = hashMapOf<String, Any>()
        val macAddress: String = deviceInfo.getMacAddress()
        val mac = macAddress.replace(":", "")
        vo["mac"] = mac
        vo["mac1"] = macAddress
        vo["androidid"] = deviceInfo.getAndroidId(null)
        vo["imei"] = deviceInfo.iMEI
        vo["oaid"] =OaidManager.getOaid()
        vo["os"] = "android"
        vo["model"] = deviceInfo.newModel
        vo["uuid"] = DeviceUuidFactory().deviceUuid
        vo["osversion"] = deviceInfo.oSVersion
        vo["appversion"] = deviceInfo.versionName

        ApiModel.saveTerminalInfo(lifecycleProvider,vo).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<Any>>(
                        this@LoginViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<Any>) {
                        KLog.e("saveTerminalInfo", response.result.toString())
                    }
                })
    }
}