package com.mints.street.main

import android.Manifest
import android.os.Build
import android.os.Bundle
import android.view.KeyEvent
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentActivity
import androidx.viewpager2.adapter.FragmentStateAdapter
import androidx.viewpager2.widget.ViewPager2
import com.fry.base.base.BaseActivity
import com.fry.base.utils.ImageLoader
import com.ittianyu.bottomnavigationviewex.BottomNavigationViewEx
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.databinding.ActivityMainBinding
import com.mints.street.main.googlemap.GoogleMapFragment
import com.mints.street.main.home.HomeFragment
import com.mints.street.main.my.MyFragment
import com.mints.street.main.vr.VRFragment
import com.mints.street.manager.UserManager
import com.mints.street.manager.oaid.OaidManager
import com.tbruyelle.rxpermissions2.RxPermissions
import me.goldze.mvvmhabit.base.AppManager
import me.goldze.mvvmhabit.utils.ToastUtils


class MainActivity : BaseActivity<ActivityMainBinding, MainViewModel>() {
    override fun initContentView(savedInstanceState: Bundle?) = R.layout.activity_main

    override fun initVariableId() = BR.viewModel


    override fun getStatusBarHeightView(): View? {
        return null
    }
    override fun initData() {
        super.initData()
        OaidManager.init()

        //初始化viewpager2
        binding.mainViewpager.initMain(this)
        //初始化 bottomBarF
        binding.mainBottom.init {
            when (it) {
                R.id.menu_home -> {
                    if(UserManager.INSTANCE.userIsLogin()&&UserManager.INSTANCE.getVipFlag()){
                        binding.mainViewpager.setCurrentItem(3, false)

                    }else{
                        binding.mainViewpager.setCurrentItem(0, false)
                    }
                }
                R.id.menu_square -> binding.mainViewpager.setCurrentItem(1, false)
                R.id.menu_my -> {
                    binding.mainViewpager.setCurrentItem(2, false)
//                     stopAnim()
                }
            }
        }
        //加载本地GIF动图
        ImageLoader.ImageBuilder.with(this)
                .setDrawable(R.mipmap.ic_earth_map)
                .setTargetView(binding.ivTakephoto).start()

        binding.mainBottom.interceptLongClick(
                R.id.menu_home,
                R.id.menu_square,
                R.id.menu_my
        )

        RxPermissions(this).request(
            if( Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) Manifest.permission.ACCESS_BACKGROUND_LOCATION else "",
           Manifest.permission.ACCESS_FINE_LOCATION ,
             Manifest.permission.WRITE_EXTERNAL_STORAGE ,
             Manifest.permission.READ_PHONE_STATE
        )
            .subscribe {
            }
    }




    var oldTime: Long = 0
    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        when (keyCode) {
            KeyEvent.KEYCODE_BACK -> {
                // 设置为后台
                val currentTime = System.currentTimeMillis()
                if (currentTime - oldTime < 2 * 1000) {
                    AppManager.getAppManager().finishAllActivity()
                } else {
                    ToastUtils.showLong("再次点击退出" + getString(R.string.app_name))
                    oldTime = currentTime
                }
            }
        }
        return true
    }

}


private fun BottomNavigationViewEx.init(navigationItemSelectedAction: (Int) -> Unit): BottomNavigationViewEx {
    enableAnimation(true)
    enableShiftingMode(false)
    enableItemShiftingMode(true)
    setTextSize(12F)
    setOnNavigationItemSelectedListener {
        navigationItemSelectedAction.invoke(it.itemId)
        true
    }
    return this
}

private fun ViewPager2.initMain(activity: FragmentActivity): ViewPager2 {
    //是否可滑动
    this.isUserInputEnabled = false
    //预加载
    this.offscreenPageLimit = 4

    //设置适配器
    adapter = object : FragmentStateAdapter(activity) {
        override fun createFragment(position: Int): Fragment {
            return when (position) {
                0 -> {
                    //首页
                    HomeFragment()
                }
                1 -> {
                    //广场
                    VRFragment()
                }
                2 -> {
                    //个人中心
                    MyFragment()
                }3 -> {
                    GoogleMapFragment()
                }

                else -> {
                    //首页
                    HomeFragment()
                }

            }
        }

        override fun getItemCount() = 4
    }
    if(UserManager.INSTANCE.userIsLogin()&&UserManager.INSTANCE.getVipFlag()){
        this.setCurrentItem(3, false)

    }else{
        this.setCurrentItem(0, false)
    }
    return this
}

/**
 * 拦截BottomNavigation长按事件 防止长按时出现Toast ---- 追求完美的大屌群友提的bug
 * @receiver BottomNavigationViewEx
 * @param ids IntArray
 */
private fun BottomNavigationViewEx.interceptLongClick(vararg ids: Int) {
    val bottomNavigationMenuView: ViewGroup = (this.getChildAt(0) as ViewGroup)
    for (index in ids.indices) {
        bottomNavigationMenuView.getChildAt(index).findViewById<View>(ids[index])
                .setOnLongClickListener {
                    true
                }
    }
}
