package com.mints.street.main.googlemap

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.webkit.WebChromeClient
import androidx.lifecycle.Observer
import com.baidu.location.BDLocation
import com.baidu.location.BDLocationListener
import com.baidu.location.LocationClient
import com.baidu.location.LocationClientOption
import com.baidu.mapapi.map.MapStatusUpdateFactory
import com.fry.base.base.BaseFragment
import com.github.lzyzsd.jsbridge.CallBackFunction
import com.github.lzyzsd.jsbridge.DefaultHandler
import com.google.gson.Gson
import com.lxj.xpopup.interfaces.OnConfirmListener
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.bean.GPS
import com.mints.street.bean.MutualBean
import com.mints.street.databinding.ActivityGoogleMapBinding
import com.mints.street.main.home.HomeViewModel
import com.mints.street.main.home.SearchMapActivity
import com.mints.street.utils.GPSConverterUtils
import com.mints.street.webview.MintsWebViewActivity
import com.mints.street.widget.ScrollLayout
import me.goldze.mvvmhabit.base.AppManager
import me.goldze.mvvmhabit.utils.CommonDialogUtils
import me.goldze.mvvmhabit.utils.KLog
import me.goldze.mvvmhabit.utils.RxUtils
import me.goldze.mvvmhabit.utils.ToastUtils


/**
 * Created by 冯瑞雨 on 2021/7/29.
 */
class GoogleMapFragment() : BaseFragment<ActivityGoogleMapBinding, HomeViewModel>() {

    var mLocationClient: LocationClient? = null
    private val myListener: MyLocationListener = MyLocationListener()

    private var gps = GPS()

    inner class MyLocationListener : BDLocationListener {
        override fun onReceiveLocation(location: BDLocation?) {
            gps.lat = location?.latitude?:0.0
            gps.lng = location?.longitude?:0.0
        }
    }

    override fun initContentView(
        inflater: LayoutInflater?,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ) = R.layout.activity_google_map

    override fun initVariableId() = BR.viewModel

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return super.onCreateView(inflater, container, savedInstanceState)
    }

    override fun onResume() {
            viewModel.authorized()
        super.onResume()

    }

    override fun initData() {
        super.initData()
        binding.webview.setDefaultHandler(DefaultHandler())
        binding.webview.webChromeClient = WebChromeClient()
        binding.webview.setOnLongClickListener { true }

        binding.webview.setDefaultHandler(object : DefaultHandler() {
            override fun handler(data: String, function: CallBackFunction) {
                CommonDialogUtils.showMessage(AppManager.getAppManager().currentActivity,
                    content = data,
                    confirmListener = OnConfirmListener {

                    })
            }
        })
        //提供给js使用
        binding.webview.registerHandler("tips") { data, _ ->

            ToastUtils.showLong(data)
        }
        //跳转国外google街景
        binding.webview.registerHandler("toVr") { data, _ ->
            KLog.a("js - > android toVr", data)
            var mutualBean: MutualBean? = null
            try {
                mutualBean = Gson().fromJson(data, MutualBean::class.java)
            } catch (e: Exception) {
                KLog.e("实体类转换错误")
            }
            MintsWebViewActivity.startWebView(
                mutualBean?.title,
                MintsWebViewActivity.toGoogleVR(
                    viewModel.authorizedBean.value?.u,
                    viewModel.authorizedBean.value?.gk,
                    viewModel.authorizedBean.value?.base,
                    mutualBean?.lat,
                    mutualBean?.lng,
                    viewModel.authorizedBean.value?.ut
                )
            )
        }
        //跳转国内百度街景
        binding.webview.registerHandler("toBVr") { data, _ ->
            KLog.a("js - > android toVr", data)
            var mutualBean: MutualBean? = null
            try {
                mutualBean = Gson().fromJson(data, MutualBean::class.java)
            } catch (e: Exception) {
                KLog.e("实体类转换错误")
            }
            MintsWebViewActivity.startWebView(
                mutualBean?.title, MintsWebViewActivity.toBaduVR(
                    mutualBean?.d,
                    viewModel.authorizedBean.value?.bk
                )
            )
        }
        //加载完成
        binding.webview.registerHandler("allIsOk") { _, _ ->
            KLog.e("Sfsadfa","allIsOk")
            initZsMap()
        }

        viewModel.getMapBean()
        //定位
        initPosition()

        //使地图缩放级别增大一级
        binding.imageIn.setOnClickListener {
            binding.webview.callHandler("addZoom"
                ,null ) {
            }
        }
        //使地图缩放级别减小一级
        binding.imageOut.setOnClickListener {
            binding.webview.callHandler("reduceZoom"
                ,null ) {
            }
        }

        binding.imageLocationIcon.setOnClickListener {
            initZsMap()
        }
        RxUtils.onMultiClick(binding.stbBgSearch) {
            SearchMapActivity.startSearchMapActivity(this)
        }

    }

    /**
     * 调用JS
     */
    private fun initZsMap() {
        binding.webview.callHandler("initZsMap"
            ,Gson().toJson(GPSConverterUtils.bd09_To_Gcj02(gps.lat,gps.lng)) ) {
        }
    }

    private fun initPosition() {
        //定位
        mLocationClient = LocationClient(context)
        //声明LocationClient类
        mLocationClient?.registerLocationListener(myListener);
        val option = LocationClientOption()
        option.locationMode = LocationClientOption.LocationMode.Hight_Accuracy
        //可选，设置定位模式，默认高精度
        //LocationMode.Hight_Accuracy：高精度；
        //LocationMode. Battery_Saving：低功耗；
        //LocationMode. Device_Sensors：仅使用设备；
        option.setCoorType("bd09ll")
        //可选，设置返回经纬度坐标类型，默认GCJ02
        //GCJ02：国测局坐标；
        //BD09ll：百度经纬度坐标；
        //BD09：百度墨卡托坐标；
        //海外地区定位，无需设置坐标类型，统一返回WGS84类型坐标
        option.setScanSpan(1000)
        //可选，设置发起定位请求的间隔，int类型，单位ms
        //如果设置为0，则代表单次定位，即仅定位一次，默认为0
        //如果设置非0，需设置1000ms以上才有效
        option.isOpenGps = true
        //可选，设置是否使用gps，默认false
        //使用高精度和仅用设备两种定位模式的，参数必须设置为true
        option.isLocationNotify = true
        //可选，设置是否当GPS有效时按照1S/1次频率输出GPS结果，默认false
        option.setIgnoreKillProcess(false)
        //可选，定位SDK内部是一个service，并放到了独立进程。
        //设置是否在stop的时候杀死这个进程，默认（建议）不杀死，即setIgnoreKillProcess(true)
        option.SetIgnoreCacheException(false)
        //可选，设置是否收集Crash信息，默认收集，即参数为false
        option.setWifiCacheTimeOut(5 * 60 * 1000)
        //可选，V7.2版本新增能力
        //如果设置了该接口，首次启动定位时，会先判断当前Wi-Fi是否超出有效期，若超出有效期，会先重新扫描Wi-Fi，然后定位
        option.setEnableSimulateGps(false)
        //可选，设置是否需要过滤GPS仿真结果，默认需要，即参数为false
        option.setNeedNewVersionRgc(true)
        //可选，设置是否需要最新版本的地址信息。默认需要，即参数为true
        mLocationClient?.locOption = option
        //mLocationClient为第二步初始化过的LocationClient对象
        //需将配置好的LocationClientOption对象，通过setLocOption方法传递给LocationClient对象使用
        //更多LocationClientOption的配置，请参照类参考中LocationClientOption类的详细说明
        mLocationClient?.start();
        //mLocationClient为第二步初始化过的LocationClient对象
        //调用LocationClient的start()方法，便可发起定位请求
    }

    override fun initViewObservable() {
        super.initViewObservable()
        viewModel.authorizedBean.observe(this, Observer {
            it?.apply {
                binding.webview.loadUrl("file:///android_asset/web/zsmap.html?u=${u
                }&gk=${gk}&bk=${bk}&base=${base}&ut=${ut}")
                KLog.e("file:///android_asset/w")
            }
        })
        viewModel.pt.observe(this, Observer {
            binding.webview.callHandler("initZsMap"
                ,Gson().toJson(it) ) {
            }
            binding.scrollLayout.toggle(ScrollLayout.STATUS_CLOSE)
        })
    }
    override fun onDestroy() {
        super.onDestroy()

        binding.webview.let {
            it.destroyDrawingCache()
            val parent = it.parent as ViewGroup
            parent.removeView(it)
            it.removeAllViews()
            it.destroy()
        }
    }

}

