package com.mints.street.main.home

import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.View.OnTouchListener
import android.view.ViewTreeObserver.OnGlobalLayoutListener
import android.view.animation.DecelerateInterpolator
import android.widget.AbsListView
import androidx.databinding.ViewDataBinding
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.fry.base.base.BaseActivity
import com.fry.base.recycler_view.VHMAdapter
import com.fry.base.recycler_view.VHModel
import com.fry.base.utils.DensityUtils
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.databinding.ActivityHistoricalImageBinding
import me.goldze.mvvmhabit.utils.KLog
import java.util.*

/**
 * 描述：历史影像
 * 作者：孟崔广
 * 时间：2021/7/12 17:51
 */
class HistoricalimageActivity :
    BaseActivity<ActivityHistoricalImageBinding, HistoricalimageModel>() {

    private var centerToLiftDistance = 0//RecyclerView款度的一半 ,也就是控件中间位置到左部的距离 ，

    private var childViewHalfCount = 0 //当前RecyclerView一半最多可以存在几个Item


    override fun initVariableId() = BR.viewModel
    override fun initContentView(savedInstanceState: Bundle?) = R.layout.activity_historical_image
    override fun initData() {
        super.initData()
        init()
    }

    private fun init() {
        binding.recyclerView.layoutManager = LinearLayoutManager(
            this,
            LinearLayoutManager.HORIZONTAL,
            false
        )
        binding.recyclerView.viewTreeObserver
            .addOnGlobalLayoutListener(object : OnGlobalLayoutListener {
                override fun onGlobalLayout() {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                        binding.recyclerView.viewTreeObserver.removeOnGlobalLayoutListener(this)
                    }
                    centerToLiftDistance = binding.recyclerView.width / 2
                    val childViewHeight: Int = DensityUtils.dip2px(
                        100F
                    ) //是当前已知的 Item的高度
                    childViewHalfCount = (binding.recyclerView.width / childViewHeight +1 ) / 2
                    initRcData()
                    findView()
                }
            })
        binding.recyclerView.postDelayed(Runnable { scrollToCenter(childViewHalfCount) }, 100L)
    }

    private var mDatas: MutableList<String?> = ArrayList<String?>()
    private fun initRcData() {
        if (mDatas == null) mDatas = ArrayList<String?>()
        for (i in 0..9) {
            mDatas?.add("条目$i")
        }
        for (j in 0 until childViewHalfCount) { //头部的空布局
            mDatas?.add(0, null)
        }
        for (k in 0 until childViewHalfCount) {  //尾部的空布局
            mDatas?.add(null)
        }
    }

    private var isTouch = false
    private val centerViewItems: MutableList<CenterViewItem> = ArrayList()
    var mAdapter: VHMAdapter<HistorytemVHM>? = null
    private fun findView() {
        mAdapter = VHMAdapter<HistorytemVHM>()
        mDatas?.forEach {
            mAdapter?.add(HistorytemVHM(it))
        }
        binding.recyclerView.adapter = mAdapter
        binding.recyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
                super.onScrollStateChanged(recyclerView, newState)
                if (newState == AbsListView.OnScrollListener.SCROLL_STATE_IDLE) {
                    val linearLayoutManager = recyclerView.layoutManager as LinearLayoutManager?
                    val fi = linearLayoutManager!!.findFirstVisibleItemPosition()
                    val la = linearLayoutManager.findLastVisibleItemPosition()

                    if (isTouch) {
                        isTouch = false
                        //获取最中间的Item View
                        val centerPositionDiffer = (la - fi) / 2
                        var centerChildViewPosition = fi + centerPositionDiffer //获取当前所有条目中中间的一个条目索引
                        centerViewItems.clear()
                        //遍历循环，获取到和中线相差最小的条目索引(精准查找最居中的条目)
                        if (centerChildViewPosition != 0) {
                            for (i in centerChildViewPosition - 1 until centerChildViewPosition + 2) {
                                val cView = recyclerView.layoutManager!!.findViewByPosition(i)
                                val viewLeft = cView!!.left + cView.width / 2
                                centerViewItems.add(
                                    CenterViewItem(
                                        i,
                                        Math.abs(centerToLiftDistance - viewLeft)
                                    )
                                )
                            }
                            val centerViewItem: CenterViewItem? =
                                getMinDifferItem(centerViewItems)
                            centerChildViewPosition = centerViewItem?.position ?: 0
                        }
                        scrollToCenter(centerChildViewPosition)
                    }
                }
            }

            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                for (i in 0 until recyclerView.childCount) {
                    recyclerView.getChildAt(i).invalidate()
                }
            }
        })
        binding.recyclerView.setOnTouchListener(OnTouchListener { _, _ ->
            isTouch = true
            false
        })
    }

    private val decelerateInterpolator = DecelerateInterpolator()

    /**
     * 移动指定索引到中心处 ， 只可以移动可见区域的内容
     * @param movePosition
     */
    private fun scrollToCenter(movePosition: Int) {
        var position =0
        if(movePosition >= mDatas.size - childViewHalfCount ){
            position =mDatas.size - childViewHalfCount -1
        }else{
            position = movePosition
        }
        position = if (position < childViewHalfCount) childViewHalfCount else position
        position =
            if (position < mAdapter?.itemCount ?: 0 - childViewHalfCount - 1) position else mAdapter?.itemCount
                ?: 0 - childViewHalfCount - 1
        val linearLayoutManager = binding.recyclerView.layoutManager as LinearLayoutManager
        val childView = linearLayoutManager.findViewByPosition(position)
        Log.i("ccb", "滑动后中间View的索引: $position")
        //把当前View移动到居中位置
        if (childView == null) return
        val childVhalf = childView.width / 2
        val childViewLeft = childView.left
        val viewCTop = centerToLiftDistance
        val smoothDistance = childViewLeft - viewCTop + childVhalf
        Log.i(
            "ccb", """
     
     居中位置距离左部距离: $viewCTop
     当前居中控件距离左部距离: $childViewLeft
     当前居中控件的一半高度: $childVhalf
     滑动后再次移动距离: $smoothDistance
     """.trimIndent()
        )
        binding.recyclerView.smoothScrollBy(smoothDistance, 0, decelerateInterpolator)
        //mAdapter?.setSelectPosition(position)
        //tv.setText("当前选中:" + mDatas!![position])
    }

    /**
     * 计算距离中间最近的一个ItemView
     * @param itemHeights
     * @return
     */
    fun getMinDifferItem(itemHeights: List<CenterViewItem>): CenterViewItem? {
        var minItem = itemHeights[0] //默认第一个是最小差值
        for (i in itemHeights.indices) {
            //遍历获取最小差值
            if (itemHeights[i].differ <= minItem.differ) {
                minItem = itemHeights[i]
            }
        }
        return minItem
    }
}


class HistorytemVHM(
    var string: String?
) : VHModel() {
    override fun getLayoutId() = R.layout.item_history

    override fun getVariableId() = BR.vhm

    override fun onBind(binding: ViewDataBinding) {
        super.onBind(binding)

    }
}


class CenterViewItem     //当前Item索引
//当前item和居中位置的差值
    (var position: Int, var differ: Int)

