package com.mints.street.main.home

import android.app.Dialog
import android.content.Context.SENSOR_SERVICE
import android.hardware.Sensor
import android.hardware.SensorEvent
import android.hardware.SensorEventListener
import android.hardware.SensorManager
import android.os.Bundle
import android.os.Handler
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.Observer
import com.baidu.location.BDLocation
import com.baidu.location.BDLocationListener
import com.baidu.location.LocationClient
import com.baidu.location.LocationClientOption
import com.baidu.mapapi.map.*
import com.baidu.mapapi.model.LatLng
import com.fry.base.base.BaseFragment
import com.google.gson.Gson
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.bean.MapBean
import com.mints.street.bean.PositioningBean
import com.mints.street.databinding.FragmentHomeBinding
import com.mints.street.manager.UmengManager
import com.mints.street.utils.GPSConverterUtils
import com.mints.street.widget.ScrollLayout
import com.mints.street.widget.dialog.DialogListener
import com.mints.street.widget.dialog.DialogUtils
import me.goldze.mvvmhabit.base.AppManager
import me.goldze.mvvmhabit.utils.KLog
import me.goldze.mvvmhabit.utils.RxUtils
import me.goldze.mvvmhabit.utils.SPUtils

/**
 * Created by 冯瑞雨 on 2021/7/2.
 * 首页页面
 */
class HomeFragment : BaseFragment<FragmentHomeBinding, HomeViewModel>(), SensorEventListener {

    companion object {
        val IS_FIRST_OPEN: String = "isFirstOpen"
    }

    private var mBaiduMap: BaiduMap? = null

    // 定位相关
    private var mLocClient: LocationClient? = null

    private var isScrollLayoutClose: Boolean = false

    // 定位图层显示方式
    private var mCurrentMode: MyLocationConfiguration.LocationMode? = null
    private var mSensorManager: SensorManager? = null
    private var lastX = 0.0

    //// 此处设置开发者获取到的方向信息，顺时针0-360
    private var mCurrentDirection = 0

    //纬度
    private var mCurrentLat = 0.0

    //经度
    private var mCurrentLon = 0.00

    //当前的经纬度
    //纬度
    private var currentLat = 0.0

    //经度
    private var currentLon = 0.0


    //设置定位数据的精度信息，单位：米
    private var mCurrentAccracy = 0f

    // 是否首次定位
    private var isFirstLoc = true
    private var myLocationData: MyLocationData? = null


    override fun initContentView(
            inflater: LayoutInflater?,
            container: ViewGroup?,
            savedInstanceState: Bundle?
    ) = R.layout.fragment_home

    override fun initVariableId() = BR.viewModel

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        initMap()
        // 打开激活
        uploadDeviceInfo()
    }

    private fun uploadDeviceInfo() {

        UmengManager.initUm()
        viewModel.comSaveTerminalInfo()
    }

    override fun initData() {
        super.initData()
        Handler().postDelayed({ isScrollLayoutClose = true }, 2000)
        RxUtils.onMultiClick(binding.stbBgSearch) {
            SearchMapActivity.startSearchMapActivity(this)
        }
        RxUtils.onMultiClick(binding.ivClick) {
            viewModel.authorized("", currentLat.toString(), currentLon.toString(), false)
        }
        viewModel.getMapBean()
    }

    override fun initViewObservable() {
        viewModel.pt.observe(this, Observer {
            if (it == null) return@Observer
            // 默认 天安门
            val gps = GPSConverterUtils.gcj02_To_Bd09(it.lat, it.lng)
            val center = LatLng(gps.lat, gps.lng)
            setNewMapStatus(center)
            binding.scrollLayout.toggle(ScrollLayout.STATUS_CLOSE)
        })

        viewModel.mapbeanData.observe(this, Observer<MapBean> {
            DialogUtils.showbennfitsDialog(context!!, viewModel, it.innerPlaces?.places)
        })
    }

    private fun initMap() {
        mBaiduMap = binding.bmapView.map
        mBaiduMap?.mapType = BaiduMap.MAP_TYPE_SATELLITE

        //设置是否显示缩放控件
        binding.bmapView.showZoomControls(false)
        // 构建地图状态
        val builder = MapStatus.Builder()
        // 默认 天安门
        val center = LatLng(39.915071, 116.403907)
        // 默认 11级
        val zoom = 11.0f
        builder.target(center).zoom(zoom)
        val mapStatusUpdate = MapStatusUpdateFactory.newMapStatus(builder.build())

        // 设置地图状态
        mBaiduMap?.setMapStatus(mapStatusUpdate)
        mBaiduMap?.setOnMapStatusChangeListener(object : BaiduMap.OnMapStatusChangeListener {
            override fun onMapStatusChangeStart(status: MapStatus?) {
                KLog.a("百度map", "移动开始")
            }

            override fun onMapStatusChangeStart(status: MapStatus?, reason: Int) {
                KLog.a("百度map", "移动开始，两个参数")

            }

            override fun onMapStatusChange(status: MapStatus?) {
                KLog.a("百度map", "移动改变")
                if (!binding.scrollLayout.isClose && isScrollLayoutClose) {
                    binding.scrollLayout.post {
                        binding.scrollLayout.toggle(ScrollLayout.STATUS_CLOSE)
                    }
                }
            }

            override fun onMapStatusChangeFinish(status: MapStatus?) {
                KLog.a("百度map", "移动结束")
                currentLat = status?.target?.latitude ?: 0.0
                currentLon = status?.target?.longitude ?: 0.0
            }
        })
        //使地图缩放级别增大一级
        binding.imageIn.setOnClickListener {
            mBaiduMap?.setMapStatus(MapStatusUpdateFactory.zoomIn())
        }
        //使地图缩放级别减小一级
        binding.imageOut.setOnClickListener {
            mBaiduMap?.setMapStatus(MapStatusUpdateFactory.zoomOut())
        }
        //历史影像
        binding.imageMapHistory.setOnClickListener {
            startActivity(HistoricalimageActivity::class.java)
        }
        //重新回到定位点
        RxUtils.onMultiClick(binding.imageLocationIcon) {
            // 默认 天安门
            val center = LatLng(mCurrentLat, mCurrentLon)
            // 默认 11级
            val zoom = 17.0f
            builder.target(center).zoom(zoom)
            val mapStatusUpdate = MapStatusUpdateFactory.newMapStatus(builder.build())

            // 设置地图状态
            mBaiduMap?.setMapStatus(mapStatusUpdate)
        }
        //定位初始化
        initLocation()
    }

    /**
     * 定位初始化
     */
    private fun initLocation() {
        // 获取传感器管理服务
        mSensorManager = AppManager.getAppManager().currentActivity
                .getSystemService(SENSOR_SERVICE) as SensorManager
        mCurrentMode = MyLocationConfiguration.LocationMode.NORMAL
        // 为系统的方向传感器注册监听器
        mSensorManager?.registerListener(
                this,
                mSensorManager?.getDefaultSensor(Sensor.TYPE_ORIENTATION),
                SensorManager.SENSOR_DELAY_UI
        )

        // 开启定位图层
        mBaiduMap?.isMyLocationEnabled = true
        // 定位初始化
        mLocClient = LocationClient(context)
        mLocClient?.registerLocationListener(MyLocationListener())
        val option = LocationClientOption()
        // 打开gps
        option.isOpenGps = true
        // 设置坐标类型
        option.setCoorType("bd09ll")
        //可选，默认0，即仅定位一次，设置发起连续定位请求的间隔需要大于等于1000ms才是有效的
        option.setScanSpan(100000)//十秒一次
        option.setIsNeedAddress(true) // 可选，设置是否需要地址信息，默认不需要
        option.setIsNeedLocationDescribe(true) // 可选，设置是否需要地址描述

        mLocClient?.locOption = option
        mLocClient?.start()
    }


    override fun onPause() {
        super.onPause()
        binding.bmapView.onPause()
    }

    override fun onResume() {
        super.onResume()
        binding.bmapView.onResume()
    }

    override fun onDestroy() {
        super.onDestroy()
        // 取消注册传感器监听
        mSensorManager?.unregisterListener(this)
        // 退出时销毁定位
        mLocClient?.stop()
        // 关闭定位图层
        mBaiduMap?.isMyLocationEnabled = false
        binding.bmapView.onDestroy()

    }


    /**
     * 定位SDK监听函数
     */
    var sp: SPUtils? = null
    var positioningBean: PositioningBean? = null

    inner class MyLocationListener : BDLocationListener {
        override fun onReceiveLocation(location: BDLocation) {
            if (sp == null) {
                sp = SPUtils.getInstance(SPUtils.POSITIONING_MAP)
            }
            if (positioningBean == null) {
                positioningBean = PositioningBean()
            }
            positioningBean?.latitude = location.latitude
            positioningBean?.longitude = location.longitude
            positioningBean?.countryCode = location.countryCode
            positioningBean?.country = location.country
            positioningBean?.province = location.province
            positioningBean?.city = location.city
            positioningBean?.district = location.district
            positioningBean?.town = location.town
            positioningBean?.street = location.street
            positioningBean?.addrStr = location.addrStr

            sp?.encode(SPUtils.POSITIONING_MAP, Gson().toJson(positioningBean).toString())
            // MapView 销毁后不在处理新接收的位置
            mBaiduMap?.apply {
                mCurrentLat = location.latitude
                mCurrentLon = location.longitude
                mCurrentAccracy = location.radius
                myLocationData = MyLocationData.Builder()
                        .accuracy(location.radius) // 设置定位数据的精度信息，单位：米
                        .direction(mCurrentDirection.toFloat()) // 此处设置开发者获取到的方向信息，顺时针0-360
                        .latitude(location.latitude)
                        .longitude(location.longitude)
                        .build()
                this.setMyLocationData(myLocationData)
                if (isFirstLoc) {
                    isFirstLoc = false
                    val ll = LatLng(location.latitude, location.longitude)
                    val builder = MapStatus.Builder()
                    builder.target(ll).zoom(18.0f)
                    this.animateMapStatus(MapStatusUpdateFactory.newMapStatus(builder.build()))
                }
            }

        }
    }

    override fun onSensorChanged(sensorEvent: SensorEvent?) {
        val x: Double =
                sensorEvent?.values?.get(SensorManager.DATA_X)?.toDouble() ?: 0.0
        if (kotlin.math.abs(x - lastX) > 1.0) {
            mCurrentDirection = x.toInt()
            myLocationData = MyLocationData.Builder()
                    .accuracy(mCurrentAccracy) // 设置定位数据的精度信息，单位：米
                    .direction(mCurrentDirection.toFloat()) // 此处设置开发者获取到的方向信息，顺时针0-360
                    .latitude(mCurrentLat)
                    .longitude(mCurrentLon)
                    .build()
            mBaiduMap?.setMyLocationData(myLocationData)
        }
        lastX = x
    }

    override fun onAccuracyChanged(sensor: Sensor?, accuracy: Int) {

    }

    /**
     * 设置地图新状态
     *
     * @param v
     */
    private fun setNewMapStatus(latLng: LatLng) {
        val builder = MapStatus.Builder()
        builder.target(latLng).zoom(18.5f).overlook(-21f).rotate(0f)
        // 更新地图状态
        mBaiduMap!!.setMapStatus(MapStatusUpdateFactory.newMapStatus(builder.build()))
        //mMarker.setPosition(latLng)
    }
}