package com.mints.street.main.home

import android.app.Application
import android.content.Intent
import androidx.databinding.ObservableArrayList
import androidx.lifecycle.MutableLiveData
import com.google.gson.Gson
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.bean.*
import com.mints.street.common.DeviceInfo
import com.mints.street.db.ScenicSpotBean
import com.mints.street.main.my.OpenvipActivity
import com.mints.street.manager.UserManager
import com.mints.street.manager.oaid.OaidManager
import com.mints.street.model.ApiModel
import com.mints.street.netwrok.base.HttpSubscribeImpl
import com.mints.street.utils.DeviceUuidFactory
import com.mints.street.webview.MintsWebViewActivity
import me.goldze.mvvmhabit.base.AppManager
import me.goldze.mvvmhabit.base.BaseViewModel
import me.goldze.mvvmhabit.utils.KLog
import me.goldze.mvvmhabit.utils.SPUtils
import me.tatarka.bindingcollectionadapter2.ItemBinding
import java.io.BufferedReader
import java.io.InputStreamReader

/**
 * Created by 冯瑞雨 on 2021/7/2.
 */
class HomeViewModel(application: Application) : BaseViewModel(application) {


    var addressList = ObservableArrayList<HomeButtonItemVM?>()
    var addressItemBinding = ItemBinding.of<HomeButtonItemVM>(BR.viewModel, R.layout.item_home_button)

    var pt = MutableLiveData<GPS>()
    val authorizedBean = MutableLiveData<AuthorizedBean>()


    fun authorized() {
        ApiModel.authorized().safeSubscribe(object
            : HttpSubscribeImpl<BaseResponse<AuthorizedBean>>(
                null, false
        ) {
            override fun onBusinessSuccess(response: BaseResponse<AuthorizedBean>) {
                if (Gson().toJson(response.result).equals(Gson().toJson(authorizedBean.value))) {
                    return
                }
                authorizedBean.value = response.result
            }

        })
    }

    fun authorized(name: String? = null, lat: String?, lng: String?, isLook: Boolean) {
        if (!isLook) {
            if ((!UserManager.INSTANCE.userIsLogin() && !UserManager.INSTANCE.getVipFlag())) {
                startActivity(OpenvipActivity::class.java)
                return
            }
        }

        ApiModel.authorized().safeSubscribe(object
            : HttpSubscribeImpl<BaseResponse<AuthorizedBean>>(
                this@HomeViewModel, false
        ) {
            override fun onBusinessSuccess(response: BaseResponse<AuthorizedBean>) {
                if (!Gson().toJson(response.result).equals(Gson().toJson(authorizedBean.value))) {
                    authorizedBean.value = response.result
                }
                MintsWebViewActivity.startWebView(name,
                        MintsWebViewActivity.toBaduVR(lat, lng, response.result?.bk))

            }

        })
    }


    var mapbeanData = MutableLiveData<MapBean>()
    fun getMapBean() {
        val assets = AppManager.getAppManager().currentActivity.assets
        val stringBuilder = StringBuilder()
        var mapBean: MapBean? = null
        try {
            //通过管理器打开文件并读取
            val bf = BufferedReader(
                    InputStreamReader(
                            assets.open("map.json"))
            )
            var line: String?
            while (bf.readLine().also { line = it } != null) {
                stringBuilder.append(line)
            }

            if ( SPUtils.getInstance().decodeBoolean(HomeFragment.IS_FIRST_OPEN,true)){
                mapbeanData.value = Gson().fromJson(stringBuilder.toString(), MapBean::class.java)
                SPUtils.getInstance().encode(HomeFragment.IS_FIRST_OPEN,false)
            }
            mapBean = Gson().fromJson(stringBuilder.toString(), MapBean::class.java)

        } catch (e: Exception) {
            KLog.e("读取assets文件下的json出问题")
        }
        val bean = mapBean?.homePlaces?.places
        bean?.iterator()?.forEach { item ->
            if (item.images == null || item.images?.size!! < 3) {
                return
            }
            addressList.add(HomeButtonItemVM(this, item.name
                    , item.images!![0], item.images!![1], item.images!![2]
                    , latitude = item.latitude.toString(), longitude = item.longitude.toString(), isLook = true
            ))

        }
    }

    /**
     * 打开激活
     */
    fun comSaveTerminalInfo() {
        val vo = HashMap<String, Any>()
        val deviceInfo = DeviceInfo.instance
        val macAddress: String = deviceInfo.getMacAddress()
        val mac = macAddress.replace(":", "")
        vo["mac"] = mac
        vo["mac1"] = macAddress
        vo["androidid"] = deviceInfo.getAndroidId(null)
        vo["imei"] = deviceInfo.iMEI
        vo["oaid"] = OaidManager.getOaid()
        vo["os"] = "android"
        vo["model"] = deviceInfo.newModel
        vo["uuid"] = DeviceUuidFactory().deviceUuid
        vo["osversion"] = deviceInfo.oSVersion
        vo["appversion"] = deviceInfo.versionName

        ApiModel.comSaveTerminalInfo(lifecycleProvider, vo).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<Any>>(
                        this@HomeViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<Any>) {
                    }

                    override fun onError(e: Throwable) {
                    }
                })
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == 200 && resultCode == SearchMapActivity.REQUEST_CODE) {
            data?.let {
                pt.value = GPS(it.getDoubleExtra("latitude", 0.0), it.getDoubleExtra("longitude", 0.0))
            }
        }
    }
}