package com.mints.street.main.home

import android.content.Intent
import android.os.Bundle
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import androidx.fragment.app.Fragment
import androidx.lifecycle.Observer
import com.alibaba.android.vlayout.DelegateAdapter
import com.alibaba.android.vlayout.VirtualLayoutManager
import com.fry.base.base.BaseActivity
import com.fry.base.utils.ResourceUtils
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.adapter.HistoryRecordAdapter
import com.mints.street.adapter.PopularSceneAdapter
import com.mints.street.bean.HistoryBean
import com.mints.street.bean.MapBean
import com.mints.street.bean.PositionBean
import com.mints.street.databinding.ActivitySearchMapBinding

/**
 * Created by 冯瑞雨 on 2021/7/13.
 */
class SearchMapActivity : BaseActivity<ActivitySearchMapBinding, SearchMapViewModel>() {
    companion object {
        const val HISTORY_NAME = "history_name"
        const val REQUEST_CODE = 201
        fun startSearchMapActivity(fragment: Fragment) {
            fragment.startActivityForResult(
                Intent(fragment.context, SearchMapActivity::class.java),
                200
            )
        }
    }

    private var vLayout: VirtualLayoutManager? = null
    private var mAdapter: DelegateAdapter? = null

    private var historyRecordAdapter: HistoryRecordAdapter? = null
    private var popularSceneAdapter: PopularSceneAdapter? = null


    override fun initContentView(savedInstanceState: Bundle?) = R.layout.activity_search_map

    override fun initVariableId() = BR.viewModel


    override fun initData() {
        super.initData()
        //初始化布局管理器
        vLayout = VirtualLayoutManager(this)
        mAdapter = DelegateAdapter(vLayout, false)

        binding.recyList.layoutManager = vLayout
        binding.recyList.adapter = mAdapter

        binding.et.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
            }

            override fun afterTextChanged(s: Editable?) {
                if (TextUtils.isEmpty(s)) {
                    viewModel.searchAfterOneItems.clear()
                    return
                }
                if (viewModel.index.value == 0) {
                    //国外
                    viewModel.searchdomestic(s.toString())
                } else {
                    //传入参数【这里是可以获取到参数的】【国外】
                    viewModel.searchabroad(s.toString())
                }

            }

        })


        //历史记录
        historyRecordAdapter = HistoryRecordAdapter(this, viewModel)

        //热门景点
        popularSceneAdapter = PopularSceneAdapter(this,viewMode = viewModel)

        mAdapter?.addAdapter(historyRecordAdapter)
        mAdapter?.addAdapter(popularSceneAdapter)

        //境内
        binding.territory.setOnClickListener {
            territory()
        }
        //境外
        binding.abroad.setOnClickListener {
            abroad()
        }

        viewModel.getMapBean()

    }


    private fun update() {
        historyRecordAdapter?.apply {
            this.type = viewModel.index.value ?: 0
        }
        popularSceneAdapter?.apply {
            if (viewModel.index.value ?: 0 == 0) {
                this.data = viewModel.mapBean.value?.innerPlaces?.places
            } else {
                this.data = viewModel.mapBean.value?.outerPlaces?.places
            }
        }
    }

    override fun initViewObservable() {
        viewModel.mapBean.observe(this, Observer<MapBean> {
            it?.let {
                update()
            }
        })
        viewModel.searchName.observe(this, Observer<HistoryBean> {
            if (TextUtils.isEmpty(it.name)) {
                return@Observer
            }
            historyRecordAdapter?.addHistoryData(it)
        })
        viewModel.positiondata.observe(this@SearchMapActivity, Observer {
            //srl_my.finishRefresh(true)
            it?.apply {
                onGetSuggestionResult(this)
            }
        })
        viewModel.index.observe(this, Observer {
            if(TextUtils.isEmpty(binding.et.text)){
                return@Observer
            }
            if (it == 0) {
                //国外
                viewModel.searchdomestic(binding.et.text.toString())
            } else {
                //传入参数【这里是可以获取到参数的】【国外】
                viewModel.searchabroad(binding.et.text.toString())
            }
        })
    }

    private fun territory() {
        if (viewModel.index.value ?: 0 == 0) return
        viewModel.index.value = 0
        binding.territory.setTextColor(ResourceUtils.getColor(R.color.color_668BFF))
        binding.abroad.setTextColor(ResourceUtils.getColor(R.color.white))
        binding.territory.solid = ResourceUtils.getColor(R.color.white)
        binding.abroad.solid = ResourceUtils.getColor(R.color.color_668BFF)
        update()
    }

    private fun abroad() {
        if (viewModel.index.value ?: 0 == 1) return
        viewModel.index.value = 1
        binding.territory.setTextColor(ResourceUtils.getColor(R.color.white))
        binding.abroad.setTextColor(ResourceUtils.getColor(R.color.color_668BFF))
        binding.territory.solid = ResourceUtils.getColor(R.color.color_668BFF)
        binding.abroad.solid = ResourceUtils.getColor(R.color.white)
        update()
    }

    fun onGetSuggestionResult(postionConfig: List<PositionBean>?) {
        if (postionConfig == null) {
            return
        }
        viewModel.searchAfterOneItems.clear()

        if (TextUtils.isEmpty(binding.et.text)) {
            return
        }
        for (info in postionConfig!!) {
            viewModel.searchAfterOneItems.add(
                SearchAfterItem(
                    viewModel, info.name, info.address, info.location
                )
            )
        }
    }

}