package com.mints.street.main.my

import android.Manifest
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import androidx.lifecycle.Observer
import com.fry.base.base.BaseActivity
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.bean.UserBean
import com.mints.street.databinding.ActivityBindmobileBinding
import com.mints.street.main.MainActivity
import com.mints.street.utils.BackInputUtil
import com.mints.street.widget.dialog.DialogUtils
import com.tbruyelle.rxpermissions2.RxPermissions
import kotlinx.android.synthetic.main.activity_bindmobile.*
import kotlinx.android.synthetic.main.include_header.*
import me.goldze.mvvmhabit.utils.ToastUtils
import me.goldze.mvvmhabit.utils.systembar.StatusBarUtil

class BindMobileActivity : BaseActivity<ActivityBindmobileBinding, BindMobileViewModel>(), View.OnClickListener {

    private var mobile: String? = null

    override fun getStatusBarHeightView(): View? {
        return null
    }

    override fun initVariableId() = BR.viewModel
    override fun initContentView(savedInstanceState: Bundle?) = R.layout.activity_bindmobile
    override fun initData() {
        super.initData()
        StatusBarUtil.setHeightAndPadding(this, binding.ivApplogo)
        initView()
        initListener()
    }

    private fun initListener() {
        iv_left_icon.setOnClickListener(this)
        tvLoginSendcode.setOnClickListener(this)
        tvLoginNext.setOnClickListener(this)
    }

    private fun initView() {
        tv_title.text = "绑定手机号"
        iv_left_icon.visibility = View.VISIBLE
        iv_left_icon.setImageResource(R.mipmap.ic_arrow_back)

        BackInputUtil.phoneNumAddSpace(etLoginMobile)
//        mobile = UserManager.getInstance().mobile
        if (!TextUtils.isEmpty(mobile)) {
            etLoginMobile.setText(mobile)
            etLoginMobile.setSelection(mobile!!.length + 2)
        }
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.iv_left_icon -> {
                finish()
            }
            R.id.tvLoginSendcode -> {
                var mobile = etLoginMobile.text.toString().trim()
                if (mobile.length < 13) {
                    ToastUtils.showShortSafe("请输入手机号")
                    return
                }
                sendCodeThread()

                mobile = mobile.replace(" ".toRegex(), "")
//                loginPresenter.sendMobileBindingCode(mobile)
                viewModel.bindingMobilecode(mobile)
            }
            R.id.tvLoginNext -> {
                var mobile = etLoginMobile.text.toString().trim()
                if (mobile.length < 13) {
                    ToastUtils.showShortSafe("请输入手机号")
                    return
                }

                val code = etLoginCode.text.toString().trim()
                if (code.length < 4) {
                    ToastUtils.showShortSafe("请输入验证码")
                    return
                }

                RxPermissions(this@BindMobileActivity)
                        .request(Manifest.permission.READ_PHONE_STATE)
                        .subscribe { granted: Boolean ->
                            if (granted) {
                                mobile = mobile.replace(" ".toRegex(), "")
                                viewModel.bindingMobile(mobile, code)
                            } else {
                                DialogUtils.showPermissionDialog(this@BindMobileActivity,"设备")
                            }
                        }
            }
        }
    }

    override fun initViewObservable() {
        super.initViewObservable()
        viewModel.bindmobiledata.observe(this, Observer<UserBean> {
            ToastUtils.showLong("手机号绑定成功")
            startActivityThenKill(MainActivity::class.java)
        })
        viewModel.data.observe(this, Observer {

        })
    }

    /**
     * 发送手机验证码，按钮动态改变状态
     */
    var num = 0
    var run: Runnable? = null
    fun sendCodeThread() {
        num = 60
        tvLoginSendcode?.let {
            it.isEnabled = false
            it.text = "（$num）重新获取"

            run = object : Runnable {
                override fun run() {
                    num--
                    if (num == 0) {
                        it.text = "重新获取"
                        it.isEnabled = true
                    } else {
                        it.text = "（$num）重新获取"
                        it.postDelayed(this, 1000)
                    }
                }
            }
            //关闭线程
            it.postDelayed(run, 1000)
        }
    }
}
