package com.mints.street.main.my

import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.fry.base.base.BaseActivity
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.common.Constant
import com.mints.street.databinding.ActivityMoresettingsBinding
import com.mints.street.manager.UserManager
import com.mints.street.webview.MintsWebViewActivity
import kotlinx.android.synthetic.main.activity_moresettings.*
import kotlinx.android.synthetic.main.include_header.*
import me.goldze.mvvmhabit.utils.KLog
import me.goldze.mvvmhabit.utils.RxUtils
import me.goldze.mvvmhabit.utils.ToastUtils
import me.goldze.mvvmhabit.utils.systembar.StatusBarUtil
import org.w3c.dom.Text

class MoresettingsActivity : BaseActivity<ActivityMoresettingsBinding, MoresettingsViewModel>(), View.OnClickListener {

    private val userManager by lazy { UserManager.INSTANCE }

    override fun initVariableId() = BR.viewModel
    override fun initContentView(savedInstanceState: Bundle?) = R.layout.activity_moresettings
    override fun getStatusBarHeightView(): View? {
        return binding.elView
    }

    override fun initData() {
        super.initData()
        initView()
        initListener()

    }

    private fun initListener() {

        iv_left_icon.setOnClickListener(this)
        item_userAgree.setOnClickListener(this)
        item_privacyAgree.setOnClickListener(this)
        btn_switch.setOnClickListener(this)

    }

    override fun onResume() {
        super.onResume()
        //判断当前手机号是否绑定
        if (!userManager.getMobile().isEmpty()) {
            item_phone.findViewById<TextView>(R.id.tv_right).visibility = View.VISIBLE
            item_phone.findViewById<TextView>(R.id.tv_right).text = userManager.getMobile()
            item_phone.findViewById<TextView>(R.id.tv_right).setTextColor(Color.BLACK)
        } else {
            item_phone.findViewById<TextView>(R.id.tv_right).visibility = View.VISIBLE
            item_phone.findViewById<TextView>(R.id.tv_right).setTextColor(Color.BLACK)
            item_phone.findViewById<TextView>(R.id.tv_right).text = "去绑定"
            item_phone.setOnClickListener(this)
            RxUtils.onMultiClick(item_phone) {
                if (userManager.userIsLogin()) {
                    startActivity(BindMobileActivity::class.java)
                    return@onMultiClick
                } else {
                    ToastUtils.showLong("请先使用微信登录")
                }
            }
        }
    }

    private fun initView() {

        tv_title.text = "更多设置"
        tv_title.setTextColor(Color.BLACK)
        iv_left_icon.setImageResource(R.mipmap.ic_arrow_back)
        iv_left_icon.visibility = View.VISIBLE

        item_phone.findViewById<TextView>(R.id.tv_title).text = "绑定手机号"
        val phone = ContextCompat.getDrawable(this, R.mipmap.icon_settings_tel)
        phone?.setBounds(0, 0, 56, 56)
        item_phone.findViewById<TextView>(R.id.tv_title).setCompoundDrawables(phone, null, null, null)
        item_phone.findViewById<ImageView>(R.id.iv_right).visibility = View.GONE

        item_userAgree.findViewById<TextView>(R.id.tv_title).text = "注册协议"
        val userAgree = ContextCompat.getDrawable(this, R.mipmap.icon_settings_user)
        userAgree?.setBounds(0, 0, 56, 56)
        item_userAgree.findViewById<TextView>(R.id.tv_title).setCompoundDrawables(userAgree, null, null, null)
        item_userAgree.findViewById<TextView>(R.id.tv_right).visibility = View.GONE
        item_userAgree.findViewById<ImageView>(R.id.iv_right).visibility = View.VISIBLE

        item_privacyAgree.findViewById<TextView>(R.id.tv_title).text = "隐私协议"
        val privacyAgree = ContextCompat.getDrawable(this, R.mipmap.icon_settings_privacy)
        privacyAgree?.setBounds(0, 0, 56, 56)
        item_privacyAgree.findViewById<TextView>(R.id.tv_title).setCompoundDrawables(privacyAgree, null, null, null)
        item_privacyAgree.findViewById<TextView>(R.id.tv_right).visibility = View.GONE
        item_privacyAgree.findViewById<ImageView>(R.id.iv_right).visibility = View.VISIBLE


        if (userManager.userIsLogin()) {
            btn_switch.visibility = View.VISIBLE
        } else {
            btn_switch.visibility = View.GONE

        }
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.iv_left_icon -> {
                finish()
            }
//            R.id.item_phone -> {
//                if (userManager.userIsLogin()) {
//                    startActivity(BindMobileActivity::class.java)
//                    return
//                } else {
//                    ToastUtils.showLong("请先使用微信登录")
//                }
//            }
            R.id.btn_switch -> {//退出登录

                UserManager.INSTANCE.userLogout()
                finish()
                KLog.e("setOnClick", "btn_switch has been clicked.")
            }
            R.id.item_userAgree -> {//用户协议
                MintsWebViewActivity.startWebView(name = "用户协议", url =  Constant.REGISTER_URL)
            }
            R.id.item_privacyAgree -> {//隐私协议
                MintsWebViewActivity.startWebView(name = "隐私协议", url =  Constant.PRIVACY_URL)
            }

        }

    }

}

