package com.mints.street.main.my

import android.Manifest
import android.app.Dialog
import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.CheckBox
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.widget.LinearLayoutCompat
import androidx.core.content.ContextCompat
import androidx.lifecycle.Observer
import cn.sharesdk.framework.ShareSDK
import cn.sharesdk.wechat.friends.Wechat
import com.bumptech.glide.Glide
import com.coorchice.library.utils.LogUtils
import com.fry.base.base.BaseFragment
import com.google.android.material.bottomsheet.BottomSheetDialog
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.bean.UserBean
import com.mints.street.bean.WXInfo
import com.mints.street.common.Constant
import com.mints.street.databinding.FragmentMyBinding
import com.mints.street.login.LoginActivity
import com.mints.street.login.LoginApi
import com.mints.street.login.OnLoginListener
import com.mints.street.manager.UserManager
import com.mints.street.manager.oaid.OaidManager
import com.mints.street.utils.json.JsonUtil
import com.mints.street.webview.MintsWebViewActivity
import com.mints.street.widget.dialog.WxLoginDialog
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnRefreshListener
import com.tbruyelle.rxpermissions2.RxPermissions
import kotlinx.android.synthetic.main.fragment_my.*
import me.goldze.mvvmhabit.base.AppManager
import me.goldze.mvvmhabit.utils.RxUtils
import me.goldze.mvvmhabit.utils.ToastUtils
import me.goldze.mvvmhabit.utils.systembar.StatusBarUtil
import java.util.*


/**
 * Created by 冯瑞雨 on 2021/7/2.
 * 个人中心页面
 */
class MyFragment : BaseFragment<FragmentMyBinding, MyViewModel>(), OnRefreshListener {

    private val userManager by lazy { UserManager.INSTANCE }
    private var userConfig: UserBean? = null

    private val loginDialog by lazy { WxLoginDialog(requireActivity()).init() }

    override fun initContentView(inflater: LayoutInflater?,
                                 container: ViewGroup?,
                                 savedInstanceState: Bundle?) = R.layout.fragment_my

    override fun initVariableId() = BR.viewModel
    override fun initData() {
        super.initData()
        StatusBarUtil.setHeightAndPadding(AppManager.getAppManager().currentActivity, binding.llMyRoot)
        srl_my.setOnRefreshListener(this)
        initView()
        initListener()
    }

    override fun onResume() {
        super.onResume()

        if (userManager.userIsLogin()) {
            //查询用户登录信息
            viewModel.getmyInfo()
        } else {
            //重新显示界面信息
            setUserLoginStatus()
        }
    }

    private fun initListener() {
        //设置登录点击事件
        RxUtils.onMultiClick(binding.llMyLogin) {
            if (!userManager.userIsLogin()) {
                loginDialog.show()
            }
        }
        //设置立即领取点击事件
        binding.btTry.setOnClickListener {
            startActivity(OpenvipActivity::class.java)
        }
        RxUtils.onMultiClick(aboutus) {
            startActivity(AboutusActivity::class.java)
//            viewModel.test()
        }
        RxUtils.onMultiClick(moresettings) {
            startActivity(MoresettingsActivity::class.java)
        }

        loginDialog.setListener(object : WxLoginDialog.WxLoginListener {
            override fun loginSuc(wxInfo: String) {
                if (!isHidden) {//当前fragment正在显示调用微信登陆接口
                    viewModel.wechatlogin(wxInfo)
                }
            }
        })
    }


    override fun initViewObservable() {
        super.initViewObservable()
        viewModel.myInfodata.observe(this, Observer<UserBean> {
            srl_my.finishRefresh(true)
            if (it == null) {
                userConfig = null
                return@Observer
            }
            userConfig = it
            //获取我的数据
            setUserLoginStatus()
        })
    }

    private fun setUserLoginStatus() {
        if (userManager.userIsLogin()) {
            // 已登录
            item_title_id.text = userManager.getWxName()
            Glide.with(requireContext()).load(userManager.getWxHeader()).into(item_title_avatar)

            // 是否会员
            if (userManager.getVipFlag()) {
                tv_tips.text = "恭喜您，成为全球会员啦~"
                bt_try.visibility = View.INVISIBLE
            } else {
                tv_tips.text = "恭喜你被全球会员卡砸中~"
                bt_try.visibility = View.VISIBLE
            }
        } else {
            // 未登录
            item_title_id.text = "未登录"
            tv_tips.text = "恭喜你被全球会员卡砸中~"
            bt_try.visibility = View.VISIBLE
            item_title_avatar.setImageResource(R.mipmap.ic_my)
        }
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        viewModel.getmyInfo()
        setUserLoginStatus()
        srl_my.finishRefresh(true)
    }

    override fun onDestroy() {
        super.onDestroy()

        loginDialog.setListener(null)
        loginDialog.onDestroy()
    }


    private fun initView() {
        aboutus.findViewById<TextView>(R.id.tv_title).text = "关于我们"
        val userAgree = ContextCompat.getDrawable(context!!, R.mipmap.iv_about_us)
        userAgree?.setBounds(0, 0, 56, 56)
        aboutus.findViewById<TextView>(R.id.tv_title).setCompoundDrawables(userAgree, null, null, null)
        aboutus.findViewById<TextView>(R.id.tv_right).visibility = View.GONE
        aboutus.findViewById<ImageView>(R.id.iv_right).visibility = View.VISIBLE

        item_my_phone.findViewById<TextView>(R.id.tv_title).text = "人工客服"
        item_my_phone.findViewById<TextView>(R.id.tv_right).text = "400-777-6667"
        item_my_phone.findViewById<TextView>(R.id.tv_right).visibility = View.VISIBLE
        val itemMyPhone = ContextCompat.getDrawable(context!!, R.mipmap.ic_my_phone)
        itemMyPhone?.setBounds(0, 0, 56, 56)
        item_my_phone.findViewById<TextView>(R.id.tv_title).setCompoundDrawables(itemMyPhone, null, null, null)
        item_my_phone.findViewById<ImageView>(R.id.iv_right).visibility = View.GONE

        moresettings.findViewById<TextView>(R.id.tv_title).text = "更多设置"
        val privacyAgree = ContextCompat.getDrawable(context!!, R.mipmap.iv_more_settings)
        privacyAgree?.setBounds(0, 0, 56, 56)
        moresettings.findViewById<TextView>(R.id.tv_title).setCompoundDrawables(privacyAgree, null, null, null)
        moresettings.findViewById<TextView>(R.id.tv_right).visibility = View.GONE
        moresettings.findViewById<ImageView>(R.id.iv_right).visibility = View.VISIBLE

        ic_theme.findViewById<TextView>(R.id.tv_theme_name).text = "享8大特权"
        ic_theme.findViewById<TextView>(R.id.tv_me1).text = "无线缩放"
        ic_theme.findViewById<TextView>(R.id.tv_me2).text = "国内外街景"
        ic_theme.findViewById<TextView>(R.id.tv_me3).text = "VR全景"
        ic_theme.findViewById<TextView>(R.id.tv_me4).text = "3D地图"
        ic_theme.findViewById<TextView>(R.id.tv_me5).text = "国外搜索"
        ic_theme.findViewById<TextView>(R.id.tv_me6).text = "高清地图"
        ic_theme.findViewById<TextView>(R.id.tv_me7).text = "历史影像"
        ic_theme.findViewById<TextView>(R.id.tv_me8).text = "景点推荐"
        ic_theme.findViewById<ImageView>(R.id.iv_decorate1).setImageResource(R.mipmap.icon_me_tiao)
        ic_theme.findViewById<ImageView>(R.id.iv_decorate2).setImageResource(R.mipmap.icon_me_tiao)
        ic_theme.findViewById<ImageView>(R.id.iv_me1).setImageResource(R.mipmap.icon_me1)
        ic_theme.findViewById<ImageView>(R.id.iv_me2).setImageResource(R.mipmap.icon_me2)
        ic_theme.findViewById<ImageView>(R.id.iv_me3).setImageResource(R.mipmap.icon_me3)
        ic_theme.findViewById<ImageView>(R.id.iv_me4).setImageResource(R.mipmap.icon_me4)
        ic_theme.findViewById<ImageView>(R.id.iv_me5).setImageResource(R.mipmap.icon_me5)
        ic_theme.findViewById<ImageView>(R.id.iv_me6).setImageResource(R.mipmap.icon_me6)
        ic_theme.findViewById<ImageView>(R.id.iv_me7).setImageResource(R.mipmap.icon_me7)
        ic_theme.findViewById<ImageView>(R.id.iv_me8).setImageResource(R.mipmap.icon_me8)
    }
}

