package com.mints.street.main.my

import android.app.Application
import androidx.lifecycle.MutableLiveData
import com.mints.street.bean.BaseResponse
import com.mints.street.bean.UserBean
import com.mints.street.common.DeviceInfo
import com.mints.street.manager.UserManager
import com.mints.street.manager.oaid.OaidManager
import com.mints.street.model.ApiModel
import com.mints.street.netwrok.base.HttpSubscribeImpl
import com.mints.street.utils.DeviceUuidFactory
import me.goldze.mvvmhabit.base.BaseViewModel
import me.goldze.mvvmhabit.utils.KLog
import java.util.HashMap

/**
 * Created by 冯瑞雨 on 2021/7/2.
 */
class MyViewModel(application: Application) : BaseViewModel(application) {

    val myInfodata: MutableLiveData<UserBean> = MutableLiveData()

    val deviceInfo: DeviceInfo
        get() {
            return DeviceInfo.instance
        }

    /**
     * 获取个人信息
     */
    fun getmyInfo() {
        ApiModel.getuserInfo(lifecycleProvider).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<UserBean>>(
                        this@MyViewModel, true) {

                    override fun onBusinessSuccess(response: BaseResponse<UserBean>) {
                        myInfodata.value = response.result
                        //保存用户信息到Usermanager
                        UserManager.INSTANCE.saveUserInfo(response.result)
                        KLog.e("getmyInfo", response.result.toString())
                    }

                    override fun onError(e: Throwable) {
                        super.onError(e)
                        KLog.e("getmyInfo", "用户登录信息请求失败")
                    }

                }
        )
    }

    fun wechatlogin(wxInfo: String) {
        val vo = HashMap<String, Any>()
        vo["wxInfo"] = wxInfo
//        vo["shumeiId"] = ShumeiManager.getInstance().shumeiDeviceId
        vo["device"] = DeviceUuidFactory().deviceUuid.toString()
        ApiModel.wechatlogin(lifecycleProvider,vo).safeSubscribe(object : HttpSubscribeImpl<BaseResponse<UserBean>>(
                this@MyViewModel, true) {
            override fun onBusinessSuccess(response: BaseResponse<UserBean>) {
                //保存用户信息到Usermanager
                UserManager.INSTANCE.saveUserInfo(response.result)

                this@MyViewModel.saveTerminalInfo()//提交设备信息
                this@MyViewModel.getmyInfo()
                KLog.e("wechatlogin","微信登录接口成功")

            }

            override fun onError(e: Throwable) {
                super.onError(e)
                KLog.e("wechatlogin", "微信登录接口失败")
            }
        })
    }

    /**
     *  提交设备信息
     */
    fun saveTerminalInfo() {
        val vo = hashMapOf<String, Any>()
        val macAddress: String = deviceInfo.getMacAddress()
        val mac = macAddress.replace(":", "")
        vo["mac"] = mac
        vo["mac1"] = macAddress
        vo["androidid"] = deviceInfo.getAndroidId(null)
        vo["imei"] = deviceInfo.iMEI
        vo["oaid"] = OaidManager.getOaid()
        vo["os"] = "android"
        vo["model"] = deviceInfo.newModel
        vo["uuid"] = DeviceUuidFactory().deviceUuid
        vo["osversion"] = deviceInfo.oSVersion
        vo["appversion"] = deviceInfo.versionName

        ApiModel.saveTerminalInfo(lifecycleProvider, vo).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<Any>>(
                        this@MyViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<Any>) {
                        KLog.e("saveTerminalInfo", response.result.toString())
                    }
                })
    }

}