package com.mints.street.main.my

import android.app.Application
import androidx.lifecycle.MutableLiveData
import com.blankj.utilcode.util.ToastUtils
import com.mints.street.bean.*
import com.mints.street.common.DeviceInfo
import com.mints.street.manager.UserManager
import com.mints.street.manager.oaid.OaidManager
import com.mints.street.model.ApiModel
import com.mints.street.netwrok.base.HttpSubscribeImpl
import com.mints.street.utils.DeviceUuidFactory
import me.goldze.mvvmhabit.base.BaseViewModel
import me.goldze.mvvmhabit.utils.KLog
import java.util.HashMap

class OpenvipViewModel(application: Application) : BaseViewModel(application) {


    val deviceInfo: DeviceInfo
        get() {
            return DeviceInfo.instance
        }

    val Vipdata: MutableLiveData<VipBean> = MutableLiveData()
    val Paymentdata: MutableLiveData<PaymentBean> = MutableLiveData()
    val vippayParams: MutableLiveData<WxPayParamBean> = MutableLiveData()
    val isPaySuc: MutableLiveData<Boolean> = MutableLiveData()

    /**
     * 开通Vip
     *
     * @param mobile
     */
    fun getVipProductsByType() {
        val vo = HashMap<String, Any>()
        vo["type"] = 1

        ApiModel.getVipProductsByType(lifecycleProvider, vo).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<VipBean>>(
                        this@OpenvipViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<VipBean>) {
                        Vipdata.value = response.result
                    }

                    override fun onError(e: Throwable) {
                        KLog.e("getVipProductsByType", e.printStackTrace())
                    }
                })
    }

    /**
     * Vip支付方式
     */
    fun getpaymentwey(os: String) {
        val vo = HashMap<String, Any>()
        vo["os"] = os
        ApiModel.getpaymentwey(lifecycleProvider, vo).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<PaymentBean>>(
                        this@OpenvipViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<PaymentBean>) {
                        Paymentdata.value = response.result
                        KLog.e("getpaymentwey", response.result.toString())
                    }

                    override fun onError(e: Throwable) {
                        KLog.e("getpaymentwey", "获取支付方式失败")
                    }
                }
        )
    }

    /**
     * 获取支付参数
     */
    fun getVipPayParams(payChannel:String,pid: String) {
        val vo = HashMap<String, Any>()
        vo["payChannel"] = payChannel
        vo["pid"] = pid
        ApiModel.getVipPayParams(lifecycleProvider, vo).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<WxPayParamBean>>(
                        this@OpenvipViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<WxPayParamBean>) {
                        vippayParams.value = response.result
                    }

                    override fun onError(e: Throwable) {
                        KLog.e("getVipPayParams", e.printStackTrace())
                    }   
                }
        )
    }

    /**
     * 支付成功
     */
    fun queryVipOrder(tid: String) {
        val vo = HashMap<String, Any>()
        vo["tid"] = tid
        ApiModel.queryVipOrder(lifecycleProvider, vo).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<Any>>(
                        this@OpenvipViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<Any>) {
                        isPaySuc.value = true
                    }

                    override fun onError(e: Throwable) {
                    }
                }
        )
    }

    fun wechatlogin(wxInfo: String) {
        val vo = HashMap<String, Any>()
        vo["wxInfo"] = wxInfo
//        vo["shumeiId"] = ShumeiManager.getInstance().shumeiDeviceId
        vo["device"] = DeviceUuidFactory().deviceUuid.toString()
        ApiModel.wechatlogin(lifecycleProvider,vo).safeSubscribe(object : HttpSubscribeImpl<BaseResponse<UserBean>>(
                this@OpenvipViewModel, true) {
            override fun onBusinessSuccess(response: BaseResponse<UserBean>) {
                //保存用户信息到Usermanager
                UserManager.INSTANCE.saveUserInfo(response.result)

                this@OpenvipViewModel.saveTerminalInfo()//提交设备信息
                KLog.e("wechatlogin","微信登录接口成功")

                ToastUtils.showLong("登录成功")
            }

            override fun onError(e: Throwable) {
                super.onError(e)
                KLog.e("wechatlogin", "微信登录接口失败")
            }
        })
    }

    /**
     *  提交设备信息
     */
    fun saveTerminalInfo() {
        val vo = hashMapOf<String, Any>()
        val macAddress: String = deviceInfo.getMacAddress()
        val mac = macAddress.replace(":", "")
        vo["mac"] = mac
        vo["mac1"] = macAddress
        vo["androidid"] = deviceInfo.getAndroidId(null)
        vo["imei"] = deviceInfo.iMEI
        vo["oaid"] = OaidManager.getOaid()
        vo["os"] = "android"
        vo["model"] = deviceInfo.newModel
        vo["uuid"] = DeviceUuidFactory().deviceUuid
        vo["osversion"] = deviceInfo.oSVersion
        vo["appversion"] = deviceInfo.versionName

        ApiModel.saveTerminalInfo(lifecycleProvider, vo).safeSubscribe(
                object : HttpSubscribeImpl<BaseResponse<Any>>(
                        this@OpenvipViewModel, true) {
                    override fun onBusinessSuccess(response: BaseResponse<Any>) {
                        KLog.e("saveTerminalInfo", response.result.toString())
                    }
                })
    }
}
