package com.mints.street.main.my

import android.Manifest
import android.os.Build
import android.os.Bundle
import android.view.View
import com.fry.base.base.BaseActivity
import com.github.iielse.switchbutton.SwitchView
import com.mints.street.AppApplication
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.common.Constant
import com.mints.street.databinding.ActivityPermissionsBinding
import com.mints.street.main.MainActivity
import com.mints.street.utils.NetUtils
import com.mints.street.utils.nodoubleclick.AntiShake
import com.mints.street.webview.MintsWebViewActivity
import com.tbruyelle.rxpermissions2.RxPermissions
import com.tencent.mmkv.MMKV
import kotlinx.android.synthetic.main.activity_permissions.*
import me.goldze.mvvmhabit.utils.ToastUtils

/**
 * 描述：权限授权
 * 作者：孟崔广
 * 时间：2021/7/14 15:51
 */
class PermissionsActivity  : BaseActivity<ActivityPermissionsBinding, PermissionsViewModel>(), View.OnClickListener {

    override fun initVariableId() = BR.viewModel
    override fun initContentView(savedInstanceState: Bundle?)= R.layout.activity_permissions
    override fun initData() {
        super.initData()
        initView()
        initListener()
    }

    private fun initListener() {
        btnPermissionsDont.setOnClickListener(this)
        btnPermissionsNext.setOnClickListener(this)
        tv_private.setOnClickListener(this)
        tv_regist.setOnClickListener(this)
    }

    private fun initView() {
        svPosition.setOnStateChangedListener(object : SwitchView.OnStateChangedListener {
            override fun toggleToOn(view: SwitchView?) {
                svPosition?.isOpened = true
            }

            override fun toggleToOff(view: SwitchView?) {
                svPosition?.isOpened = false
            }
        })

        svStrong.setOnStateChangedListener(object : SwitchView.OnStateChangedListener {
            override fun toggleToOn(view: SwitchView?) {
                svStrong?.isOpened = true
            }

            override fun toggleToOff(view: SwitchView?) {
                svStrong?.isOpened = false
            }
        })

        svReadmsg.setOnStateChangedListener(object : SwitchView.OnStateChangedListener {
            override fun toggleToOn(view: SwitchView?) {
                svReadmsg?.isOpened = true
            }

            override fun toggleToOff(view: SwitchView?) {
                svReadmsg?.isOpened = false
            }
        })

    }

    override fun onClick(v: View?) {
        when(v?.id){
            R.id.btnPermissionsDont -> ToastUtils.showLong("请您同意授权,否则将无法使用APP功能")
            R.id.btnPermissionsNext->{
                if (AntiShake.check(v?.id)) return

                com.blankj.utilcode.util.ToastUtils.showLong("努力加载中...")
                val rxPermissions = RxPermissions(this)
                rxPermissions.request(
                        if(svPosition.isOpened&&Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) Manifest.permission.ACCESS_BACKGROUND_LOCATION else "",
                        if(svPosition.isOpened) Manifest.permission.ACCESS_FINE_LOCATION else "",
                        if(svStrong.isOpened) Manifest.permission.WRITE_EXTERNAL_STORAGE else "",
                        if(svReadmsg.isOpened) Manifest.permission.READ_PHONE_STATE else ""
                )
                        .subscribe {
                            startActivityThenKill(MainActivity::class.java)
                        }

                MMKV.mmkvWithID(Constant.FIRST_OPEN_PERMISSIONS).encode(Constant.FIRST_OPEN_PERMISSIONS, true)
            }
            R.id.tv_regist->{
                if (!NetUtils.isNetworkConnected(AppApplication.getContext())) {
                    com.blankj.utilcode.util.ToastUtils.showShort("网络异常,请检测网络")
                    return
                }

                MintsWebViewActivity.startWebView(name = "用户协议",url =  Constant.REGISTER_URL)
            }
            R.id.tv_private->{
                if (!NetUtils.isNetworkConnected(AppApplication.getContext())) {
                    com.blankj.utilcode.util.ToastUtils.showShort("网络异常,请检测网络")
                    return
                }

                MintsWebViewActivity.startWebView(name = "隐私协议",url = "http://test.mints-id.com/map-api/map/privacy.html")
            }
        }
    }
}