package com.mints.street.main.vr

import android.content.Context
import android.graphics.Color
import android.graphics.Typeface
import android.os.Bundle
import android.view.LayoutInflater
import android.view.ViewGroup
import android.view.animation.AccelerateInterpolator
import android.view.animation.DecelerateInterpolator
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.viewpager2.adapter.FragmentStateAdapter
import androidx.viewpager2.widget.ViewPager2
import com.fry.base.base.BaseFragment
import com.fry.base.widget.ScaleTransitionPagerTitleView
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.databinding.FragmentVrBinding
import com.mints.street.main.my.MyFragment
import me.goldze.mvvmhabit.base.AppManager
import me.goldze.mvvmhabit.utils.Utils
import me.goldze.mvvmhabit.utils.systembar.StatusBarUtil
import net.lucode.hackware.magicindicator.MagicIndicator
import net.lucode.hackware.magicindicator.buildins.UIUtil
import net.lucode.hackware.magicindicator.buildins.commonnavigator.CommonNavigator
import net.lucode.hackware.magicindicator.buildins.commonnavigator.abs.CommonNavigatorAdapter
import net.lucode.hackware.magicindicator.buildins.commonnavigator.abs.IPagerIndicator
import net.lucode.hackware.magicindicator.buildins.commonnavigator.abs.IPagerTitleView
import net.lucode.hackware.magicindicator.buildins.commonnavigator.indicators.LinePagerIndicator

/**
 * Created by 冯瑞雨 on 2021/7/2.
 * VR页面
 */
class VRFragment : BaseFragment<FragmentVrBinding, VRViewModel>() {
    override fun initContentView(
            inflater: LayoutInflater?, container: ViewGroup?,
            savedInstanceState: Bundle?) =
            R.layout.fragment_vr

    override fun initVariableId() = BR.viewModel

    override fun initData() {
        super.initData()
        StatusBarUtil.setHeightAndPadding(AppManager.getAppManager().currentActivity, binding.magicIndicator)
        initTabFragment()
    }

    private fun initTabFragment() {
        val tabNames = mutableListOf<String>()
        tabNames.add("免费专区")
        tabNames.add("精选体验")

        val fragments = mutableListOf<Fragment>()
        fragments.add(FreeZoneFragment())
        fragments.add(FeaturedExperienceFragment())


        //初始化viewpager2
        binding.viewPager.init(this, fragments)

        //初始化 magic_indicator
        binding.magicIndicator.bindViewPager2(binding.viewPager, tabNames, {}, 20f, true)


    }

    private fun ViewPager2.init(
            fragment: Fragment,
            fragments: List<Fragment>,
            isUserInputEnabled: Boolean = true
    ): ViewPager2 {
        //是否可滑动
        this.isUserInputEnabled = isUserInputEnabled
        //设置适配器
        adapter = object : FragmentStateAdapter(fragment) {
            override fun createFragment(position: Int) = fragments[position]
            override fun getItemCount() = fragments.size
        }
        return this
    }

    private fun MagicIndicator.bindViewPager2(
            viewPager: ViewPager2,
            mStringList: List<String> = arrayListOf(),
            action: (index: Int) -> Unit = {},
            mTextSize: Float = 18f,
            isBold: Boolean = false
    ) {
        val commonNavigator = CommonNavigator(Utils.getContext())
        commonNavigator.adapter = object : CommonNavigatorAdapter() {

            override fun getCount(): Int {
                return mStringList.size
            }

            override fun getTitleView(context: Context, index: Int): IPagerTitleView {
                return ScaleTransitionPagerTitleView(Utils.getContext()).apply {
                    //设置文本
                    text = mStringList[index]
                    //字体大小
                    textSize = mTextSize
                    // 加粗
                    typeface =
                            if (isBold) Typeface.defaultFromStyle(Typeface.BOLD) else Typeface.defaultFromStyle(
                                    Typeface.NORMAL
                            )
                    //未选中颜色
                    normalColor = Color.BLACK
                    //选中颜色
                    selectedColor = ContextCompat.getColor(context, R.color.main_blue_lower)
                    //点击事件
                    setOnClickListener {
                        viewPager.currentItem = index
                        action.invoke(index)
                    }
                }
            }

            override fun getIndicator(context: Context): IPagerIndicator {
                return LinePagerIndicator(context).apply {
                    mode = LinePagerIndicator.MODE_EXACTLY
                    //线条的宽高度
                    lineHeight = UIUtil.dip2px(Utils.getContext(), 3.0).toFloat()
                    lineWidth = UIUtil.dip2px(Utils.getContext(), 30.0).toFloat()
                    //线条的圆角
                    roundRadius = UIUtil.dip2px(Utils.getContext(), 6.0).toFloat()
                    startInterpolator = AccelerateInterpolator()
                    endInterpolator = DecelerateInterpolator(2.0f)
                    //线条的颜色
                    setColors(ContextCompat.getColor(context, R.color.main_blue_lower))
                }
            }
        }
        this.navigator = commonNavigator

        viewPager.registerOnPageChangeCallback(object : ViewPager2.OnPageChangeCallback() {
            override fun onPageSelected(position: Int) {
                super.onPageSelected(position)
                this@bindViewPager2.onPageSelected(position)
                action.invoke(position)
            }

            override fun onPageScrolled(
                    position: Int,
                    positionOffset: Float,
                    positionOffsetPixels: Int
            ) {
                super.onPageScrolled(position, positionOffset, positionOffsetPixels)
                this@bindViewPager2.onPageScrolled(position, positionOffset, positionOffsetPixels)
            }

            override fun onPageScrollStateChanged(state: Int) {
                super.onPageScrollStateChanged(state)
                this@bindViewPager2.onPageScrollStateChanged(state)
            }
        })
    }
}