package com.mints.street.manager

import android.text.TextUtils
import com.mints.street.bean.UserBean
import com.tencent.mmkv.MMKV

/**
 * 描述：管理用户信息
 * 作者：孟崔广
 * 时间：2021/7/9 14:48
 */
class UserManager {

    companion object {

        /**
         * 用户ID
         */
        private const val USER_ID = "userId"

        /**
         * 用户CODE_ID
         */
        private const val CODE_ID = "codeId"

        /**
         * 登陆状态ID
         */
        private const val TOKEN_ID = "tokenId"

        /**
         * 手机号
         */
        private const val MOBILE = "mobile"

        /**
         * 微信名称
         */
        private const val WX_NAME = "wx_name"

        /**
         * 微信头像
         */
        private const val WX_HEADER = "wx_header"

        /**
         * 微信id
         */
        private const val WX_OPENID = "wx_openid"

        /**
         * 是否购买会员
         */
        private const val IS_FOREVER = "is_forever"

        /**
         * 会员是否永久
         */
        private const val EXPIRETIME = "expiretime"

        /**
         * vip支付是否有效
         */
        private const val VIP_FLAG = "vipFlag"

        val INSTANCE: UserManager by lazy(mode = LazyThreadSafetyMode.SYNCHRONIZED) {
            UserManager()
        }
    }

    init {
    }

    /**
     * 保存用户信息
     *
     * @param userInfo
     */
    fun saveUserInfo(userInfo: UserBean) {

        val user = userInfo.consumer
        val token = userInfo.token
        if (token != null) {
            MMKV.mmkvWithID(TOKEN_ID).encode(TOKEN_ID, token)
        }

        if (user != null) {
            val mobile: String? = user.mobile
            val openid: String? = user.openid
            MMKV.mmkvWithID(USER_ID).encode(USER_ID, user.pk_id.toString())
            MMKV.mmkvWithID(CODE_ID).encode(CODE_ID, user.idcode)
            if (!TextUtils.isEmpty(mobile)) {
                MMKV.mmkvWithID(MOBILE).encode(MOBILE, mobile)
            }
            if (!TextUtils.isEmpty(user.nickname)) {
                MMKV.mmkvWithID(WX_NAME).encode(WX_NAME, user.nickname)
            }
            if (!TextUtils.isEmpty(user.head)) {
                MMKV.mmkvWithID(WX_HEADER).encode(WX_HEADER, user.head)
            }
            MMKV.mmkvWithID(IS_FOREVER).encode(IS_FOREVER, user.isForever)
            MMKV.mmkvWithID(EXPIRETIME).encode(EXPIRETIME, user.expireTime)
            if (!TextUtils.isEmpty(openid)) {
                MMKV.mmkvWithID(WX_OPENID).encode(WX_OPENID, openid)
            }
            if (user.expireTime > 0) {
                MMKV.mmkvWithID(VIP_FLAG).encode(VIP_FLAG, true)
            } else {
                MMKV.mmkvWithID(VIP_FLAG).encode(VIP_FLAG, false)
            }
        }
    }

    /**
     * 单独获取用户id
     */
    fun getMobile(): String {
        val kv = MMKV.mmkvWithID(MOBILE)
        return kv.decodeString(MOBILE, "")
    }

    /**
     * 判断用户是否已登录
     *
     * @return true为已登录   false 为未登录
     */
    fun userIsLogin(): Boolean {
        val wxId = MMKV.mmkvWithID(WX_OPENID).decodeString(WX_OPENID, "")
        val mobile = MMKV.mmkvWithID(MOBILE).decodeString(MOBILE, "")
        return !TextUtils.isEmpty(wxId)||!TextUtils.isEmpty(mobile)
    }

    fun getUserID(): String {
        val kv = MMKV.mmkvWithID(USER_ID)
        return kv.decodeString(USER_ID, "")
    }

    fun getTokenID(): String {
        val kv = MMKV.mmkvWithID(TOKEN_ID)
        return kv.decodeString(TOKEN_ID, "")
    }

    fun getCodeID(): String {
        val kv = MMKV.mmkvWithID(CODE_ID)
        return kv.decodeString(CODE_ID, "")
    }

    fun getWxName(): String {
        val kv = MMKV.mmkvWithID(WX_NAME)
        return kv.decodeString(WX_NAME, "")
    }

    fun getWxHeader(): String {
        val kv = MMKV.mmkvWithID(WX_HEADER)
        return kv.decodeString(WX_HEADER, "")
    }

    fun getWxOpenid(): String {
        val kv = MMKV.mmkvWithID(WX_OPENID)
        return kv.decodeString(WX_OPENID, "")
    }

    fun getIsforever(): Boolean {
        val kv = MMKV.mmkvWithID(IS_FOREVER)
        return kv.decodeBool(IS_FOREVER, false)
    }

    fun getExpiretime(): Long {
        val kv = MMKV.mmkvWithID(EXPIRETIME)
        return kv.decodeLong(EXPIRETIME, 0)
    }

    fun getVipFlag(): Boolean {
        val kv = MMKV.mmkvWithID(VIP_FLAG)
        return kv.decodeBool(VIP_FLAG, false)
    }

    fun setVipFlag(vipFlag:Boolean){
        MMKV.mmkvWithID(VIP_FLAG).encode(VIP_FLAG, vipFlag)
    }

    fun userLogout() {
        MMKV.mmkvWithID(USER_ID).remove(USER_ID)
        MMKV.mmkvWithID(MOBILE).remove(MOBILE)
        MMKV.mmkvWithID(CODE_ID).remove(CODE_ID)
        MMKV.mmkvWithID(TOKEN_ID).remove(TOKEN_ID)
        MMKV.mmkvWithID(WX_OPENID).remove(WX_OPENID)
        MMKV.mmkvWithID(WX_NAME).remove(WX_NAME)
        MMKV.mmkvWithID(WX_HEADER).remove(WX_HEADER)
        MMKV.mmkvWithID(IS_FOREVER).remove(IS_FOREVER)
        MMKV.mmkvWithID(EXPIRETIME).remove(EXPIRETIME)
        MMKV.mmkvWithID(VIP_FLAG).encode(VIP_FLAG, false)
    }

}