package com.mints.street.netwrok.base

import androidx.lifecycle.ViewModel
import com.mints.street.adapter.GridBenfitViewModel
import com.mints.street.bean.BaseResponse
import com.mints.street.main.home.SearchMapViewModel
import me.goldze.mvvmhabit._enum.ViewStatus
import me.goldze.mvvmhabit.base.BaseViewModel
import me.goldze.mvvmhabit.http.ExceptionHandle
import me.goldze.mvvmhabit.http.ResponseThrowable
import me.goldze.mvvmhabit.utils.ToastUtils
import me.goldze.mvvmhabit.utils.Utils
import retrofit2.Response

/**
 *
 * @author jeme
 * @date 2020/1/10
 */

abstract class HttpSubscribeImpl<T : BaseResponse<*>>(viewModel: BaseViewModel?, val needToast: Boolean = true) : IHttpSubscribe<T>(viewModel) {

    constructor(viewModel: BaseViewModel?) : this(viewModel, true)

    /***
     * 默认有数据
     */
    open fun isEmpty(body: T?) = false

    override fun onNext(t: Response<T>) {

        if(t.isSuccessful){
            isBusinessSuccess = t.body()?.isOk?:false
            if (t.body()?.isOk == true) {
                onBusinessSuccess(t.body()!!)
            } else {
                onBusinessFail(t.body())
            }

            if (isEmpty(t.body())) {
                viewModel?.viewSwitch(ViewStatus.NODATA)
            } else {
                viewModel?.viewSwitch(ViewStatus.HASDATA)
            }
        }else{
            val throwable = ResponseThrowable(null, ExceptionHandle.ERROR.UNKNOWN,t.message())
            if(t.errorBody() != null){
                val response = Utils.getGson().fromJson(String(t.errorBody()!!.bytes()), BaseResponse::class.java)
                throwable.responseMessage = response?.message?:"未知错误"
            }
            onFailed(throwable)
        }
    }

    override fun onError(e: Throwable) {
        super.onError(e)
        if (e is ResponseThrowable) {
            onFailed(e)
        }
    }

    abstract fun onBusinessSuccess(response: T)
    open fun onBusinessFail(response: T?) {
        if (response == null) {
            viewModel?.viewSwitch(ViewStatus.NODATA)
            return
        }
        if (needToast) {
            ToastUtils.showLong(response.getMessage())
        }
    }

    open fun onFailed(throwable: ResponseThrowable?) {
        if (needToast) {
            ToastUtils.showLong(throwable?.responseMessage)
        }
        if (throwable?.code == ExceptionHandle.ERROR.HTTP_ERROR || throwable?.code == ExceptionHandle.ERROR.NETWORD_ERROR || throwable?.code == ExceptionHandle.ERROR.TIMEOUT_ERROR) {
            viewModel?.viewSwitch(ViewStatus.NONETWORK)
        }
    }

}