package com.mints.street.netwrok.base;

import android.content.Context;
import android.text.TextUtils;

import androidx.annotation.NonNull;

import com.fry.base.BuildConfig;
import com.mints.street.netwrok.common.GsonConverterFactory;
import com.mints.street.netwrok.common.OkHttpInterceptor;
import com.mints.street.utils.encry.AESUtils;
import com.orhanobut.logger.Logger;

import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;
import me.goldze.mvvmhabit.http.cookie.CookieJarImpl;
import me.goldze.mvvmhabit.http.cookie.store.PersistentCookieStore;
import me.goldze.mvvmhabit.http.interceptor.logging.Level;
import me.goldze.mvvmhabit.http.interceptor.logging.LoggingInterceptor;
import me.goldze.mvvmhabit.utils.Utils;
import okhttp3.ConnectionPool;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.internal.platform.Platform;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory;

/**
 * Created by mcg
 */
public class RetrofitClient {

    public HttpConfiguation mHttpConfig;

    private Context mContext = Utils.getContext();

    private OkHttpClient mOkHttpClient;
    private Retrofit mRetrofit;

    public RetrofitClient(@NonNull HttpConfiguation configuation) {
        mHttpConfig = configuation;
        if (TextUtils.isEmpty(configuation.getBaseUrl())) {
            throw new RuntimeException("baseUrl 不能为空");
        }

        OkHttpClient.Builder clientBuilder = new OkHttpClient.Builder()
                .cookieJar(new CookieJarImpl(new PersistentCookieStore(mContext)))
                .connectTimeout(mHttpConfig.getTimeOut(), TimeUnit.SECONDS)
                .writeTimeout(mHttpConfig.getTimeOut(), TimeUnit.SECONDS)
                .readTimeout(mHttpConfig.getTimeOut(), TimeUnit.SECONDS)
                .connectionPool(new ConnectionPool(8, 15, TimeUnit.SECONDS));

        if (BuildConfig.DEBUG) {
            HttpLoggingInterceptor logging = new HttpLoggingInterceptor(message -> {
                if (TextUtils.isEmpty(message)) return;
                String s = message.substring(0, 1);
                String request = message.substring(0, 4);
                if ("{".equals(s) || "[".equals(s)) {
                    Logger.json(message);
                } else if (request.contains("-->") || request.contains("<--")) {
                    Logger.d("Method" + message);
                } else {
                    Logger.d("params:" + message);
                }
            });
            logging.setLevel(HttpLoggingInterceptor.Level.BODY);
            clientBuilder.interceptors().add(logging);
        }

        OkHttpInterceptor okHttpInterceptor = new OkHttpInterceptor(AESUtils.getDefaultKey());
        clientBuilder.interceptors().add(okHttpInterceptor);

        mOkHttpClient = clientBuilder.build();
        mRetrofit = new Retrofit.Builder()
                .client(mOkHttpClient)
                .addConverterFactory(GsonConverterFactory.create())
                .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
                .baseUrl(mHttpConfig.getBaseUrl())
                .build();
    }

    /**
     * create you ApiService
     * Create an implementation of the API endpoints defined by the {@code service} interface.
     */
    public <T> T create(final Class<T> service) {
        if (service == null) {
            throw new RuntimeException("Api service is null!");
        }
        return mRetrofit.create(service);
    }
}
