package com.mints.street.netwrok.common;

import android.annotation.SuppressLint;

import com.fry.base.global.Constants;
import com.mints.street.bean.BaseResponse;
import com.mints.street.netwrok.base.HttpConfiguation;
import com.mints.street.netwrok.base.IHttpResponseListener;
import com.mints.street.netwrok.base.RetrofitClient;
import com.trello.rxlifecycle2.LifecycleProvider;

import io.reactivex.Observable;
import io.reactivex.ObservableSource;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Action;
import io.reactivex.functions.Consumer;
import io.reactivex.functions.Function;
import io.reactivex.schedulers.Schedulers;
import me.goldze.mvvmhabit.http.ResponseThrowable;
import me.goldze.mvvmhabit.utils.RxUtils;
import retrofit2.Response;

/**
 * Created by jeme on 2019/1/31
 */
public class HttpManager {

    private RetrofitClient mDefaultClient;
    private RetrofitClient mTimeClient;

    private static class SingletonHolder {
        private static HttpManager INSTANCE = new HttpManager();
    }


    public static HttpManager getInstance() {
        return SingletonHolder.INSTANCE;
    }

    private HttpManager() {
        mDefaultClient = new RetrofitClient(
                HttpConfiguation.create(Constants.getBaseUrl())
        );
    }

    public RetrofitClient getDefaultClient() {
        return mDefaultClient;
    }

    public RetrofitClient getTimeClient() {
        if (mTimeClient == null) {
            mTimeClient = new RetrofitClient(
                    HttpConfiguation.create(Constants.getBaseUrl())
                            .setTimeOut(5000));
        }
        return mTimeClient;
    }

    public <T> T getDefaultServices(Class<T> clz) {
        return getDefaultClient().create(clz);
    }

    public <B extends BaseResponse, T extends Response<B>>
    Observable<T> execute(LifecycleProvider life, final Observable<T> observable) {
        Observable<T> newObservable = Observable.just(observable)
                .observeOn(Schedulers.io())
                .flatMap((Function<Observable<T>, ObservableSource<T>>) tObservable -> {
                    //    ServerTimeHelper.requestServerTime();
                    return observable;
                })
                .observeOn(AndroidSchedulers.mainThread());
        if (life != null) {
            newObservable = newObservable.compose(RxUtils.bindToLifecycle(life));
        }
        return newObservable
                .compose(RxUtils.schedulersTransformer())
                .compose(RxUtils.exceptionTransformer())
                .compose(HttpErrorProcess.businessExceptionTransformer());
    }
}
