package com.mints.street.services;

import android.app.Service;
import android.content.Intent;
import android.os.IBinder;

import com.fry.base.utils.NotificationUtils;
import com.mints.street.BuildConfig;
import com.mints.street.eventbean.DownloadEvent;
import com.mints.street.utils.UpdateHelper;

import java.io.File;

import me.goldze.mvvmhabit.bus.RxBus;
import me.goldze.mvvmhabit.http.DownLoadManager;
import me.goldze.mvvmhabit.http.download.ProgressCallBack;
import me.goldze.mvvmhabit.utils.AppUtils;
import me.goldze.mvvmhabit.utils.ToastUtils;
import me.goldze.mvvmhabit.utils.Utils;
import okhttp3.ResponseBody;

/**
 * 升级下载服务
 *
 * @author 冯瑞雨
 * @date 2021/7/23
 */
public class UpdateServices extends Service {

    public static final String VERSION_CODE_KEY = "versionCode";
    public static final String DOWNLOAD_URL_KEY = "url";
    public static final String APK_MD5 = "apkMd5";
    public static final String IS_FORCED_UPDATE_KEY = "isForcedUpdate";

    private boolean isForcedUpdate;

    /***
     * 文件存放的路径
     */
    private String destFileDir;
    /***
     * 文件存放的名称
     */
    private String destFileName = "last-" + BuildConfig.VERSION_CODE + ".apk";
    private String targetVersionName;

    private NotificationUtils mNotificationUtils;
    private boolean hasDownloading = false;

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onStart(Intent intent, int startId) {
        super.onStart(intent, startId);
    }

    @Override
    public void onCreate() {
        super.onCreate();
        destFileDir = UpdateHelper.getDownloadDir(this);
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        if (intent == null) {
            return super.onStartCommand(intent, flags, startId);
        }
        if (intent.hasExtra(VERSION_CODE_KEY)) {
            targetVersionName = intent.getStringExtra(VERSION_CODE_KEY);
            destFileName = UpdateHelper.getFileName(targetVersionName);
        }
        if (mNotificationUtils == null) {
            mNotificationUtils = new NotificationUtils(getBaseContext());
        }
        //启动线程开始执行下载任务
        if (!hasDownloading && intent.hasExtra(DOWNLOAD_URL_KEY)) {
            startForeground(1, mNotificationUtils.getNotification("下载", "准备下载").build());
            downFile(intent.getStringExtra(DOWNLOAD_URL_KEY));
        }

        if (intent.hasExtra(IS_FORCED_UPDATE_KEY)) {
            isForcedUpdate = intent.getBooleanExtra(IS_FORCED_UPDATE_KEY, false);
        }
        /**
         * 这个常量是{@link #onStartCommand}方法的其中一个返回值：如果这个服务的进程在这个服务
         * 已经启动（也就是{@link #onStartCommand}方法已经运行完了）之后因为系统资源紧张等问题
         * 被系统关闭了，这个服务会被系统用最近一次启动这个服务所用的Intent重新启动它。这个Intent
         * 会被系统一直保存到{@link #stopSelf(int)}方法被调用为止。这种情况下，服务当中的
         * {@link #onStartCommand(Intent, int, int)}里面的intent不会是null。因为这种服务只
         * 会在所有启动它的intent都执行完了的情况下才不会重新启动。
         */

        return Service.START_REDELIVER_INTENT;/*super.onStartCommand(intent, flags, startId);*/
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }


    //下载更新文件
    private void downFile(final String url) {
        if (hasDownloading) {
            return;
        }
        DownLoadManager.getInstance().load(url, new ProgressCallBack<ResponseBody>(destFileDir, destFileName) {

            @Override
            public void onStart() {
                super.onStart();
                hasDownloading = true;
                ToastUtils.showShort("开始下载！");
            }

            @Override
            public void onSuccess(ResponseBody o) {
                //如果md5为空，则不校验文件是否合法
                ToastUtils.showShort("文件下载成功！");
                installApk(destFileDir + "/" + destFileName);

                if (isForcedUpdate) {
                    RxBus.getDefault().post(new DownloadEvent(0, 100, 0));
                }
                hasDownloading = false;
                //停止掉当前的服务
                stopSelf();
            }

            @Override
            public void progress(long progress, long total) {
                //更新状态栏上的下载进度信息
                int processValue = (int) ((progress / (double) total) * 100);
                if (isForcedUpdate) {
                    RxBus.getDefault().post(new DownloadEvent(total, processValue, progress));
                } else {
                    mNotificationUtils.sendNotificationProgress("升级", "已下载" + processValue + "%", processValue, null);
                }
            }

            @Override
            public void onError(Throwable e) {
                //如果断网，会抛IO异常
                e.printStackTrace();
                ToastUtils.showShort("文件下载失败！");
                File file = new File(destFileDir + File.separator + destFileName);
                if (file.exists()) {
                    file.delete();
                }

//                nm.cancel(notificationId);
                hasDownloading = false;
                //停止掉当前的服务
                stopSelf();
//                progressDialog.dismiss();
            }

            @Override
            public void onCompleted() {
//                progressDialog.dismiss();
                //下载完成后清除所有下载信息，执行安装提示
//                nm.cancel(notificationId);

                hasDownloading = false;
                //停止掉当前的服务
                stopSelf();
            }
        });
    }

    private void installApk(String s) {
        AppUtils.installApk(Utils.getContext(), s);
    }
}
