package com.mints.street.splash

import android.os.Bundle
import android.view.View
import android.widget.FrameLayout
import com.blankj.utilcode.util.ToastUtils
import com.bytedance.msdk.adapter.pangle.PangleNetworkRequestInfo
import com.bytedance.msdk.api.AdError
import com.bytedance.msdk.api.AdSlot
import com.bytedance.msdk.api.TTNetworkRequestInfo
import com.bytedance.msdk.api.splash.TTSplashAd
import com.bytedance.msdk.api.splash.TTSplashAdListener
import com.bytedance.msdk.api.splash.TTSplashAdLoadCallback
import com.fry.base.base.BaseActivity
import com.mints.street.BR
import com.mints.street.R
import com.mints.street.ad.CsjGroMoreManager
import com.mints.street.common.Constant
import com.mints.street.common.DeviceInfo
import com.mints.street.databinding.ActivitySplashBinding
import com.mints.street.main.MainActivity
import com.mints.street.main.my.PermissionsActivity
import com.mints.street.webview.MintsWebViewActivity
import com.mints.street.widget.dialog.DialogListener
import com.mints.street.widget.dialog.PowerDialog
import com.tencent.mmkv.MMKV
import me.goldze.mvvmhabit.utils.systembar.StatusBarUtil

/**
 * 描述：gromore开屏页
 * 作者：孟崔广
 * 时间：2021/7/9 13:39
 */
class SplashActivity : BaseActivity<ActivitySplashBinding, SplashViewModel>() {

    private var powerDialog: PowerDialog? = null

    private var msplash: FrameLayout? = null
    private var mTTSplashAd: TTSplashAd? = null
    private val AD_TIME_OUT = 4000

    //是否强制跳转到主页面
    private var mForceGoMain = false
    private var isClickAgree = false


    override fun initContentView(savedInstanceState: Bundle?) = R.layout.activity_splash

    override fun initVariableId() = BR.viewModel

    override fun getStatusBarHeightView()=binding.viewSplash

    override fun initData() {
        super.initData()
        //点击Home键后再点击App图标，会回到原来的界面。今天重新打包后，但是在Debug模式下，一切正常，
        // 但是把打完包的apk（Release）安装在其他手机上时，回到桌面后点击图标会重新打开启动页
        if (!isTaskRoot) {
            finish()
            return
        }

//        StatusBarUtil.immersive(this)

        //校验App签名
        checkAppSign()
    }

    override fun onResume() {
        if (isClickAgree) {
            //判断是否该跳转到主页面
            if (mForceGoMain) {
                goToMainActivity()
            }
        }

        super.onResume()
    }

    override fun onPause() {
        super.onPause()
    }

    override fun onStop() {
        super.onStop()
        mForceGoMain = true
    }

    override fun onDestroy() {
        super.onDestroy()
        mTTSplashAd?.destroy()
        mTTSplashAd = null

//        msplash?.removeAllViews()
//        msplash = null
    }

    /**
     * 校验App签名
     */
    private fun checkAppSign() {
        val deviceInfo: DeviceInfo = DeviceInfo.instance
        if (!deviceInfo.signInfo) {
            finish()
        } else {
//            showPowerDialog()
            isClickAgree = true
            try {
                msplash = findViewById(R.id.fl_splash)
                //加载开屏广告
                loadSplashAd()
//                goToMainActivity()
            } catch (e: Exception) {
                goToMainActivity()
            }
        }
    }

    /**
     * 加载开屏广告
     */
    private fun loadSplashAd() {
        /**
         * 注：每次加载开屏广告的时候需要新建一个TTSplashAd，否则可能会出现广告填充问题
         * （ 例如：mTTSplashAd = new TTSplashAd(this, mAdUnitId);）
         */
        mTTSplashAd = TTSplashAd(this, CsjGroMoreManager.AD_UNIT_SPLASH_ID)
        mTTSplashAd?.setTTAdSplashListener(mSplashAdListener)

        //step3:创建开屏广告请求参数AdSlot,具体参数含义参考文档
        val adSlot = AdSlot.Builder()
                .setImageAdSize(1080, 1920) // 既适用于原生类型，也适用于模版类型。
                .build()

        //自定义兜底方案 选择使用
        val ttNetworkRequestInfo: TTNetworkRequestInfo
        //穿山甲兜底
        ttNetworkRequestInfo = PangleNetworkRequestInfo(CsjGroMoreManager.TT_AD_APPID, CsjGroMoreManager.AD_CSJ_SPLASH_ID)
        //step4:请求广告，调用开屏广告异步请求接口，对请求回调的广告作渲染处理
        mTTSplashAd?.loadAd(adSlot, ttNetworkRequestInfo, object : TTSplashAdLoadCallback {
            //        mTTSplashAd.loadAd(adSlot, new TTSplashAdLoadCallback() {
            override fun onSplashAdLoadFail(adError: AdError) {
                goToMainActivity()
            }

            override fun onSplashAdLoadSuccess() {
                mTTSplashAd?.showAd(msplash)
            }

            override fun onAdLoadTimeout() {
                goToMainActivity()
            }
        }, AD_TIME_OUT)
    }

    var mSplashAdListener: TTSplashAdListener = object : TTSplashAdListener {
        override fun onAdClicked() {
        }

        override fun onAdShow() {
        }

        override fun onAdSkip() {
            goToMainActivity()
        }

        override fun onAdDismiss() {
            goToMainActivity()
        }
    }

    /**
     * 跳转到主页面
     */
    private fun goToMainActivity() {
        if (MMKV.mmkvWithID(Constant.FIRST_OPEN_PERMISSIONS).decodeBool(Constant.FIRST_OPEN_PERMISSIONS, false)) {
            // 非新用户
            startActivityThenKill(MainActivity::class.java)
        } else {
            startActivityThenKill(PermissionsActivity::class.java)
        }
    }

    private fun showPowerDialog() {
        if (MMKV.mmkvWithID(Constant.LOAN_PERMISSION_FLAG).decodeBool(Constant.LOAN_PERMISSION_FLAG, true)) {
            powerDialog()
        } else {
            isClickAgree = true
            try {
                msplash = findViewById(R.id.fl_splash)
                //加载开屏广告
                loadSplashAd()
            } catch (e: Exception) {
                goToMainActivity()
            }
        }
    }

    /**
     * 权限弹窗
     */
    private fun powerDialog() {
        powerDialog = PowerDialog(this, object : DialogListener() {
            override fun onClick(v: View) {
                when (v.id) {
                    R.id.tv_dialogper_agreement -> {
                        MintsWebViewActivity.startWebView(name = "用户协议", url = Constant.REGISTER_URL)

                    }
                    R.id.tv_dialogper_policy -> {
                        MintsWebViewActivity.startWebView(name = "隐私协议", url = Constant.PRIVACY_URL)
                    }
                    R.id.btn_dialogper_back -> {
                        if (powerDialog != null && powerDialog!!.isShowing) {
                            ToastUtils.showLong("请您同意授权,否则将无法使用APP功能")
                        }
                    }
                    R.id.btn_dialogper_next -> {
                        if (powerDialog != null && powerDialog!!.isShowing) {
                            powerDialog!!.dismiss()

                            try {
                                msplash = findViewById(R.id.fl_splash)
                                //加载开屏广告
                                loadSplashAd()
                            } catch (e: Exception) {
                                goToMainActivity()
                            }

                            isClickAgree = true

                            MMKV.mmkvWithID(Constant.LOAN_PERMISSION_FLAG).encode(Constant.LOAN_PERMISSION_FLAG, false)
                        }
                    }
                }
            }
        })
        powerDialog?.show()
    }
}