package com.mints.street.widget.dialog;

import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ProgressBar;
import android.widget.TextView;


import com.mints.street.R;
import com.mints.street.eventbean.DownloadEvent;

import java.text.DecimalFormat;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Consumer;
import me.goldze.mvvmhabit.bus.RxBus;
import me.goldze.mvvmhabit.bus.RxSubscriptions;
import me.goldze.mvvmhabit.utils.Utils;

/**
 * @author 冯瑞雨
 * @date 2019/10/23 14:54
 */
public class DownloadDialog extends DialogFragment {

    private TextView tvDownloadSchedule, tvDownloadSize, tvApkSize;
    private ProgressBar progressBar;
    private Disposable mSubscribe;
    private DecimalFormat df = new DecimalFormat("#.00");
    private double apkSize;


    public static DownloadDialog newInstance() {
        Bundle args = new Bundle();
        DownloadDialog fragment = new DownloadDialog();
        fragment.setArguments(args);

        return fragment;
    }

    @Override
    public void onStart() {
        super.onStart();
        Window window = getDialog().getWindow();
        window.setBackgroundDrawable(new ColorDrawable());
        DisplayMetrics dm = new DisplayMetrics();
        getActivity().getWindowManager().getDefaultDisplay().getMetrics(dm);
        window.setLayout(WindowManager.LayoutParams.WRAP_CONTENT, WindowManager.LayoutParams.WRAP_CONTENT);
        WindowManager.LayoutParams params = window.getAttributes();
        params.gravity = Gravity.CENTER;
        window.setAttributes(params);
        setCancelable(false);
        window.getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                | View.SYSTEM_UI_FLAG_LAYOUT_STABLE);
//        window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
        window.setStatusBarColor(Color.TRANSPARENT);

    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View inflate = inflater.inflate(R.layout.main_dialog_download_fragment, container, false);
        return inflate;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        tvDownloadSchedule = view.findViewById(R.id.tv_download_schedule);
        tvDownloadSize = view.findViewById(R.id.tv_download_size);
        progressBar = view.findViewById(R.id.pb_progress_bar);
        tvApkSize = view.findViewById(R.id.tv_apk_size);

        mSubscribe = RxBus.getDefault().toObservable(DownloadEvent.class).subscribe(new Consumer<DownloadEvent>() {
            @Override
            public void accept(DownloadEvent downloadEvent) throws Exception {
                Utils.getHandler().post(new Runnable() {
                    @Override
                    public void run() {
                        if (apkSize == 0){
                            apkSize = downloadEvent.getApkSize()/(1024f*1024f);
                            if (tvApkSize != null) {
                                tvApkSize.setText(String.format("/%sM",df.format(apkSize
                                )));
                            }
                        }
                        if (tvDownloadSchedule != null) {
                            tvDownloadSchedule.setText(String.format("下载%s%%", downloadEvent.getApkCurrentProgress()));
                        }
                        if (progressBar != null) {
                            progressBar.setProgress(downloadEvent.getApkCurrentProgress());
                        }
                        if (tvDownloadSize != null) {
                            tvDownloadSize.setText(String.format("%sM",df.format(downloadEvent.getApkSchedule()/(1024f*1024f))));
                        }
                        if (downloadEvent.getApkSize() == 0 && downloadEvent.getApkSchedule()==0){
                            tvDownloadSize.setText(String.format("%sM",df.format(apkSize)));
                        }
                    }
                });

            }
        });
        RxSubscriptions.add(mSubscribe);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        RxSubscriptions.remove(mSubscribe);
    }
}
