package com.mints.street.widget.dialog

import android.Manifest
import android.annotation.SuppressLint
import android.annotation.TargetApi
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.provider.Settings
import android.text.TextUtils
import android.view.View
import androidx.fragment.app.FragmentActivity
import com.fry.base.utils.SpannableUtils
import com.lxj.xpopup.XPopup
import com.lxj.xpopup.core.CenterPopupView
import com.lxj.xpopup.interfaces.SimpleCallback
import com.mints.street.R
import com.mints.street.bean.Version

import com.mints.street.services.UpdateServices


import com.tbruyelle.rxpermissions2.RxPermissions
import kotlinx.android.synthetic.main.main_update_dialog.view.*
import me.goldze.mvvmhabit.base.AppManager
import me.goldze.mvvmhabit.utils.KLog
import me.goldze.mvvmhabit.utils.RxUtils
import me.goldze.mvvmhabit.utils.ToastUtils
/**
 *
 * @author 冯瑞雨
 * @date 2021/7/23
 */
class UpdateDialog(val activity: FragmentActivity, val version: Version) : CenterPopupView(activity) {

    companion object {
        @JvmStatic
        fun showDialog(activity: FragmentActivity, version: Version): UpdateDialog {
            return XPopup.Builder(activity)
                .setPopupCallback(object : SimpleCallback() {
                    override fun onBackPressed(): Boolean {
                        if (!version.isForceUpgrade) { //非强制升级
                            return false
                        }
                        return true
                    }
                })
                .dismissOnBackPressed(!version.isForceUpgrade)
                .dismissOnTouchOutside(false/*!upgradeInfoBean.isForce*/)
                .asCustom(UpdateDialog(activity, version))
                .show() as UpdateDialog
        }
    }

    override fun getImplLayoutId() = R.layout.main_update_dialog

    override fun onCreate() {
        super.onCreate()
        initView()
        initClick()
    }

    private fun initView() {
        if ( version.upgrades!=null &&version.upgrades?.size?:0>0){
            version.upgrades[0]?.let {
                tv_update_content.text =it.updatemsg
//                    tv_tip.text = SpannableUtils.getGradientTxt(upgradeInfoBean.title,
//                    intArrayOf(0xFFFFDA81.toInt(), 0xFFFFF1D5.toInt(), 0xFFFFEDAA.toInt()), null)
                tv_title.text = "V${it.version}诚邀更新"
//                    intArrayOf(0xFFFFD56F.toInt(), 0xFFFFF1DA.toInt(), 0xFFFFEC98.toInt()), null)

                if (version.isForceUpgrade) { //强制升级
                    stv_download_next.visibility = View.GONE
                } else { //普通升级
                    stv_download_next.visibility = View.VISIBLE
                }
            }

        }

    }

    private fun initClick() {
        stv_download_next.setOnClickListener { this.dismiss() }
        RxUtils.onMultiClick(stv_download_now){
            permissionStorage()
        }
    }


    @SuppressLint("CheckResult")
    private fun permissionStorage() {
        //请求存储权限
        val rxPermissions = RxPermissions(activity)
        rxPermissions.request(Manifest.permission.WRITE_EXTERNAL_STORAGE
            , Manifest.permission.READ_EXTERNAL_STORAGE)
            .subscribe{ grant: Boolean ->
                if (grant) {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) { //  8.0 检测是否允许安装apk
                        val b: Boolean = activity.packageManager.canRequestPackageInstalls()
                        if (b) {
                            startDownload()
                        } else {
                            permissionInstall()
                        }
                    } else {
                        startDownload()
                    }
                } else {
                    if (version.isForceUpgrade) {
                        AppManager.getAppManager().finishAllActivity()
                    } else {
                        dismiss()
                    }
                    ToastUtils.showLong("请前往系统设置里允许读写权限")
                }
            }
    }
    /**
     * 8.0安装新版本APK权限
     */
    @TargetApi(Build.VERSION_CODES.O)
    @SuppressLint("CheckResult")
    private fun permissionInstall() {
        //请求安装权限
        val rxPermissions = RxPermissions(activity)
        rxPermissions.request(Manifest.permission.REQUEST_INSTALL_PACKAGES)
            .subscribe { grant: Boolean ->
                if (grant) {
                    startDownload()
                } else {
                    //跳转到安装未知应用界面
                    val packageURI = Uri.parse("package:" + activity.packageName)
                    val intent = Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES, packageURI)
                    activity.startActivity(intent)
                }
            }
    }

    private fun startDownload() {
        if ( version.upgrades==null ||version.upgrades?.size?:0<=0) {
            return
        }
        val upgrades = version.upgrades?.get(0)
        if (TextUtils.isEmpty(upgrades?.url)) {
            return
        }
        //打开服务
        val intent = Intent(activity, UpdateServices::class.java)
        intent.putExtra(UpdateServices.DOWNLOAD_URL_KEY, upgrades?.url)
        intent.putExtra(UpdateServices.VERSION_CODE_KEY, upgrades?.version)
        if (version.isForceUpgrade) { //强制更新
            //弹出新的进度框
            intent.putExtra(UpdateServices.IS_FORCED_UPDATE_KEY, true)
            DownloadDialog.newInstance().show(activity.supportFragmentManager, "")
        } else { //后台下载
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            activity.startForegroundService(intent)
        } else {
            activity.startService(intent)
        }
        dismiss()

    }

}