package com.mints.street.widget.dialog

import android.Manifest
import android.content.Intent
import android.graphics.Color
import android.view.LayoutInflater
import android.view.View
import android.widget.CheckBox
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.widget.LinearLayoutCompat
import androidx.fragment.app.FragmentActivity
import cn.sharesdk.framework.ShareSDK
import cn.sharesdk.wechat.friends.Wechat
import com.google.android.material.bottomsheet.BottomSheetDialog
import com.mints.street.R
import com.mints.street.bean.WXInfo
import com.mints.street.common.Constant
import com.mints.street.login.LoginActivity
import com.mints.street.login.LoginApi
import com.mints.street.login.OnLoginListener
import com.mints.street.utils.json.JsonUtil
import com.mints.street.webview.MintsWebViewActivity
import com.tbruyelle.rxpermissions2.RxPermissions
import me.goldze.mvvmhabit.utils.RxUtils
import me.goldze.mvvmhabit.utils.ToastUtils
import java.util.*

/**
 * 描述：微信登录
 * 作者：孟崔广
 * 时间：2021/7/29 10:51
 */
class WxLoginDialog(context: FragmentActivity) : OnLoginListener {

    var wxLoginListener: WxLoginListener? = null
    private var mBottomSheetDialog: BottomSheetDialog? = null
    private var context: FragmentActivity
    private val loginApi by lazy { LoginApi() }

    init {
        this.context = context
    }

    /**
     * 初始化
     */
    fun init(): WxLoginDialog {
        //创建弹窗
        if (mBottomSheetDialog != null) return this

        mBottomSheetDialog = BottomSheetDialog(context)
        val view1: View = LayoutInflater.from(context).inflate(R.layout.layout_popupwindow, null)
        mBottomSheetDialog!!.setContentView(view1)
        mBottomSheetDialog!!.window!!.findViewById<View>(R.id.design_bottom_sheet).setBackgroundColor(Color.TRANSPARENT)

        val iv_weixin: ImageView = view1.findViewById(R.id.iv_weixin)
        val ly_phone: LinearLayoutCompat = view1.findViewById(R.id.ly_phone)
        val cb_checked: CheckBox = view1.findViewById(R.id.cb_checked)
        val tv_private: TextView = view1.findViewById(R.id.tv_private)
        val tv_regist: TextView = view1.findViewById(R.id.tv_regist)

        tv_regist.setOnClickListener {
            MintsWebViewActivity.startWebView(name = "用户协议", url = Constant.REGISTER_URL)
        }
        tv_private.setOnClickListener {
            MintsWebViewActivity.startWebView(name = "隐私协议", url = Constant.PRIVACY_URL)
        }
        iv_weixin.setOnClickListener {
            if (cb_checked!!.isChecked) {
                val wechat = ShareSDK.getPlatform(Wechat.NAME)
                if (!wechat.isClientValid) {
                    ToastUtils.showShort("请先安装微信")
                    return@setOnClickListener
                }
                RxPermissions(context)
                        .request(Manifest.permission.READ_PHONE_STATE)
                        .subscribe { granted: Boolean ->
                            if (granted) {
                                loginApi.setOnLoginListener(this)
                                loginApi.setPlatform(Wechat.NAME)
                                loginApi.login(context)
                            } else {
                                DialogUtils.showPermissionDialog(context,"设备")
                            }
                        }
                mBottomSheetDialog!!.dismiss()
            } else {
                ToastUtils.showShort("请先同意用户协议与隐私政策")
            }
        }

        RxUtils.onMultiClick(ly_phone) {
            if (cb_checked!!.isChecked) {
                context.startActivity(Intent(context, LoginActivity::class.java))
                mBottomSheetDialog!!.dismiss()
            } else {
                ToastUtils.showShort("请先同意用户协议与隐私政策")
            }
        }

        return this
    }

    /**
     * 展示
     */
    fun show() {
        mBottomSheetDialog?.show()
    }

    override fun onLogin(platform: String?, res: HashMap<String, Any>?): Boolean {
        try {
            val wxInfo = WXInfo()
            wxInfo?.let {
                it.unionid = res?.get("unionid").toString() + ""
                it.openid = res?.get("openid").toString() + ""
                it.province = res?.get("province").toString() + ""
                it.city = res?.get("city").toString() + ""
                it.country = res?.get("country").toString() + ""
                it.headimgurl = res?.get("headimgurl").toString() + ""
                it.nickname = res?.get("nickname").toString() + ""
                it.sex = (res?.get("sex").toString() + "").toInt()
            }
            wxLoginListener?.loginSuc(JsonUtil.toJson(wxInfo))
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return false
    }

    fun onDestroy() {
        loginApi.setOnLoginListener(null)
        mBottomSheetDialog = null
    }

    fun setListener(listener: WxLoginListener?) {
        wxLoginListener = listener
    }

    interface WxLoginListener {
        fun loginSuc(wxInfo: String)
    }
}