package com.fry.base.adapter;

import android.content.Context;
import androidx.databinding.ViewDataBinding;
import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.ViewGroup;

import java.util.ArrayList;
import java.util.List;

/**
 * @author jeme
 * @date 2019/9/21
 */
public abstract class AbstractBindingAdapter<T extends ViewDataBinding,D> extends RecyclerView.Adapter<BindingViewHolder<T>> {

    protected Context mContext;
    protected List<D> mData;
    protected LayoutInflater mLayoutInflater;

    public AbstractBindingAdapter(Context context){
        this.mContext = context;
        mLayoutInflater = LayoutInflater.from(context);
    }
    public AbstractBindingAdapter(Context context, List<D> datas){
        this(context);
        mData = datas;
    }
    public void setData(List<D> data){
        setData(data,true);
    }
    public void setData(List<D> data,boolean isRefresh){
        if(isRefresh){
            mData = data;
        }else {
            if(mData == null){
                mData = new ArrayList<>();
            }
            mData.addAll(data);
        }
        notifyDataSetChanged();
    }


    protected abstract int getLayoutId(int viewType);
    protected abstract void onBindingView(@NonNull T binding, D item,int position);

    @NonNull
    @Override
    public BindingViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new BindingViewHolder(mLayoutInflater.inflate(getLayoutId(viewType), parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull BindingViewHolder holder, int position) {
        onBindingView((T) holder.binding,mData.get(position),position);
    }

    @Override
    public int getItemCount() {
        return mData == null ? 0 : mData.size();
    }
}
