package com.fry.base.adapter;

import android.graphics.Color;
import android.os.Parcelable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentStatePagerAdapter;
import android.view.ViewGroup;

import com.fry.base.utils.SpannableUtils;

import java.util.List;
import java.util.Locale;

/**
 * 2019-09-28 created
 *
 * @author KOM
 */
public class TabLayoutAdapter extends FragmentStatePagerAdapter {
    private List<TabName> tabNames;
    private List<Fragment> fragments;

    public TabLayoutAdapter(FragmentManager fm, List<TabName> tabNames, List<Fragment> fragments) {
        super(fm);
        this.tabNames = tabNames;
        this.fragments = fragments;
    }

    @Override
    public Fragment getItem(int position) {
        return fragments.get(position);
    }

    @Override
    public int getCount() {
        return fragments.size();
    }

    /**
     * 这个函数就是给TabLayout的Tab设定Title
     */
    @Override
    public CharSequence getPageTitle(int position) {
        TabName tabName = tabNames.get(position);
        if (tabName.count == 0) {
            return tabName.name;
        }
        String countStr = String.format(Locale.CANADA, "(%d)", tabName.count);
        return SpannableUtils.highLightText(
                tabName.name + countStr, countStr, Color.parseColor("#FF5F00"));
    }

    @Override
    public Object instantiateItem(ViewGroup container, int position) {
        return super.instantiateItem(container, position);
    }

    /**
     * java.lang.IllegalStateException: Fragment no longer exists for key f0: index 0
     */
    @Override
    public Parcelable saveState() {
        return null;
    }

    @Override
    public void destroyItem(ViewGroup container, int position, Object object) {
        //如果注释这行，那么不管怎么切换，page都不会被销毁
//        super.destroyItem(container, position, object);
    }

    public static class TabName {
        public String name;
        public int count;
        public String tag;

        public TabName(String name,int count) {
            this.count = count;
            this.name = name;
        }

        public TabName(String name) {
            this.name = name;
        }

        public TabName(String name,String tag) {
            this.name = name;
            this.tag = tag;
        }

        public TabName(String name, int count, String tag) {
            this.name = name;
            this.count = count;
            this.tag = tag;
        }
    }
}
