package com.fry.base.base;

import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;

import androidx.annotation.CallSuper;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.databinding.ViewDataBinding;
import androidx.lifecycle.Observer;

import com.fry.base.R;
import com.fry.base.ui.widget.ViewStatusLayout;
import com.umeng.analytics.MobclickAgent;

import me.goldze.mvvmhabit._enum.ViewStatus;
import me.goldze.mvvmhabit.base.BaseViewModel;
import me.goldze.mvvmhabit.base.HabitBaseActivity;
import me.goldze.mvvmhabit.utils.AppUtils;

/**
 * Created by jeme on 2019/3/4
 */
public abstract class BaseActivity<V extends ViewDataBinding, VM extends BaseViewModel> extends HabitBaseActivity<V, VM> {

    protected ViewStatusLayout mViewStatusLayout;
    private Handler mHandler;

    protected Handler getHandler(){
        if(mHandler == null){
            mHandler = new Handler(getMainLooper());
        }
        return mHandler;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        AppUtils.fixAndroidTransWindow(this);

        super.onCreate(savedInstanceState);
        if(this.getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE){
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        }
    }


    @Override
    public void setRequestedOrientation(int requestedOrientation) {
        if (Build.VERSION.SDK_INT == Build.VERSION_CODES.O && AppUtils.isTranslucentOrFloating(this)) {
            return;
        }
        super.setRequestedOrientation(requestedOrientation);
    }
    @Override
    protected View getStatusBarHeightView() {
        View title = findViewById(R.id.ctb_title);
        if(title != null){
            return title;
        }
        ViewGroup winContent = findViewById(android.R.id.content);
        if (winContent.getChildCount() > 0) {
            return winContent.getChildAt(0);
        }
        return null;
    }

    @CallSuper
    @Override
    public void initData() {
        registerUCObserver();
        initViewStatusLayout();
        super.initData();
    }
    private void registerUCObserver() {

        viewModel.getUC().getViewStatusEvent().observe(this, new Observer<Integer>() {
            @Override
            public void onChanged(@ViewStatus Integer integer) {
                if (mViewStatusLayout != null) {
                    mViewStatusLayout.switchView(integer);
                }
            }
        });
    }
    /***
     * 是否需要全局空数据页
     */
    protected boolean needViewStatus(){
        return true;
    }

    protected ViewGroup getViewStatusParent() {
        //如果指定了具体容器，则直接加载到此容器中
        ViewGroup viewStatusContainer = findViewById(R.id.view_status_container);
        if (viewStatusContainer != null) return viewStatusContainer;

        ViewGroup content = findViewById(android.R.id.content);
        if (content.getChildCount() == 0) return content;
        //rootV就是自己定义的跟布局
        View rootV = content.getChildAt(0);
        //如果子布局不是viewgroup直接加载到content布局中
        if (!(rootV instanceof ViewGroup)) return content;
        ViewGroup firstVp = (ViewGroup) rootV;
        //如果跟布局下没有别的控件，则加载到跟布局中
        if (firstVp.getChildCount() == 0) return firstVp;
        ViewParent ctb = firstVp.findViewById(R.id.ctb_title);
        if (ctb != null) {
            return createStatusView(firstVp);
        } else {
            return firstVp;
        }
    }

    private FrameLayout createStatusView(ViewGroup rootV) {
        FrameLayout container = new FrameLayout(this);
        container.setId(R.id.view_status_container);
        ViewGroup.LayoutParams lp;
        if (rootV instanceof ConstraintLayout) {
            ConstraintLayout.LayoutParams clp = new ConstraintLayout.LayoutParams(ConstraintLayout.LayoutParams.MATCH_PARENT,
                    ConstraintLayout.LayoutParams.MATCH_PARENT);
            clp.topToBottom = R.id.ctb_title;
            clp.bottomToBottom = rootV.getId();
            clp.startToStart = rootV.getId();
            lp = clp;
        } else if (rootV instanceof RelativeLayout) {
            RelativeLayout.LayoutParams rlp = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.MATCH_PARENT);
            rlp.addRule(RelativeLayout.BELOW, R.id.ctb_title);
            lp = rlp;
        } else {
            lp = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        }
        rootV.addView(container, lp);
        return container;
    }

    protected ViewStatusLayout.Builder createViewStatusBuilder() {
        return new ViewStatusLayout.Builder()
                .setNoDataTip("暂无数据")
                .setNoDataResId(R.mipmap.ic_no_data)
                .setNoNetworkTip("无网络，请先检查网络连接")
                .setNoNetworkResId(R.mipmap.ic_no_no_network);
    }

    private void initViewStatusLayout() {
        if(!needViewStatus()) {
            return;
        }
        if(mViewStatusLayout != null ) {
            return;
        }
        ViewStatusLayout.Builder builder = createViewStatusBuilder();
        if(builder.getContainer() == null){
            ViewGroup vp = getViewStatusParent();
            builder.setContainer(vp);
        }
        if (builder.getContainer() != null) {
            mViewStatusLayout = builder
                    .attach();
        }
    }

    /***
     * 不跟随系统字体大小
     */
    @Override
    public Resources getResources() {
        Resources res = super.getResources();
        if (res != null) {
            Configuration config = res.getConfiguration();
            if (config != null && config.fontScale != 1.0f) {
                config.fontScale = 1.0f;
                res.updateConfiguration(config, res.getDisplayMetrics());
            }
        }
        return res;
    }

    @Override
    public void showDialog(String title) {
        if (mViewStatusLayout != null) {
            mViewStatusLayout.showLoading(title);
        }
    }

    @Override
    public void dismissDialog() {
        if (mViewStatusLayout != null) {
            mViewStatusLayout.dismissLoading();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        // 友盟
        MobclickAgent.onResume(this);
    }

    @Override
    protected void onPause() {
        super.onPause();
        // 友盟
        MobclickAgent.onPause(this);
    }

}
