package com.fry.base.base;

import android.app.Activity;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.fragment.app.DialogFragment;

import java.lang.ref.WeakReference;

/**
 * Description : 防止内存泄漏
 * Created by yue on 2020/5/10
 */
public class BaseDialogFragment extends DialogFragment {

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        boolean isShow = getShowsDialog();
        setShowsDialog(false);
        super.onActivityCreated(savedInstanceState);
        setShowsDialog(isShow);
        if (getDialog() == null) {
            return;
        }
        View view = getView();
        if (view != null) {
            if (view.getParent() != null) {
                throw new IllegalStateException(
                        "DialogFragment can not be attached to a container view");
            }
            getDialog().setContentView(view);
        }
        final Activity activity = getActivity();
        if (activity != null) {
            getDialog().setOwnerActivity(activity);
        }
        getDialog().setCancelable(isCancelable());
        // 使用静态内部类取代，防止message中持有fragment的引用而造成内存泄漏
        getDialog().setOnCancelListener(onCancelListener);
        getDialog().setOnDismissListener(onDismissListener);
        if (savedInstanceState != null) {
            Bundle dialogState = savedInstanceState.getBundle("android:savedDialogState");
            if (dialogState != null) {
                getDialog().onRestoreInstanceState(dialogState);
            }
        }
    }

    @Override
    public void onCancel(@NonNull DialogInterface dialog) {
        super.onCancel(dialog);
    }

    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        super.onDismiss(dialog);
    }

    private final OnCancelListener onCancelListener = new OnCancelListener(this);
    public static class OnCancelListener implements DialogInterface.OnCancelListener {

        private WeakReference<BaseDialogFragment> fragmentWeakReference;

        public OnCancelListener(BaseDialogFragment fragment) {
            fragmentWeakReference = new WeakReference<>(fragment);
        }

        @Override
        public void onCancel(DialogInterface dialog) {
            if (fragmentWeakReference != null) {
                BaseDialogFragment dialogFragment = fragmentWeakReference.get();
                if (dialogFragment != null) {
                    dialogFragment.onCancel(dialog);
                }
            }
        }
    }

    private final OnDismissListener onDismissListener = new OnDismissListener(this);
    public static class OnDismissListener implements DialogInterface.OnDismissListener {

        private WeakReference<BaseDialogFragment> fragmentWeakReference;

        public OnDismissListener(BaseDialogFragment fragment) {
            fragmentWeakReference = new WeakReference<>(fragment);
        }

        @Override
        public void onDismiss(DialogInterface dialog) {
            if (fragmentWeakReference != null) {
                BaseDialogFragment dialogFragment = fragmentWeakReference.get();
                if (dialogFragment != null) {
                    dialogFragment.onDismiss(dialog);
                }
            }
        }
    }

}
