package com.fry.base.base;

import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;

import androidx.annotation.CallSuper;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.view.ScrollingView;
import androidx.databinding.ViewDataBinding;
import androidx.lifecycle.Observer;

import com.fry.base.R;
import com.fry.base.ui.widget.ViewStatusLayout;
import com.umeng.analytics.MobclickAgent;

import me.goldze.mvvmhabit._enum.ViewStatus;
import me.goldze.mvvmhabit.base.BaseViewModel;

/**
 * Created by jeme on 2019/3/4
 */
public abstract class BaseFragment<V extends ViewDataBinding, VM extends BaseViewModel> extends BaseCacheFragment<V,VM> {

    protected ViewStatusLayout mViewStatusLayout;
    @CallSuper
    @Override
    public void initData() {
        registerUCObserver();
        initViewStatusLayout(getView());
        super.initData();
    }

    private void registerUCObserver(){

        viewModel.getUC().getViewStatusEvent().observe(this, new Observer<Integer>() {
            @Override
            public void onChanged(@ViewStatus Integer integer) {
                if(mViewStatusLayout != null){
                    mViewStatusLayout.switchView(integer);
                }
            }
        });
    }

    /***
     * 是否需要全局空数据页
     */
    protected boolean needViewStatus(){
        return true;
    }

    protected ViewGroup getViewStatusParent(View view) {
        //如果指定了具体容器，则直接加载到此容器中
        ViewGroup viewStatusContainer = view.findViewById(R.id.view_status_container);
        if (viewStatusContainer != null) {
            return viewStatusContainer;
        }
        if(!(view instanceof ViewGroup) || view instanceof ScrollingView) {
            return null;
        }
        viewStatusContainer = (ViewGroup)view;
        //如果跟布局下没有别的控件，则加载到跟布局中
        if (viewStatusContainer.getChildCount() == 0) {
            return viewStatusContainer;
        }
        ViewParent ctb = viewStatusContainer.findViewById(R.id.ctb_title);
        if (ctb != null) {
            return createStatusView(viewStatusContainer);
        } else {
            return viewStatusContainer;
        }
    }
    private FrameLayout createStatusView(ViewGroup rootV) {
        if(getContext() == null) {
            return null;
        }
        FrameLayout container = new FrameLayout(getContext());
        container.setId(R.id.view_status_container);
        ViewGroup.LayoutParams lp;
        if (rootV instanceof ConstraintLayout) {
            ConstraintLayout.LayoutParams clp = new ConstraintLayout.LayoutParams(ConstraintLayout.LayoutParams.MATCH_PARENT, ConstraintLayout.LayoutParams.MATCH_PARENT);
            clp.topToBottom = R.id.ctb_title;
            clp.startToStart = rootV.getId();
            lp = clp;
        } else if (rootV instanceof RelativeLayout) {
            RelativeLayout.LayoutParams rlp = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.MATCH_PARENT);
            rlp.addRule(RelativeLayout.BELOW, R.id.ctb_title);
            lp = rlp;
        } else {
            lp = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        }
        rootV.addView(container, lp);
        return container;
    }

    protected ViewStatusLayout.Builder createViewStatusBuilder(){
        return new ViewStatusLayout.Builder()
                .setNoDataTip("暂无数据")
                .setNoDataResId(R.mipmap.ic_no_data)
                .setNoNetworkTip("网络不稳定，请重试")
                .setNoNetworkResId(R.mipmap.ic_no_no_network);
    }

    private void initViewStatusLayout(View view){
        if(!needViewStatus()) {
            return;
        }
        if(mViewStatusLayout != null ) {
            return;
        }
        ViewStatusLayout.Builder builder = createViewStatusBuilder();
        if(builder.getContainer() == null){
            ViewGroup vp = getViewStatusParent(view);
            builder.setContainer(vp);
        }
        if (builder.getContainer() != null) {
            mViewStatusLayout = builder
                    .attach();
        }
    }


    @Override
    public void showDialog(String title) {
        if (mViewStatusLayout != null) {
            mViewStatusLayout.showLoading(title);
        }
    }

    @Override
    public void dismissDialog() {
        if (mViewStatusLayout != null) {
            mViewStatusLayout.dismissLoading();
        }
    }

    /***
     * 是否需要统计
     */
    protected boolean needStatistics(){
        return true;
    }

    @CallSuper
    @Override
    public void onFirstUserVisible() {
        super.onFirstUserVisible();
        if(needStatistics()) {
            MobclickAgent.onPageStart(getClass().getName());
        }
    }
    @CallSuper
    @Override
    public void onUserVisible() {
        super.onUserVisible();
        if(needStatistics()) {
            MobclickAgent.onPageStart(getClass().getName());

        }
    }

    @CallSuper
    @Override
    public void onFirstUserInvisible() {
        super.onFirstUserInvisible();
        if(needStatistics()) {
            MobclickAgent.onPageEnd(getClass().getName());
        }
    }

    @CallSuper
    @Override
    public void onUserInvisible() {
        super.onUserInvisible();
        if(needStatistics()) {
            MobclickAgent.onPageEnd(getClass().getName());

        }
    }

}
