package com.fry.base.binding

import android.text.Editable
import android.text.TextWatcher
import android.view.View
import android.widget.EditText
import androidx.databinding.BindingAdapter

/**
 * 2020/6/26 created
 * @author KOM
 */

private val cache: MutableMap<EditText, TextWatcher> by lazy {
    LinkedHashMap<EditText, TextWatcher>()
}

/**
 * 控制光标在[EditText]文本的最后
 */
@BindingAdapter("selectionLast")
fun EditText.setSelectionLast(selectionLast: Boolean) {
    var textWatcher: TextWatcher? = cache[this]
    if (selectionLast) {
        if (textWatcher == null) {
            textWatcher = object : TextWatcher {
                override fun afterTextChanged(s: Editable?) {
                    with(this@setSelectionLast) {
                        val str = text.toString()
                        setSelection(str.length)
                    }
                }

                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
                }

                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                }

            }
            this.addTextChangedListener(textWatcher)
            cache[this] = textWatcher
            this.addOnAttachStateChangeListener(object : View.OnAttachStateChangeListener {
                override fun onViewDetachedFromWindow(v: View?) {
                    v?.run {
                        removeTextChangedListener(textWatcher)
                        cache.remove(this@setSelectionLast)
                    }
                }

                override fun onViewAttachedToWindow(v: View?) {

                }
            })
        }
    } else {
        textWatcher?.run {
            removeTextChangedListener(textWatcher)
            cache.remove(this@setSelectionLast)
        }
    }
}

/**
 * 当值为true时获取焦点，否则不做任何处理
 *
 * val requestFocusCode = ObservableField<Boolean>(false)
 * requestFocusCode.set(false) // 重置
 * requestFocusCode.set(true) // 获取焦点
 *
 * xml:
 * app:getFocus="@{viewModel.requestFocusCode}"
 */
@BindingAdapter("getFocus")
fun EditText.getFocus(isGetFocus: Boolean?) {
    if (isGetFocus == true) {
        isFocusable = true
        isFocusableInTouchMode = true
        requestFocus()
    }
}