package com.fry.base.binding.view;


import android.graphics.Color;
import android.graphics.drawable.GradientDrawable;
import android.os.Build;
import android.text.Html;
import android.text.TextUtils;
import android.view.View;
import android.widget.TextView;

import androidx.databinding.BindingAdapter;

import com.coorchice.library.SuperTextView;
import com.fry.base.utils.DensityUtils;
import com.fry.base.utils.HtmlTagHandler;


/**
 * TwinklingRefreshLayout列表刷新的绑定适配器
 */
public class ViewAdapter {

    @BindingAdapter(value = "htmlText")
    public static void setHtmlText(TextView textView, final String htmlText) {
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            textView.setText(Html.fromHtml(htmlText, Html.FROM_HTML_MODE_LEGACY, null, HtmlTagHandler.newInstantce()));
        }else{
            textView.setText(Html.fromHtml(htmlText,null, HtmlTagHandler.newInstantce()));
        }
    }
    @BindingAdapter(value = "stv_url")
    public static void setStvUrl(SuperTextView textView, final String url) {
        textView.setUrlImage(url);
    }

    @BindingAdapter(value = "stv_solid_color")
    public static void setStvSolidColor(SuperTextView textView, final int color) {
        textView.setSolid(color);
    }

    /**
     * 设置背景 shape
     */
    @BindingAdapter(value = {"color", "startColor", "endColor",
            "radius", "topLeftRadius", "topRightRadius", "bottomLeftRadius", "bottomRightRadius",
            "strokeWidth", "strokeColor"}, requireAll = false)
    public static void setBackground(View view, String color, String startColor, String endColor,
                                     float radius, float topLeftRadius, float topRightRadius,
                                     float bottomLeftRadius, float bottomRightRadius, float strokeWidth,
                                     String strokeColor) {
        try {
            GradientDrawable drawable = new GradientDrawable();
            if (!TextUtils.isEmpty(startColor) && !TextUtils.isEmpty(endColor)) {
                drawable.setColors(new int[]{Color.parseColor(startColor), Color.parseColor(endColor)});
            } else if (!TextUtils.isEmpty(color)) {
                drawable.setColor(Color.parseColor(color));
            }
            if (topLeftRadius != 0 || topRightRadius != 0 || bottomLeftRadius != 0 || bottomRightRadius != 0) {
                drawable.setCornerRadii(new float[]{
                        dip2px(topLeftRadius), dip2px(topLeftRadius),
                        dip2px(topRightRadius), dip2px(topRightRadius),
                        dip2px(bottomRightRadius), dip2px(bottomRightRadius),
                        dip2px(bottomLeftRadius), dip2px(bottomLeftRadius)
                });
            }
            if (radius > 0) {
                drawable.setCornerRadius(dip2px(radius));
            }
            if (strokeWidth > 0) {
                drawable.setStroke(DensityUtils.dip2px(strokeWidth), Color.parseColor(strokeColor));
            }
            view.setBackground(drawable);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static float dip2px(float dp) {
        if (dp == 0) return 0;
        return DensityUtils.dip2px(dp);
    }

    @BindingAdapter(value = {"visibility"})
    public static void setVisibility(View view, boolean visibility) {
        view.setVisibility(visibility? View.VISIBLE: View.GONE);
    }

    @BindingAdapter(value = {"android:background"})
    public static void setBackground(View view, final String color) {
        try {
            view.setBackgroundColor(Color.parseColor(color));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
