package com.fry.base.config;

import android.app.Application;

import com.fry.base.base.IModuleInit;

import java.util.HashMap;
import java.util.Map;

import io.reactivex.annotations.Nullable;
import me.goldze.mvvmhabit.utils.Utils;

/**
 * Created by goldze on 2018/6/21 0021.
 * 作为组件生命周期初始化的配置类，通过反射机制，动态调用每个组件初始化逻辑
 */

public class ModuleLifecycleConfig {

    /***
     * 内部类，在装载该内部类时才会去创建单例对象
     */
    private static class SingletonHolder {
        public static ModuleLifecycleConfig instance = new ModuleLifecycleConfig();
    }

    public static ModuleLifecycleConfig getInstance() {
        return SingletonHolder.instance;
    }

    private ModuleLifecycleConfig() {}


    /***
     * 缓存对象，避免两次反射
     */
    private Map<String,IModuleInit> allModule = new HashMap<>();
    //

    /***
     * 初始化组件-靠前
     * @param isMainProcess 是否主进程
     */
    public void initModuleAhead(@Nullable Application application,boolean isMainProcess) {
        for (String moduleInitName : ModuleLifecycleReflexs.initModuleNames) {
            try {
                Class<?> clazz = Class.forName(moduleInitName);
                IModuleInit init = (IModuleInit) clazz.newInstance();
                //调用初始化方法
                init.onInitAhead(application,isMainProcess);
                allModule.put(moduleInitName,init);
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
    }

    /***
     * 工作在后台
     */
    public void initModuleAsWork(@Nullable Application application,boolean isMainProcess) {
        for(Map.Entry<String,IModuleInit> entry : allModule.entrySet()){
            IModuleInit init = entry.getValue();
            Object obj = init.runBackground(application,isMainProcess);
            Utils.getHandler().post(() -> init.runMainAfterBackground(application,isMainProcess,obj));
        }
    }
    /***
     * 初始化组件-靠后
     */
    public void initModuleLow(@Nullable Application application,boolean isMainProcess) {
      for(Map.Entry<String,IModuleInit> entry : allModule.entrySet()){
          IModuleInit init = entry.getValue();
          //调用初始化方法
          init.onInitLow(application,isMainProcess);
      }
    }
}
