package com.fry.base.recycler_view

import androidx.databinding.BindingAdapter
import androidx.recyclerview.widget.RecyclerView
import kotlin.math.max

/**
 * 2020/6/21 created
 *
 * [BindingAdapter]
 *  1. value
 *      * 未定义任何值,则会根据setter方法规则来匹配BindingAdapter的函数名来调用
 *      * 定义值后,根据属性值匹配
 *  1. requireAll
 *      * false 只要value中定义的属性有一个在xml中使用了,就会调用[BindingAdapter]指向的函数
 *      * true 需全部被定义才会定义函数
 *
 * @author KOM
 */
@BindingAdapter("adapter", requireAll = false)
fun RecyclerView.setAdapter(newAdapter: VHMAdapter<*>?) {
    adapter = newAdapter
}

/**
 * @param gridCount 未设置或 <=1 时，为线性布局，否则为网格布局
 * @param isVertical true 竖直方向 false 水平方向
 */
@BindingAdapter("gridCount", "isVertical", requireAll = false)
fun RecyclerView.setLayoutManager(gridCount: Int?, isVertical: Boolean?) {
    val vertical = isVertical != false
    val crossAxisCount = max(1, gridCount ?: 1)
    layoutManager = when (crossAxisCount) {
        1 -> LayoutManagerUtil.linear(
                context,
                vertical
        )
        else -> LayoutManagerUtil.grid(
                context,
                crossAxisCount,
                vertical
        )
    }
}