package com.fry.base.ui;

import android.graphics.Canvas;
import android.graphics.LinearGradient;
import android.graphics.Paint;
import android.graphics.Shader;
import android.text.style.ReplacementSpan;

public class LinearGradientFontSpan extends ReplacementSpan {
    private int mSize;
    private int[] colors;
    private float[] points;
    private Shader.TileMode mode;


    public LinearGradientFontSpan(int[] colors,Shader.TileMode mode) {
        this(colors,null,Shader.TileMode.REPEAT);
    }
    public LinearGradientFontSpan(int[] colors, float[] points,Shader.TileMode mode) {
        this.colors = colors;
        this.points = points;
        this.mode = mode;
    }

    @Override
    public int getSize(Paint paint, CharSequence text, int start, int end, Paint.FontMetricsInt fm) {
        mSize = (int) (paint.measureText(text, start, end));

        //这段不可以去掉，字体高度没设置，可能出现draw（）方法没有被调用，如果你调用SpannableStringBuilder后append一个字符串，效果也是会出来，下面这段就不需要了
        // 原因详见https://stackoverflow.com/questions/20069537/replacementspans-draw-method-isnt-called
        Paint.FontMetricsInt metrics = paint.getFontMetricsInt();
        if (fm != null) {
            fm.top = metrics.top;
            fm.ascent = metrics.ascent;
            fm.descent = metrics.descent;
            fm.bottom = metrics.bottom;
        }

        return mSize;
    }

    @Override
    public void draw(Canvas canvas, CharSequence text, int start, int end, float x, int top, int y, int bottom, Paint paint) {
        LinearGradient lg = new LinearGradient(0, 0, 0, paint.descent() - paint.ascent(),
                colors,points,
                mode);
        paint.setShader(lg);

        //绘制文字
        canvas.drawText(text, start, end, x, y, paint);
    }

}