package com.fry.base.ui.dialog;

import android.os.Bundle;
import android.os.Parcel;
import android.os.Parcelable;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.ColorInt;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.fry.base.R;


/**
 * 选择对话框.
 * 1.内容+确定按钮。[默认]
 * 2.title、cancel按钮可选
 * 3.确认按钮文字可修改
 */

public class ChooseDialog extends MyBaseDialog implements View.OnClickListener {

    public static final String KEY_CONTENT = "content";

    private View mLayoutTitle;
    private TextView mTvTitle, mTvContent, mTvCancel, mTvConfirm;
    private View mLineSeparateBtn;
    private Builder mBuilder;
    private OnClickChooseBtnListener onClickChooseBtnListener;

    private static ChooseDialog getInstance(@NonNull Builder builder) {
        Bundle args = new Bundle();
        args.putParcelable(KEY_CONTENT, builder);
        ChooseDialog fragment = new ChooseDialog();
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mBuilder = getArguments().getParcelable(KEY_CONTENT);
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater,
                             @Nullable Bundle savedInstanceState,
                             ViewGroup container) {
        return inflater.inflate(R.layout.dialog_choose, container, false);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        assignViews(view);
        builderUI();
    }

    private void assignViews(View rootView) {
        mLayoutTitle = rootView.findViewById(R.id.layout_title);
        mTvTitle = (TextView) rootView.findViewById(R.id.tv_title);
        mTvContent = (TextView) rootView.findViewById(R.id.tv_content);
        mTvConfirm = (TextView) rootView.findViewById(R.id.tv_confirm);
        mTvCancel = (TextView) rootView.findViewById(R.id.tv_cancel);
        mLineSeparateBtn = rootView.findViewById(R.id.line_separate_btn);
        mTvConfirm.setOnClickListener(this);
        mTvCancel.setOnClickListener(this);
    }

    private void builderUI() {
        String title = mBuilder.getTitle();
        if (TextUtils.isEmpty(title)) {
            mLayoutTitle.setVisibility(View.GONE);
        } else {
            mLayoutTitle.setVisibility(View.VISIBLE);
            mTvTitle.setText(title);
        }

        String content = mBuilder.getContent();
        if (mBuilder.getContentGravity() == -1) {
            if (!TextUtils.isEmpty(content)
                    && (content.contains("\r")
                    || content.contains("\n"))) {
                mTvContent.setGravity(Gravity.START);
            } else {
                mTvContent.setGravity(Gravity.CENTER);
            }
        } else {
            mTvContent.setGravity(mBuilder.getContentGravity());
        }

        if (!TextUtils.isEmpty(content))
            mTvContent.setText(content);

        String confirmBtnText = mBuilder.getConfirmBtnText();
        if (TextUtils.isEmpty(confirmBtnText)) {
            mLineSeparateBtn.setVisibility(View.GONE);
            mTvConfirm.setVisibility(View.GONE);
        } else {
            mTvConfirm.setText(confirmBtnText);
            if (mBuilder.getConfirmTextColor() != -1) {
                mTvConfirm.setTextColor(mBuilder.getConfirmTextColor());
            }
        }

        String cancelBtnText = mBuilder.getCancelBtnText();
        if (TextUtils.isEmpty(cancelBtnText)) {
            mLineSeparateBtn.setVisibility(View.GONE);
            mTvCancel.setVisibility(View.GONE);
        } else {
            mTvCancel.setText(cancelBtnText);
        }
    }

    @Override
    public void onClick(View view) {
        int id = view.getId();
        if (id == R.id.tv_confirm) {
            dismiss();
            if (onClickChooseBtnListener != null) {
                onClickChooseBtnListener.onClickConfirm();
            }
        } else if (id == R.id.tv_cancel) {
            dismiss();
            if (onClickChooseBtnListener != null) {
                onClickChooseBtnListener.onClickCancel();
            }
        }
    }

    public ChooseDialog setOnClickChooseBtnListener(OnClickChooseBtnListener onClickChooseBtnListener) {
        this.onClickChooseBtnListener = onClickChooseBtnListener;
        return this;
    }


    public interface OnClickChooseBtnListener {
        void onClickCancel();

        void onClickConfirm();
    }

    public static class Builder implements Parcelable {
        private String title;
        private String content;
        private int contentGravity = -1;
        private String confirmBtnText = "确定";
        @ColorInt
        private int confirmTextColor = -1;
        private String cancelBtnText;

        public Builder(String content) {
            this.content = content;
        }

        public String getTitle() {
            return title;
        }

        public Builder setTitle(String title) {
            this.title = title;
            return this;
        }

        public String getContent() {
            return content;
        }

        public int getContentGravity() {
            return contentGravity;
        }

        public Builder setContentGravity(int contentGravity) {
            this.contentGravity = contentGravity;
            return this;
        }

        public String getConfirmBtnText() {
            return confirmBtnText;
        }

        public Builder setConfirmBtnText(String confirmBtnText) {
            this.confirmBtnText = confirmBtnText;
            return this;
        }

        public int getConfirmTextColor() {
            return confirmTextColor;
        }

        public Builder setConfirmTextColor(int confirmTextColor) {
            this.confirmTextColor = confirmTextColor;
            return this;
        }

        public String getCancelBtnText() {
            return cancelBtnText;
        }

        public Builder setCancelBtnText(String cancelBtnText) {
            this.cancelBtnText = cancelBtnText;
            return this;
        }

        public ChooseDialog builder() {
            return ChooseDialog.getInstance(this);
        }

        @Override
        public int describeContents() {
            return 0;
        }

        @Override
        public void writeToParcel(Parcel dest, int flags) {
            dest.writeString(this.title);
            dest.writeString(this.content);
            dest.writeInt(this.contentGravity);
            dest.writeString(this.confirmBtnText);
            dest.writeInt(this.confirmTextColor);
            dest.writeString(this.cancelBtnText);
        }

        protected Builder(Parcel in) {
            this.title = in.readString();
            this.content = in.readString();
            this.contentGravity = in.readInt();
            this.confirmBtnText = in.readString();
            this.confirmTextColor = in.readInt();
            this.cancelBtnText = in.readString();
        }

        public static final Creator<Builder> CREATOR = new Creator<Builder>() {
            @Override
            public Builder createFromParcel(Parcel source) {
                return new Builder(source);
            }

            @Override
            public Builder[] newArray(int size) {
                return new Builder[size];
            }
        };
    }
}
