package com.fry.base.ui.dialog;

import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import androidx.annotation.AnimatorRes;
import androidx.annotation.CallSuper;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.FrameLayout;

/**
 * 2019-09-28 created
 *
 * @author KOM
 */
public abstract class MyBaseDialog extends DialogFragment {

    @AnimatorRes
    private int dialogAnim = -1;
    private float dimAmount = 0.5f;//背景昏暗度
    private boolean isCanCancelForBackKey = true;
    private boolean isCanceledOnTouchOutside = true;
    protected FrameLayout layoutContent;
    private OnDismissListener onDismissListener;
    protected Context context;

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        this.context = context;
    }

    @Override
    @NonNull
    public Context getContext() {
        return context;
    }

    @Nullable
    @Override
    public final View onCreateView(LayoutInflater inflater,
                                   @Nullable ViewGroup container,
                                   @Nullable Bundle savedInstanceState) {
        layoutContent = new FrameLayout(getContext());
        layoutContent.setSystemUiVisibility(View.VISIBLE);
        FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT);
        layoutContent.setLayoutParams(layoutParams);
        View childView = onCreateView(inflater, savedInstanceState, layoutContent);
        if (childView != null) {
            layoutContent.addView(childView);
            if (!layoutContent.hasOnClickListeners()) {
                childView.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                    }
                });
            }
        }
        return layoutContent;
    }


    public abstract View onCreateView(LayoutInflater inflater,
                                      @Nullable Bundle savedInstanceState,
                                      ViewGroup container);

    @CallSuper
    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        getDialog().requestWindowFeature(Window.FEATURE_NO_TITLE);
        getDialog().setOnKeyListener(new DialogInterface.OnKeyListener() {
            @Override
            public boolean onKey(DialogInterface dialogInterface, int keyCode, KeyEvent keyEvent) {
                if (keyCode == KeyEvent.KEYCODE_BACK
                        && keyEvent.getAction() == KeyEvent.ACTION_DOWN
                        && !isCanCancelForBackKey) {
                    return true;
                }
                return false;
            }
        });
        getDialog().setCanceledOnTouchOutside(isCanceledOnTouchOutside);
        layoutContent.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isCanceledOnTouchOutside) {
                    dismiss();
                }
            }
        });
    }

    @Override
    public void onStart() {
        super.onStart();
        Window window = getDialog().getWindow();
        WindowManager.LayoutParams params = window.getAttributes();
        if (params != null) {
            params.dimAmount = dimAmount;
            params.width = WindowManager.LayoutParams.MATCH_PARENT;
            params.height = WindowManager.LayoutParams.MATCH_PARENT;
            if (dialogAnim != -1) {
                params.windowAnimations = dialogAnim;
            }
            window.setAttributes(params);
        }
        window.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
    }

    @Override
    public void show(FragmentManager manager, String tag) {
        if (!isAdded()) {
            if (manager.findFragmentByTag(tag) == null) {
                FragmentTransaction ft = manager.beginTransaction();
                ft.add(this, tag);
                ft.commitAllowingStateLoss();
            }
        }
    }

    @Override
    public void dismiss() {
        if (onDismissListener != null)
            onDismissListener.onDismiss();
        dismissAllowingStateLoss();
    }

    public MyBaseDialog setDimAmount(float dimAmount) {
        this.dimAmount = dimAmount;
        return this;
    }

    public MyBaseDialog setCanCancelForBackKey(boolean canCancelForBackKey) {
        isCanCancelForBackKey = canCancelForBackKey;
        return this;
    }

    public MyBaseDialog setCanceledOnTouchOutside(boolean canceledOnTouchOutside) {
        isCanceledOnTouchOutside = canceledOnTouchOutside;
        return this;
    }

    public MyBaseDialog setDialogAnim(int dialogAnim) {
        this.dialogAnim = dialogAnim;
        return this;
    }

    public void setOnDismissListener(OnDismissListener onDismissListener) {
        this.onDismissListener = onDismissListener;
    }

    public interface OnDismissListener {
        void onDismiss();
    }
}
