package com.fry.base.ui.widget

import android.content.Context
import android.util.AttributeSet
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.widget.EditText
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.TextView
import com.fry.base.R
import com.fry.base.utils.ImageLoader
import me.goldze.mvvmhabit.utils.AppUtils

/**
 * @author 冯瑞雨
 * @date 2020/2/23 14:57
 */
class CommonSettingItem @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null) : FrameLayout(context, attrs) {
    private val mName: TextView
    private val mWrite: TextView
    private val mReturn: ImageView
    private val mDividing: View
    private val mInputBox: EditText
     val mIvImage: ImageView
    var isNoShowWrite = false


    init {
        LayoutInflater.from(context).inflate(R.layout.common_setting_item, this)
        mName = findViewById(R.id.tv_name)
        mWrite = findViewById(R.id.tv_write)
        mReturn = findViewById(R.id.iv_ic_return)
        mDividing = findViewById(R.id.v_dividing)
        mInputBox = findViewById(R.id.et_input_box)
        mIvImage = findViewById(R.id.iv_image)
        initAttr(context, attrs)

    }

    private fun initAttr(context: Context, attrs: AttributeSet?) {
        val typeArray = context.obtainStyledAttributes(attrs, R.styleable.CommonSettingItem)
        //设置name
        setNameSize(typeArray.getDimensionPixelSize(R.styleable.CommonSettingItem_csi_size_name, AppUtils.dp2Px(context, 18F)).toFloat())
        setNameColor(typeArray.getColor(R.styleable.CommonSettingItem_csi_color_name, -1))
        setNameTxt(typeArray.getString(R.styleable.CommonSettingItem_csi_txt_name))

        //输入框 是否展示
        var isInputBoxShow = typeArray.getBoolean(R.styleable.CommonSettingItem_csi_show_input_box, false)
        //输入框展示
        isShowInputBox(isInputBoxShow)
        if (isInputBoxShow) {
            setHintInputBoxColor(typeArray.getColor(R.styleable.CommonSettingItem_csi_hint_color_input_box, -1))
            setHintInputBoxTxt(typeArray.getString(R.styleable.CommonSettingItem_csi_hint_txt_input_box))
            setInputBoxSize(typeArray.getDimensionPixelSize(R.styleable.CommonSettingItem_csi_size_input_box, AppUtils.dp2Px(context, 12F)).toFloat())
            setInputBoxColor(typeArray.getColor(R.styleable.CommonSettingItem_csi_color_input_box, -1))
            setInputBoxTxt(typeArray.getString(R.styleable.CommonSettingItem_csi_txt_input_box))
            //右边图片展示
        }
        var isStvImage = typeArray.getBoolean(R.styleable.CommonSettingItem_csi_show_right_image, false)
        //右边的图片展示
        isShowStvImage(isStvImage)
        if (isStvImage) {
            setStvImage(typeArray.getString(R.styleable.CommonSettingItem_csi_right_image_url))
        }
        //设置右边的TextView
        isNoShowWrite = typeArray.getBoolean(R.styleable.CommonSettingItem_csi_no_show_write, false)
        isNoShowWrite(!isNoShowWrite)
        if (!isNoShowWrite) {
            setWriteSize(typeArray.getDimensionPixelSize(R.styleable.CommonSettingItem_csi_size_write, AppUtils.dp2Px(context, 18F)).toFloat())
            setWriteColor(typeArray.getColor(R.styleable.CommonSettingItem_csi_color_write, -1))
            setWriteTxt(typeArray.getString(R.styleable.CommonSettingItem_csi_txt_write))
        }
        var isShowSbSwitch = typeArray.getBoolean(R.styleable.CommonSettingItem_csi_show_sb_switch, false)
        var isSbSwitch = typeArray.getBoolean(R.styleable.CommonSettingItem_csi_sb_switch, false)
        isShowSbSwitch(isShowSbSwitch)

        //设置下边的分割线
        isShowDividing(typeArray.getBoolean(R.styleable.CommonSettingItem_csi_no_show_dividing, false))
        //设置右边的返回图片 是否展示
        isShowReturn(typeArray.getBoolean(R.styleable.CommonSettingItem_csi_no_show_return, false))

        typeArray.recycle()

    }

    fun setNameSize(size: Float) {
        if (size > 0) {
            mName.setTextSize(TypedValue.COMPLEX_UNIT_PX, size)
        }

    }

    fun setNameColor(color: Int) {
        mName.setTextColor(color)
    }

    fun setNameTxt(txt: CharSequence?) {
        mName.text = txt

    }

    fun setWriteSize(size: Float) {
        if (size > 0) {
            mWrite.setTextSize(TypedValue.COMPLEX_UNIT_PX, size)
        }

    }

    fun setWriteColor(color: Int) {
        mWrite.setTextColor(color)
    }

    fun setWriteTxt(txt: CharSequence?) {
        mWrite.text = txt

    }

    fun isShowReturn(isShow: Boolean) {
        if (isShow) {
            mReturn.visibility = View.GONE
        } else {
            mReturn.visibility = View.VISIBLE
        }

    }

    fun isNoShowWrite(isShow: Boolean) {
        if (isShow) {
            mWrite.visibility = View.VISIBLE
        } else {
            mWrite.visibility = View.GONE
        }
    }

    fun isShowDividing(isShow: Boolean) {
        if (isShow) {
            mDividing.visibility = View.GONE
        } else {
            mDividing.visibility = View.VISIBLE
        }
    }

    fun isShowInputBox(isShow: Boolean) {
        if (isShow) {
            mWrite.visibility = View.GONE
            mReturn.visibility = View.GONE
            mInputBox.visibility = View.VISIBLE
            mIvImage.visibility = View.GONE
        }
    }


    fun setHintInputBoxColor(color: Int) {
        mInputBox.setHintTextColor(color)

    }

    fun setHintInputBoxTxt(txt: CharSequence?) {
        mInputBox.setHint(txt)

    }

    fun setInputBoxSize(size: Float) {
        mInputBox.setTextSize(TypedValue.COMPLEX_UNIT_PX, size)

    }

    fun setInputBoxColor(color: Int) {
        mInputBox.setTextColor(color)
    }

    fun setInputBoxTxt(txt: CharSequence?) {
        mInputBox.setText(txt)

    }

    fun isShowStvImage(isShow: Boolean) {
        if (isShow) {
            mWrite.visibility = View.GONE
            mReturn.visibility = View.GONE
            mInputBox.visibility = View.GONE
            mIvImage.visibility = View.VISIBLE
        }

    }

    fun setStvImage(url: String?) {
        ImageLoader.ImageBuilder.with(context).setUrl(url)
                .setCornerDp(AppUtils.dp2Px(context,14f)).setTargetView(mIvImage).start()
    }

    fun isShowSbSwitch(isShow: Boolean) {
    }

}