package com.fry.base.ui.widget;

import android.app.Activity;
import android.content.Context;
import android.content.res.ColorStateList;
import android.content.res.TypedArray;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.os.Build;

import androidx.annotation.IntDef;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import android.util.AttributeSet;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import com.fry.base.R;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

import me.goldze.mvvmhabit.base.AppManager;
import me.goldze.mvvmhabit.utils.AppUtils;

/**
 * Created by jeme on 2019/2/15
 */
public class CommonTitleBar extends FrameLayout implements View.OnClickListener {

    private ImageView mTvBack;
    private ImageView mIvClose;
    private TextView mTvTitle;
    private TextView mTvRight;
    private TextView mTvExtra;
    private View mVLine;

    private IClickListener mClickListener;

    private @TitleTheme
    int mCurrentTheme;

    public CommonTitleBar(@NonNull Context context) {
        this(context, null);
    }

    public CommonTitleBar(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CommonTitleBar(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        setId(R.id.ctb_title);
        LayoutInflater.from(context).inflate(R.layout.common_title_layout, this);
//        binding = DataBindingUtil.inflate(LayoutInflater.from(context), R.layout.common_title_layout, null, false);
//        addView(this);

        mTvBack = findViewById(R.id.iv_back);
        mIvClose = findViewById(R.id.iv_close);
        mTvTitle = findViewById(R.id.tv_title);
        mTvRight = findViewById(R.id.tv_right_btn);
        mTvExtra = findViewById(R.id.tv_extra_btn);
        mVLine = findViewById(R.id.v_line);

        initAttr(context, attrs, defStyleAttr);
        mIvClose.setOnClickListener(this);
        mTvBack.setOnClickListener(this);
        mTvRight.setOnClickListener(this);
        mTvExtra.setOnClickListener(this);
    }

    private void initAttr(Context context, AttributeSet attrs, int defStyleAttr) {
        if (isInEditMode()) {
            setTitle("自定义标题");
        }
        if (attrs == null) {
            return;
        }
        TypedArray typedArray = context.getTheme().obtainStyledAttributes(attrs, R.styleable.CommonTitleBar, defStyleAttr, 0);
        //设置整体布局，如单独设置控件大小，颜色，则展示单独设置的属性
        mCurrentTheme = typedArray.getInteger(R.styleable.CommonTitleBar_ctb_theme, TitleTheme.LIGHT);
        switchTheme(mCurrentTheme);

        //设置返回按钮
        boolean isVisiable = typedArray.getBoolean(R.styleable.CommonTitleBar_ctb_show_back, true);
        mTvBack.setVisibility(isVisiable ? VISIBLE : GONE);
        isVisiable = typedArray.getBoolean(R.styleable.CommonTitleBar_ctb_show_close, false);
        mIvClose.setVisibility(isVisiable ? VISIBLE : GONE);

        String backTitle = typedArray.getString(R.styleable.CommonTitleBar_ctb_back_title);
        int resId = typedArray.getResourceId(R.styleable.CommonTitleBar_ctb_back_icon, 0);
//        setBackTitle(backTitle);
        setBackIcon(resId);

        //设置标题
        String title = typedArray.getString(R.styleable.CommonTitleBar_ctb_title);
        int size = typedArray.getDimensionPixelSize(R.styleable.CommonTitleBar_ctb_title_size, AppUtils.dp2Px(context, 18));
        int color = typedArray.getColor(R.styleable.CommonTitleBar_ctb_title_color, -1);
        setTitle(title);
        setTitleSize(size);
        if (color != -1) {
            setTitleColor(color);
        }

        //设置右边按钮
        isVisiable = typedArray.getBoolean(R.styleable.CommonTitleBar_ctb_show_right, false);
        String rightTxt = typedArray.getString(R.styleable.CommonTitleBar_ctb_right_txt);
        size = typedArray.getDimensionPixelSize(R.styleable.CommonTitleBar_ctb_right_size, AppUtils.dp2Px(context, 16));
        color = typedArray.getColor(R.styleable.CommonTitleBar_ctb_right_color, mCurrentTheme == TitleTheme.DARK ? 0xffffff : 0x333333);
        resId = typedArray.getResourceId(R.styleable.CommonTitleBar_ctb_right_icon, 0);
        showRight(isVisiable);
        setRightTxt(rightTxt);
        setRightTxtSize(size);
        setRightTxtColor(color);
        setRightTxtIcon(resId);

        //设置附加按钮
        isVisiable = typedArray.getBoolean(R.styleable.CommonTitleBar_ctb_show_extra, false);
        String extraTxt = typedArray.getString(R.styleable.CommonTitleBar_ctb_extra_txt);
        size = typedArray.getDimensionPixelSize(R.styleable.CommonTitleBar_ctb_extra_size, AppUtils.dp2Px(context, 16));
        color = typedArray.getColor(R.styleable.CommonTitleBar_ctb_extra_color, mCurrentTheme == TitleTheme.DARK ? 0xffffff : 0x333333);
        resId = typedArray.getResourceId(R.styleable.CommonTitleBar_ctb_extra_icon, 0);
        showExtra(isVisiable);
        setExtraTxt(extraTxt);
        setExtraTxtSize(size);
        setExtraTxtColor(color);
        setExtraTxtIcon(resId);
        boolean showLine = typedArray.getBoolean(R.styleable.CommonTitleBar_ctb_show_line, false);
        showLine(showLine);

        typedArray.recycle();
    }

    public void setClickListener(IClickListener listener) {
        mClickListener = listener;
    }

    private void switchTheme(@TitleTheme int themeType) {
        switch (themeType) {
            case TitleTheme.DARK://黑色字体
                int blackColor = Color.parseColor("#000000");
                int blackColor1 = Color.parseColor("#333333");
                setTitleColor(blackColor);
                setRightTxtColor(blackColor1);
                setExtraTxtColor(blackColor1);
                setBackColor(blackColor1);
                break;
            case TitleTheme.LIGHT://白色字体

                int whiteColor = Color.parseColor("#ffffff");
                setTitleColor(whiteColor);
                setRightTxtColor(whiteColor);
                setExtraTxtColor(whiteColor);
                setBackColor(whiteColor);

                break;
        }
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
    }

    public void showLine(boolean showLine) {
        mVLine.setVisibility(showLine ? VISIBLE : GONE);
    }

    public void showBack(boolean showBack) {
        mTvBack.setVisibility(showBack ? VISIBLE : GONE);
    }

    /* public void setBackTitle(CharSequence title) {
         mTvBack.setText(title);
     }*/
    public void setBackColor(int color) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            mTvBack.setImageTintList(ColorStateList.valueOf(color));
        }
    }

    public void setBackIcon(int drawableId) {
        if (drawableId != 0) {
            mTvBack.setImageResource(drawableId);
        }
//        setDrawable(mTvBack, drawableId);
    }

    public void setTitle(CharSequence title) {
        mTvTitle.setText(title);
    }

    public void setTitleSize(int size) {
        if (size > 0) {
            mTvTitle.setTextSize(TypedValue.COMPLEX_UNIT_PX, size);
        }
    }

    public void setTitleColor(int color) {
        mTvTitle.setTextColor(color);
    }

    public void setRightTxt(CharSequence right) {
        mTvRight.setText(right);
    }

    public void setRightTxtSize(int size) {
        if (size > 0) {
            mTvRight.setTextSize(TypedValue.COMPLEX_UNIT_PX,size);
        }
    }

    public void setRightTxtColor(int color) {
        mTvRight.setTextColor(color);
    }

    public void setRightTxtIcon(int drawableId) {
        setDrawable(mTvRight, drawableId);
    }

    public void setExtraTxt(CharSequence right) {
        mTvExtra.setText(right);
    }

    public void setExtraTxtSize(int size) {
        if (size > 0) {
            mTvExtra.setTextSize(TypedValue.COMPLEX_UNIT_PX,size);
        }
    }

    public void setExtraTxtColor(int color) {
        mTvExtra.setTextColor(color);
    }

    public void setExtraTxtIcon(int drawableId) {
        setDrawable(mTvExtra, drawableId);
    }

    private void setDrawable(TextView view, int drawableId) {
        if (drawableId > 0) {
            Drawable dra = getResources().getDrawable(drawableId);
            dra.setBounds(0, 0, dra.getMinimumWidth(), dra.getMinimumHeight());
            view.setCompoundDrawables(dra, null, null, null);
        }
    }

    public void showClose(boolean show) {
        mIvClose.setVisibility(show ? VISIBLE : GONE);
    }


    public boolean isShowClose() {
        return mIvClose.getVisibility() == VISIBLE;
    }

    public void showRight(boolean show) {
        mTvRight.setVisibility(show ? VISIBLE : GONE);
    }

    public void showExtra(boolean show) {
        mTvExtra.setVisibility(show ? VISIBLE : GONE);
    }

    @Override
    public void onClick(View v) {
        if (mClickListener == null) {
            if (v.getId() == R.id.iv_back) {
                if (AppManager.getAppManager().getCurrentActivity() != null) {
                    Activity activity = AppManager.getAppManager().getCurrentActivity();
                    if (activity != null) {
                        activity.finish();
                    }
                }
            }
            return;
        }
        int vId = v.getId();
        if (vId == R.id.iv_close) {
            mClickListener.onCloseClick(v);
        } else if (vId == R.id.tv_extra_btn) {
            mClickListener.onExtraClick(v);
        } else if (vId == R.id.tv_right_btn) {
            mClickListener.onRightClick(v);
        } else if (vId == R.id.iv_back) {
            mClickListener.onBackClick(v);
        }
    }

    @IntDef({TitleTheme.LIGHT, TitleTheme.DARK})
    @Retention(RetentionPolicy.SOURCE)
    public @interface TitleTheme {
        int LIGHT = 0;
        int DARK = 1;
    }

    public interface IClickListener {
        void onCloseClick(View view);

        void onBackClick(View view);

        void onRightClick(View view);

        void onExtraClick(View view);
    }

    public static class SampleClickListenerImpl implements IClickListener {

        private void finish(){
            if (!AppManager.getActivityStack().empty()) {
                Activity activity = AppManager.getActivityStack().peek();
                if (activity != null) {
                    activity.finish();
                }
            }
        }

        @Override
        public void onBackClick(View view) {
            // 若子类传入 null 则认为是子类自己实现了 点击逻辑
            if (view == null) {
                return;
            }
            Activity activity = AppManager.getAppManager().getCurrentActivity();
            if (activity != null) {
                activity.finish();
            }
        }

        @Override
        public void onCloseClick(View view) {
            finish();
        }

        @Override
        public void onRightClick(View view) {

        }

        @Override
        public void onExtraClick(View view) {

        }
    }
}
