package com.fry.base.ui.widget;

import android.content.Context;
import android.content.res.TypedArray;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.util.AttributeSet;
import com.fry.base.R;
import me.goldze.mvvmhabit.binding.viewadapter.recyclerview.FlowLayoutManager;


/**
 * 自定义recycleview，主要用于实现布局，分割线
 * Created by jeme on 2017/11/27.
 */

public class CustomRecycleView extends RecyclerView {
    private int mSpanCount;
    public CustomRecycleView(Context context) {
        this(context, null);
    }

    public CustomRecycleView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CustomRecycleView(Context context, @Nullable AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init(context, attrs);
    }

    private void init(Context context, @Nullable AttributeSet attrs) {
        TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.CustomRecycleView);
        int layoutManagerType = a.getInteger(R.styleable.CustomRecycleView_layoutManagerType, 0);
        int spanCount = a.getInteger(R.styleable.CustomRecycleView_spanCount, 3);
        boolean canScroll = a.getBoolean(R.styleable.CustomRecycleView_canScroll, true);
        boolean divider = a.getBoolean(R.styleable.CustomRecycleView_useDivider, false);
        int dividerColor = a.getColor(R.styleable.CustomRecycleView_dividerColor, -1);
        int dividerHeight = (int) (a.getDimension(R.styleable.CustomRecycleView_dividerHeight, 3));
        int dividerResId = a.getResourceId(R.styleable.CustomRecycleView_dividerResId, -1);
//        int dividerMarginStart = (int)(a.getDimension(R.styleable.CustomRecycleView_dividerResId,0));
//        int dividerMarginEnd = (int)(a.getDimension(R.styleable.CustomRecycleView_dividerResId,0));
        mSpanCount = spanCount;


        DividerItemDecoration.DividerDec dividerDec = new DividerItemDecoration.DividerDec(
                a.getBoolean(R.styleable.CustomRecycleView_useStartDivider, false),
                a.getBoolean(R.styleable.CustomRecycleView_useTopDivider, false),
                a.getBoolean(R.styleable.CustomRecycleView_useEndDivider, false),
                a.getBoolean(R.styleable.CustomRecycleView_useBottomDivider, false)
        );
        initLayoutManager(layoutManagerType, canScroll, spanCount);
        initDivider(divider, layoutManagerType, dividerColor, dividerHeight, dividerResId, dividerDec);
        a.recycle();
    }

    private void initLayoutManager(int layoutManagerType, boolean canScroll, int spanCount) {
        setNestedScrollingEnabled(canScroll);
        switch (layoutManagerType) {
            case 0:
                setLayoutManager(new LinearLayoutManager(getContext(), LinearLayoutManager.VERTICAL, false){
                    @Override
                    public void onLayoutChildren(Recycler recycler, State state) {
                        //解决java.lang.IndexOutOfBoundsException: Inconsistency detected.的bug
                        try {
                            super.onLayoutChildren(recycler, state);
                        } catch (IndexOutOfBoundsException e) {
                            e.printStackTrace();
                        }
                    }
                });
                break;
            case 1:
                setLayoutManager(new LinearLayoutManager(getContext(), LinearLayoutManager.HORIZONTAL, false){
                    @Override
                    public void onLayoutChildren(Recycler recycler, State state) {
                        //解决java.lang.IndexOutOfBoundsException: Inconsistency detected.的bug
                        try {
                            super.onLayoutChildren(recycler, state);
                        } catch (IndexOutOfBoundsException e) {
                            e.printStackTrace();
                        }
                    }
                });
                break;
            case 2:
                setLayoutManager(new GridLayoutManager(getContext(), spanCount){
                    @Override
                    public void onLayoutChildren(Recycler recycler, State state) {
                        //解决java.lang.IndexOutOfBoundsException: Inconsistency detected.的bug
                        try {
                            super.onLayoutChildren(recycler, state);
                        } catch (IndexOutOfBoundsException e) {
                            e.printStackTrace();
                        }
                    }
                });
                break;
            case 3:
                setLayoutManager(new FlowLayoutManager());
                break;
        }
    }

    /***
     * 在setLayoutManager后面调用
     * @param divider 是否需要分割线
     * @param layoutManagerType 分割线方向
     */
    private void initDivider(boolean divider, int layoutManagerType, int dividerColor, int dividerHeight, int dividerResId, DividerItemDecoration.DividerDec dividerDec) {
        if (!divider) {
            return;
        }
        //如果分割线不是偶数，则修改为偶数
        if(dividerHeight % 2 != 0){
            dividerHeight -= 1;
        }
        DividerItemDecoration dividerItemDecoration = null;
        switch (layoutManagerType) {
            case 0:
                dividerItemDecoration = getDividerItemDecoration(DividerItemDecoration.HORIZONTAL_LIST, dividerColor, dividerHeight, dividerResId,dividerDec);
                break;
            case 1:
                dividerItemDecoration = getDividerItemDecoration(DividerItemDecoration.VERTICAL_LIST, dividerColor, dividerHeight, dividerResId,dividerDec);
                break;
            case 2:
            case 3:
                dividerItemDecoration = getDividerItemDecoration(DividerItemDecoration.BOTH_SET, dividerColor, dividerHeight, dividerResId,dividerDec);
//                GridDividerItemDecoration itemDecoration = new GridDividerItemDecoration(getResources().getDrawable(dividerResId),getResources().getDrawable(dividerResId),mSpanCount);
//                addItemDecoration(itemDecoration);
                break;
            default:
                break;
        }

        if (dividerItemDecoration != null) {
            addItemDecoration(dividerItemDecoration);
        }
    }

    private DividerItemDecoration getDividerItemDecoration(int orientation, int dividerColor, int dividerHeight, int dividerResId,DividerItemDecoration.DividerDec dividerDec) {
        DividerItemDecoration dividerItemDecoration;
        if (dividerResId > 0) {
            dividerItemDecoration = new DividerItemDecoration(getContext(), orientation, dividerResId, dividerDec);
        } else if (dividerHeight > 0) {
            dividerItemDecoration = new DividerItemDecoration(getContext(), orientation, dividerHeight, dividerColor,dividerDec);
        } else {
            dividerItemDecoration = new DividerItemDecoration(getContext(), orientation,dividerDec);
        }
        return dividerItemDecoration;
    }

    public void moveToPosition(int n) {
        if(!(getLayoutManager() instanceof LinearLayoutManager)){
            return;
        }
        LinearLayoutManager layoutManager = (LinearLayoutManager)getLayoutManager();

        //先从RecyclerView的LayoutManager中获取第一项和最后一项的Position
        int firstItem = layoutManager.findFirstVisibleItemPosition();
        int lastItem = layoutManager.findLastVisibleItemPosition();
        //然后区分情况
        if (n <= firstItem ){
            //当要置顶的项在当前显示的第一个项的前面时
            scrollToPosition(n);
        }else if ( n <= lastItem ){
            //当要置顶的项已经在屏幕上显示时
            int top = getChildAt(n - firstItem).getTop();
            scrollBy(0, top);
        }else{
            //当要置顶的项在当前显示的最后一项的后面时
            scrollToPosition(n);
            //这里这个变量是用在RecyclerView滚动监听里面的
//            move = true;
        }

    }
}
