package com.fry.base.utils;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.provider.MediaStore;
import android.text.TextUtils;

import androidx.annotation.NonNull;

import java.io.File;
import java.io.FileNotFoundException;

import me.goldze.mvvmhabit.utils.KLog;
import me.goldze.mvvmhabit.utils.Utils;

/**
 * 2019-09-30 created
 *
 * @author KOM
 */
public class AlbumUtils {

    public static String updateAlbum(@NonNull File file) {
        String urlStr = insertImageToAlbum(file);
        if (TextUtils.isEmpty(urlStr)) {
            return null;
        }
        Uri uri = Uri.parse(urlStr);
//        broadAlbum(file);
        return getRealPathFromUri(Utils.getContext(), uri);
        //因为插入图片到相册后，相册会更新，无需通知系统去扫描
        //        broadAlbum(file);
    }
    public static Uri insertImageAlbum(@NonNull File file) {
        String urlStr = insertImageToAlbum(file);
        if (TextUtils.isEmpty(urlStr)) {
            return null;
        }
        return Uri.parse(urlStr);
//        broadAlbum(file);
    }

    /***
     * 通知相册有新图片
     */
    public static void broadAlbum(File file) {
        Utils.getContext().sendBroadcast(new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE,
                Uri.fromFile(file)));
    }

    public static String insertImageToAlbum(@NonNull File file) {
        try {
            String path = MediaStore.Images.Media.insertImage(Utils.getContext().getContentResolver(),
                    file.getAbsolutePath(), file.getName(), null);
            broadAlbum(file);
            //将指定文件复制一份到系统相册目录
            return path;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } finally {
            //目前发现华为手机，插入相册时，不会复制图片
//            file.delete();
        }
        return null;
    }

    /***
     * 跳转到图片选择
     */
    public static void openAlbum(Activity activity, int requestCode) {
        Intent intent = new Intent(Intent.ACTION_PICK);
        intent.setType("image/*");
        activity.startActivityForResult(intent, requestCode);
    }

    public static String getRealPathFromUri(Context context, Uri contentUri) {
        Cursor cursor = null;
        try {
            KLog.i("getRealPathFromUri: " + contentUri);
            String[] proj = {MediaStore.Images.Media.DATA};
            cursor = context.getContentResolver().query(contentUri, proj, null, null, null);
            if (cursor != null && cursor.getColumnCount() > 0) {
                cursor.moveToFirst();
                int column_index = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA);
                String path = cursor.getString(column_index);
                KLog.i("getRealPathFromUri: column_index=" + column_index + ", path=" + path);
                return path;
            } else {
                KLog.w("getRealPathFromUri: invalid cursor=" + cursor + ", contentUri=" + contentUri);
            }
        } catch (Exception e) {
            KLog.e("getRealPathFromUri failed: " + e + ", contentUri=" + contentUri, e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return "";
    }
}
