package com.fry.base.utils;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;

import androidx.core.content.FileProvider;

import java.io.File;

/**
 * 2020/5/31 created
 * > 为了提高私有文件的安全性，防止私有文件的元数据泄漏，面向 Android 7.0 或更高版本的**应用私有目被限制访问录**。
 * > 所以从 Android 7.0 开始，禁用*file://*URI，而是需要通过FileProvider生成*content://*URI去授予临时访问权限
 * > 的方式实现应用间共享。否则系统会直接抛出 *FileUriExposedException* 异常， 应用出现故障。
 * https://blog.csdn.net/lmj623565791/article/details/72859156
 *
 * @author KOM
 */
public class FileProvider7 {

    public static Uri getUriForFile(Context context, File file) {
        Uri fileUri = null;
        if (Build.VERSION.SDK_INT >= 24) {
            fileUri = getUriForFile24(context, file);
        } else {
            fileUri = Uri.fromFile(file);
        }
        return fileUri;
    }

    public static Uri getUriForFile24(Context context, File file) {
        return FileProvider.getUriForFile(context,
                "com.base.android.android7.fileprovider",
                file);
    }


    public static Uri setIntentDataAndType(Context context,
                                           Intent intent,
                                           String type,
                                           File file,
                                           boolean writeAble) {
        Uri uri = null;
        if (Build.VERSION.SDK_INT >= 24) {
            uri = getUriForFile(context, file);
            intent.setDataAndType(uri, type);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            if (writeAble) {
                intent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
            }
        } else {
            uri = Uri.fromFile(file);
            intent.setDataAndType(uri, type);
        }
        return uri;
    }
}
