package com.fry.base.utils;

import android.content.Context;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;

/**
 * Description : Fragment 创建工具类
 * Created by yue on 2020/4/9
 */

public class FragmentUtils {

    private FragmentUtils() {
        throw new UnsupportedOperationException("u can't instantiate me...");
    }

    public static Fragment instantiate(@NonNull Context context, @NonNull String fname,
                                       @Nullable Bundle args) {
        if (context instanceof FragmentActivity) {
            FragmentActivity fragmentActivity = (FragmentActivity) context;
            Fragment f = fragmentActivity.getSupportFragmentManager().getFragmentFactory().instantiate(context.getClassLoader(), fname);
            if (args != null) {
                args.setClassLoader(f.getClass().getClassLoader());
                f.setArguments(args);
            }
            return f;
        }
        throw new RuntimeException("context not instanceof FragmentActivity");
    }

    public static <T extends Fragment> T instantiate(@NonNull Context context, @NonNull Class<T> fname,
                                       @Nullable Bundle args) {
        if (context instanceof FragmentActivity) {
            FragmentActivity fragmentActivity = (FragmentActivity) context;
            Fragment f = fragmentActivity.getSupportFragmentManager().getFragmentFactory().instantiate(context.getClassLoader(), fname.getName());
            if (args != null) {
                args.setClassLoader(f.getClass().getClassLoader());
                f.setArguments(args);
            }
            return (T)f;
        }
        throw new RuntimeException("context not instanceof FragmentActivity");
    }
}
