package com.fry.base.utils;

import android.app.Activity;

import androidx.fragment.app.Fragment;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.widget.ImageView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.RequestBuilder;
import com.bumptech.glide.RequestManager;
import com.bumptech.glide.load.MultiTransformation;
import com.bumptech.glide.load.Transformation;
import com.bumptech.glide.load.resource.bitmap.CenterCrop;
import com.bumptech.glide.load.resource.bitmap.FitCenter;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.RequestOptions;
import com.fry.base.R;

import me.goldze.mvvmhabit.utils.AppUtils;
import me.goldze.mvvmhabit.utils.CornerTransform;

/**
 * @author jeme
 * @date 2019/9.17
 */
public class ImageLoader {

    private ImageBuilder builder;

    private ImageLoader(ImageBuilder builder) {
        this.builder = builder;
    }

    private void load() {
        if (builder == null) {
            throw new RuntimeException("you need create ImageLoader with ImageBuilder");
        }
        RequestOptions options = new RequestOptions();
        Transformation tormation = null;

        if (builder.scaleType != null) {
            builder.targetView.setScaleType(builder.scaleType);
        }
        if (ImageView.ScaleType.CENTER_CROP == builder.getScaleType() && builder.getCornerDp() > 0) {
            CornerTransform cornerTransform = new CornerTransform(builder.getTargetView().getContext(), AppUtils.dp2Px(builder.getTargetView().getContext(), builder.getCornerDp()));
            cornerTransform.setExceptCorner(!builder.startTopCorner, !builder.endTopCorner, !builder.startBottomCorner, !builder.endBottomCorner);
            tormation = new MultiTransformation<>(new CenterCrop(), cornerTransform);
        } else {
            if (builder.getCornerDp() > 0) {
                CornerTransform cornerTransform = new CornerTransform(builder.getTargetView().getContext(), AppUtils.dp2Px(builder.getTargetView().getContext(), builder.getCornerDp()));
                cornerTransform.setExceptCorner(!builder.startTopCorner, !builder.endTopCorner, !builder.startBottomCorner, !builder.endBottomCorner);
                if (ImageView.ScaleType.FIT_CENTER == builder.getScaleType()) {
                    tormation = new MultiTransformation<>(new FitCenter(), cornerTransform);
                } else {
                    tormation = new MultiTransformation<>(cornerTransform);
                }

            }
        }

        options = options.placeholder(builder.getPlaceHolder() >= 0 ? builder.getPlaceHolder() : R.mipmap.empty_placeholder_small);
        RequestManager requestManager;
        if (builder.fragment != null) {
            requestManager = Glide.with(builder.fragment);
        } else if (builder.activity != null) {
            requestManager = Glide.with(builder.activity);
        } else if (builder.context != null) {
            requestManager = Glide.with(builder.context);
        } else {
            requestManager = Glide.with(this.builder.getTargetView().getContext());
        }
        RequestBuilder requestBuilder;
        //加载网络的
        if (this.builder.url!=null && !this.builder.url.equals("")){
            requestBuilder = requestManager
                    .load(this.builder.getUrl())
                    .error(builder.errorHolder)
                    .addListener(this.builder.requestListener)
                    .apply(options);
        }else {
            //加载本地的
            requestBuilder = requestManager
                    .load(this.builder.getDrawable())
                    .error(builder.errorHolder)
                    .addListener(this.builder.requestListener)
                    .apply(options);
        }

        if (tormation != null) {
            requestBuilder.transform(tormation);
        }

        requestBuilder.into(builder.getTargetView());
    }
    public static class ImageBuilder {
        //加载网络的
        private String url;
        //加载本地的
        private int drawable;
        private Fragment fragment;
        private Activity activity;
        private Context context;
        private ImageView targetView;
        private int cornerDp;
        private int placeHolder;
        private int errorHolder;
        private ImageView.ScaleType scaleType;
        private RequestListener requestListener;
        /***
         * 图片宽，配合阿里云oss使用，返回固定宽度，高度等比例的图片
         */
        private int limitWidth;
        private boolean startTopCorner = true;
        private boolean startBottomCorner = true;
        private boolean endTopCorner = true;
        private boolean endBottomCorner = true;

        public static ImageBuilder with(Fragment fragment) {
            ImageBuilder builder = new ImageBuilder();
            builder.fragment = fragment;
            return builder;
        }

        public static ImageBuilder with(Activity activity) {
            ImageBuilder builder = new ImageBuilder();
            builder.activity = activity;
            return builder;
        }

        public static ImageBuilder with(Context context) {
            ImageBuilder builder = new ImageBuilder();
            builder.context = context;
            return builder;
        }

        public static ImageBuilder with() {
            return new ImageBuilder();
        }

        public int getLimitWidth() {
            return limitWidth;
        }

        public ImageBuilder setLimitWidth(int limitWidth) {
            this.limitWidth = limitWidth;
            return this;
        }

        public String getUrl() {
            return url;
        }


        public ImageBuilder setUrl(String url) {
            this.url = url;
            return this;
        }

        public int getDrawable() {
            return drawable;
        }


        public ImageBuilder setDrawable(int drawable) {
            this.drawable = drawable;
            return this;
        }

        public int getErrorHolder() {
            return errorHolder;
        }

        public ImageBuilder setErrorHolder(int errorHolder) {
            this.errorHolder = errorHolder;
            return this;
        }

        public ImageView getTargetView() {
            return targetView;
        }

        public ImageBuilder setTargetView(ImageView targetView) {
            this.targetView = targetView;
            return this;
        }

        public int getCornerDp() {
            return cornerDp;
        }

        public ImageBuilder setCornerDp(int cornerDp) {
            this.cornerDp = cornerDp;
            return this;
        }

        public int getPlaceHolder() {
            return placeHolder;
        }

        public ImageBuilder setPlaceHolder(int placeHolder) {
            this.placeHolder = placeHolder;
            return this;
        }

        public ImageView.ScaleType getScaleType() {
            return scaleType;
        }

        public ImageBuilder setScaleType(ImageView.ScaleType scaleType) {
            this.scaleType = scaleType;
            return this;
        }

        public boolean isStartTopCorner() {
            return startTopCorner;
        }

        public ImageBuilder setStartTopCorner(boolean startTopCorner) {
            this.startTopCorner = startTopCorner;
            return this;
        }

        public boolean isStartBottomCorner() {
            return startBottomCorner;
        }

        public ImageBuilder setStartBottomCorner(boolean startBottomCorner) {
            this.startBottomCorner = startBottomCorner;
            return this;
        }

        public boolean isEndTopCorner() {
            return endTopCorner;
        }

        public ImageBuilder setEndTopCorner(boolean endTopCorner) {
            this.endTopCorner = endTopCorner;
            return this;
        }

        public boolean isEndBottomCorner() {
            return endBottomCorner;
        }

        public ImageBuilder setEndBottomCorner(boolean endBottomCorner) {
            this.endBottomCorner = endBottomCorner;
            return this;
        }

        public RequestListener getRequestListener() {
            return requestListener;
        }

        public ImageBuilder setRequestListener(RequestListener requestListener) {
            this.requestListener = requestListener;
            return this;
        }

        public void start() {
            new ImageLoader(this).load();
        }
    }
}
