package com.fry.base.utils;

import android.content.ComponentName;
import android.content.Intent;
import android.net.Uri;
import android.provider.Settings;

import me.goldze.mvvmhabit.base.AppManager;
import me.goldze.mvvmhabit.utils.KLog;
import me.goldze.mvvmhabit.utils.OSUtils;

/**
 * @author jeme
 * @date 2020/4/2
 */
public class PermissionSetting {
    /***
     * 跳转系统设置页面
     * @return
     */
    public static boolean gotoPermissionSetting() {
        boolean success = true;
        Intent intent = new Intent();
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        String packageName = AppManager.getAppManager().getCurrentActivity().getPackageName();

        if (OSUtils.isEmui()) {
            intent.putExtra("packageName", packageName);
            intent.setComponent(new ComponentName("com.huawei.systemmanager", "com.huawei.permissionmanager.ui.MainActivity"));
        } else if (OSUtils.isFlyme()) {
            intent.setAction("com.meizu.safe.security.SHOW_APPSEC");
            intent.addCategory(Intent.CATEGORY_DEFAULT);
            intent.putExtra("packageName", packageName);
        } else if (OSUtils.isMiui()) {
            String rom = OSUtils.getProp(OSUtils.KEY_VERSION_MIUI);
            if ("V6".equals(rom) || "V7".equals(rom)) {
                intent.setAction("miui.intent.action.APP_PERM_EDITOR");
                intent.setClassName("com.miui.securitycenter", "com.miui.permcenter.permissions.AppPermissionsEditorActivity");
                intent.putExtra("extra_pkgname", packageName);
            } else if ("V8".equals(rom) || "V9".equals(rom)) {
                intent.setAction("miui.intent.action.APP_PERM_EDITOR");
                intent.setClassName("com.miui.securitycenter", "com.miui.permcenter.permissions.PermissionsEditorActivity");
                intent.putExtra("extra_pkgname", packageName);
            } else {
                intent = getAppDetailsSettingsIntent(packageName);
            }
        } else if (OSUtils.isOppo()) {
            intent.putExtra("packageName", packageName);
            intent.setComponent(new ComponentName("com.coloros.safecenter", "com.coloros.safecenter.permission.PermissionManagerActivity"));
        } else {
            intent.setAction(Settings.ACTION_SETTINGS);
            KLog.i("没有适配该机型, 跳转普通设置界面");
            success = false;
        }
        try {
            AppManager.getAppManager().getCurrentActivity().startActivity(intent);
        } catch (Exception e) {
            e.printStackTrace();
            // 跳转失败, 前往普通设置界面
            AppManager.getAppManager().getCurrentActivity().startActivity(getAppDetailsSettingsIntent(packageName));
            success = false;
            KLog.i("无法跳转权限界面, 开始跳转普通设置界面");
        }
        return success;
    }

    private static Intent getAppDetailsSettingsIntent(String packageName) {
        Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
        Uri uri = Uri.fromParts("package", packageName, null);
        intent.setData(uri);
        return intent;
    }

}
