package com.fry.base.utils;

import android.Manifest;
import android.app.Activity;
import android.content.ContentValues;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.provider.MediaStore;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.core.content.FileProvider;

import com.yalantis.ucrop.UCrop;
import com.yalantis.ucrop.UCropActivity;

import java.io.File;

/**
 * 拍照工具类
 *
 * @author 冯瑞雨
 * @date 2019/10/25 14:51
 */
public class PhotographUtils {

    /**
     * 系统相机申请
     */
    public static final int CAMERA_PERMISSIONS_REQUEST_CODE = 204;
    /**
     * 系统相册
     */
    public static final int REQUEST_ALBUM = 205;
    /**
     * 拍照
     */
    public static final int REQUEST_TAKE_PHOTO_CODE = 203;
    /**
     * 拍照路径
     */
    public static Uri mImageUri;

    public static File cameraSavePath;
    /**
     * 是否是Android 10以上手机
     */
    public static boolean isAndroidQ = Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q;


    /**
     * 自动获取相机权限
     */
    public static void autoObtainCameraPermission(Activity activity) {
        if ((ContextCompat.checkSelfPermission(activity, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED)
                || (ContextCompat.checkSelfPermission(activity, Manifest.permission.READ_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED)) {
            if (ActivityCompat.shouldShowRequestPermissionRationale(activity, Manifest.permission.CAMERA)) {

            }
            ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.CAMERA, Manifest.permission.READ_EXTERNAL_STORAGE}, CAMERA_PERMISSIONS_REQUEST_CODE);
        } else {//有权限直接调用系统相机拍照
            takePhoto(activity);
        }

    }

    /**
     * 打开相机的Intent
     */
    public static void takePhoto(Activity activity) {
        //打开相机的Intent
        Intent takePhotoIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        //这句作用是如果没有相机则该应用不会闪退，要是不加这句则当系统没有相机应用的时候该应用会闪退
        if (takePhotoIntent.resolveActivity(activity.getPackageManager()) != null) {
            if (isAndroidQ) {
                // 适配android 10
                mImageUri = createImageUri(activity);
            } else {
                cameraSavePath = new File(activity.getExternalFilesDir(null).getPath() + "/touxiang.jpg");
                fileIsExists(cameraSavePath);
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                    //第二个参数为 包名.fileprovider
                    mImageUri = FileProvider.getUriForFile(activity, "com.qjzn.ddg.fileprovider", cameraSavePath);
                    takePhotoIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                } else {
                    mImageUri = Uri.fromFile(cameraSavePath);
                }
            }
            if (mImageUri != null) {
                //将用于输出的文件Uri传递给相机
                takePhotoIntent.putExtra(MediaStore.EXTRA_OUTPUT, mImageUri);
                activity.startActivityForResult(takePhotoIntent, PhotographUtils.REQUEST_TAKE_PHOTO_CODE);
            }
        }
    }

    /**
     * 创建图片地址uri,用于保存拍照后的照片 Android 10以后使用这种方法
     */
    private static Uri createImageUri(Activity activity) {
        String status = Environment.getExternalStorageState();
        // 判断是否有SD卡,优先使用SD卡存储,当没有SD卡时使用手机存储
        if (status.equals(Environment.MEDIA_MOUNTED)) {
            return activity.getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, new ContentValues());
        } else {
            return activity.getContentResolver().insert(MediaStore.Images.Media.INTERNAL_CONTENT_URI, new ContentValues());
        }
    }


    /**
     * 判断文件是否存在
     */
    public static void fileIsExists(File file) {
        try {
            if (file.exists()) {
                file.delete();
            }
        } catch (Exception e) {

        }
    }

    //图片裁剪的方法
    public static void startCrop(Activity activity, Uri uri) {
        if (uri == null) {
            return;
        }
        UCrop.Options options = new UCrop.Options();
        //裁剪后图片保存在文件夹中
        Uri destinationUri = Uri.fromFile(new File(activity.getCacheDir(), "uCrop.jpg"));
        //第一个参数是裁剪前的uri,第二个参数是裁剪后的uri
        UCrop uCrop = UCrop.of(uri, destinationUri);
        //设置裁剪框的宽高比例
        uCrop.withAspectRatio(1, 1);
        //下面参数分别是缩放,旋转,裁剪框的比例
        options.setAllowedGestures(UCropActivity.ALL, UCropActivity.NONE, UCropActivity.ALL);
        //设置标题栏文字
        options.setToolbarTitle("移动和缩放");
        //设置裁剪网格线的宽度(我这网格设置不显示，所以没效果)
        options.setCropGridStrokeWidth(2);
        //设置裁剪框的宽度
        options.setCropFrameStrokeWidth(10);
        //设置最大缩放比例
        options.setMaxScaleMultiplier(3);
        //隐藏下边控制栏
        options.setHideBottomControls(false);
        //设置是否显示裁剪网格
        options.setShowCropGrid(true);
        // options.settOvalDimmedLayer(true);//设置是否为圆形裁剪框
        //设置是否显示裁剪边框(true为方形边框)
        options.setShowCropFrame(false);
        //标题字的颜色以及按钮颜色
        options.setToolbarWidgetColor(Color.parseColor("#ffffff"));
        //设置裁剪外颜色
        options.setDimmedLayerColor(Color.parseColor("#AA000000"));
        // 设置标题栏颜色
        options.setToolbarColor(Color.parseColor("#000000"));
        //设置状态栏颜色
        options.setStatusBarColor(Color.parseColor("#000000"));
        //设置裁剪网格的颜色
        options.setCropGridColor(Color.parseColor("#ffffff"));
        //设置裁剪框的颜色
        options.setCropFrameColor(Color.parseColor("#ffffff"));
        //设置裁剪的图片质量，取值0-100
        options.setCompressionQuality(20);
        uCrop.withOptions(options);
        uCrop.start(activity);
    }

    /**
     * 打开相册
     */
    public static void openAlbum(Activity activity) {
        Intent albumIntent = new Intent(Intent.ACTION_PICK);
        albumIntent.setDataAndType(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, "image/*");
        activity.startActivityForResult(albumIntent, REQUEST_ALBUM);
    }

}
