package com.fry.base.utils;

import android.content.ContentValues;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.provider.MediaStore;
import android.text.TextUtils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import me.goldze.mvvmhabit.utils.KLog;
import me.goldze.mvvmhabit.utils.Utils;

/**
 * @author jeme
 * @date 2020/8/24
 **/
public class PictureUtils {
    /*
     * 保存文件，文件名为当前日期
     */
    public static String saveBitmap(Bitmap bitmap, String bitName) {
        return saveBitmap(bitmap,bitName,false);
    }
    public static String saveBitmap(Bitmap bitmap, String bitName,boolean hasAlpha) {
        if(TextUtils.isEmpty(bitName)){
            bitName = System.currentTimeMillis()+"";
        }
        if(hasAlpha){
            bitName = bitName + ".png";
        }else {
            bitName = bitName + ".jpg";
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
//            boolean isTrue = saveSignImage(bitName, bitmap);
            return saveSignImage(bitName, bitmap,hasAlpha);
        }
        String fileName ;
        File file;
        String brand = Build.BRAND;
        if (brand.equalsIgnoreCase("xiaomi")) { // 小米手机brand.equals("xiaomi")
            String dir = Environment.getExternalStorageDirectory().getPath() + "/DCIM/Camera/";
            File dirF = new File(dir);
            if(!dirF.exists()){
                dirF.mkdir();
            }
            fileName = dir + bitName;
        } else if (brand.equalsIgnoreCase("Huawei")) {
            String dir = Environment.getExternalStorageDirectory().getPath() + "/DCIM/Camera/";
            File dirF = new File(dir);
            if(!dirF.exists()){
                dirF.mkdir();
            }
            fileName = dir + bitName;
        } else { // Meizu 、Oppo
            fileName = Environment.getExternalStorageDirectory().getPath() + "/DCIM/" + bitName;
        }
        KLog.v("saveBitmap brand", "" + brand);
        file = new File(fileName);
        if (file.exists()) {
            file.delete();
        }
        FileOutputStream out;
        try {
            out = new FileOutputStream(file);
            // 格式为 JPEG，照相机拍出的图片为JPEG格式的，PNG格式的不能显示在相册中
            boolean success = false;
            if(hasAlpha){
                success = bitmap.compress(Bitmap.CompressFormat.PNG, 90, out);
            }else{
                success = bitmap.compress(Bitmap.CompressFormat.JPEG, 90, out);
            }
            if (success) {
                out.flush();
                out.close();
                // 插入图库
                ContentValues values = new ContentValues();
                values.put(MediaStore.Images.Media.DATA, file.getAbsolutePath());
                if(hasAlpha){
                    values.put(MediaStore.Images.Media.MIME_TYPE, "image/png");
                }else {
                    values.put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg");
                }
                Utils.getContext().getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values);
//                String path = MediaStore.Images.Media.insertImage(Utils.getContext().getContentResolver(), file.getAbsolutePath(), bitName, null);
//
//                Uri uri = Uri.parse(path);
                // 发送广播，通知刷新图库的显示
                Utils.getContext().sendBroadcast(new Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, Uri.parse("file://" + fileName)));

                return fileName;//AlbumUtils.getRealPathFromUri(Utils.getContext(),uri);
            }
        } catch (FileNotFoundException e) {
            KLog.e("FileNotFoundException", "FileNotFoundException:" + e.getMessage().toString());
            e.printStackTrace();
        } catch (IOException e) {
            KLog.e("IOException", "IOException:" + e.getMessage().toString());
            e.printStackTrace();
        } catch (Exception e) {
            KLog.e("IOException", "IOException:" + e.getMessage().toString());
            e.printStackTrace();
        }
        return "";
    }

    //将文件保存到公共的媒体文件夹
    //这里的filepath不是绝对路径，而是某个媒体文件夹下的子路径，和沙盒子文件夹类似
    //这里的filename单纯的指文件名，不包含路径
    public static String saveSignImage(/*String filePath,*/String fileName, Bitmap bitmap,boolean hasAlpha) {
        try {
            //设置保存参数到ContentValues中
            ContentValues contentValues = new ContentValues();
            //设置文件名
            contentValues.put(MediaStore.Images.Media.DISPLAY_NAME, fileName);
            //兼容Android Q和以下版本
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                //android Q中不再使用DATA字段，而用RELATIVE_PATH代替
                //RELATIVE_PATH是相对路径不是绝对路径
                //DCIM是系统文件夹，关于系统文件夹可以到系统自带的文件管理器中查看，不可以写没存在的名字
                contentValues.put(MediaStore.Images.Media.RELATIVE_PATH, "DCIM/");
                //contentValues.put(MediaStore.Images.Media.RELATIVE_PATH, "Music/signImage");
            } else {
                contentValues.put(MediaStore.Images.Media.DATA, Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES).getPath());
            }
            //设置文件类型
            if(hasAlpha){
                contentValues.put(MediaStore.Images.Media.MIME_TYPE, "image/png");
            }else {
                contentValues.put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg");
            }
//            contentValues.put(MediaStore.Images.Media.MIME_TYPE, "image/JPEG");
            //执行insert操作，向系统文件夹中添加文件
            //EXTERNAL_CONTENT_URI代表外部存储器，该值不变
            Uri uri = Utils.getContext().getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, contentValues);
            if (uri != null) {
                //若生成了uri，则表示该文件添加成功
                //使用流将内容写入该uri中即可
                OutputStream outputStream = Utils.getContext().getContentResolver().openOutputStream(uri);
                if (outputStream != null) {
                    if(hasAlpha){
                        bitmap.compress(Bitmap.CompressFormat.PNG, 90, outputStream);
                    }else {
                        bitmap.compress(Bitmap.CompressFormat.JPEG, 90, outputStream);
                    }
//                    bitmap.compress(Bitmap.CompressFormat.JPEG, 90, outputStream);
                    outputStream.flush();
                    outputStream.close();
                }
                return uri.toString();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }
}
