package com.fry.base.utils;

import android.app.Activity;
import android.app.Service;
import android.content.ComponentName;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.content.pm.ServiceInfo;
import android.graphics.drawable.Drawable;
import androidx.annotation.ColorRes;
import androidx.annotation.DimenRes;
import androidx.annotation.DrawableRes;
import androidx.annotation.StringRes;
import androidx.core.content.ContextCompat;

import me.goldze.mvvmhabit.utils.Utils;
/**
 * 2019-09-27 created
 *
 * @author KOM
 */
public class ResourceUtils {

    private ResourceUtils() {
    }

    public static Drawable getDrawable(@DrawableRes int id) {
        return ContextCompat.getDrawable(Utils.getContext(), id);
    }

    public static int getColor(@ColorRes int id) {
        return ContextCompat.getColor(Utils.getContext(), id);
    }

    public static int getDimensionPixelSize(@DimenRes int id) {
        return Utils.getContext().getResources().getDimensionPixelSize(id);
    }

    public static String getString(@StringRes int id) {
        return Utils.getContext().getResources().getString(id);
    }

    public static String getString(@StringRes int id, Object... formatArgs) {
        return Utils.getContext().getResources().getString(id, formatArgs);
    }

    public static String getMetaDataForApplication(String key)
            throws PackageManager.NameNotFoundException {
        Context context = Utils.getContext();
        ApplicationInfo appInfo = context.getPackageManager().getApplicationInfo(context.getPackageName(),
                PackageManager.GET_META_DATA);
        return appInfo.metaData.getString(key);
    }

    public static String getMetaDataForActivity(Activity activity, String key)
            throws PackageManager.NameNotFoundException {
        ComponentName cn = new ComponentName(activity, activity.getClass());
        ActivityInfo info = Utils.getContext().getPackageManager()
                .getActivityInfo(cn, PackageManager.GET_META_DATA);
        return info.metaData.getString(key);
    }

    public static String getMetaDataForService(Service service, String key)
            throws PackageManager.NameNotFoundException {
        ComponentName cn = new ComponentName(service, service.getClass());
        ServiceInfo info = service.getPackageManager()
                .getServiceInfo(cn, PackageManager.GET_META_DATA);
        return info.metaData.getString(key);
    }

    public static String getMetaDataForReceiver(Activity activity, String key)
            throws PackageManager.NameNotFoundException {
        ComponentName cn = new ComponentName(activity, activity.getClass());
        ActivityInfo info = Utils.getContext().getPackageManager()
                .getReceiverInfo(cn, PackageManager.GET_META_DATA);
        return info.metaData.getString(key);
    }
}
