package com.fry.base.utils;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.StrictMode;

import androidx.annotation.NonNull;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import me.goldze.mvvmhabit.utils.KLog;
import me.goldze.mvvmhabit.utils.ToastUtils;

/**
 * 2020/3/3 created
 *
 * @author KOM
 */
public class ShareUtils {

    private ShareUtils() {
    }

    /**
     * 分享功能
     *
     * @param context       上下文
     * @param activityTitle Activity的名字
     * @param msgTitle      消息标题
     * @param msgText       消息内容
     * @param imgPath       图片路径，不分享图片则传null
     */
    public static void shareMsg(Context context, String activityTitle, String msgTitle, String msgText,
                                String imgPath) {
        Intent intent = new Intent(Intent.ACTION_SEND);
        if (imgPath == null || imgPath.equals("")) {
            intent.setType("text/plain"); // 纯文本
        } else {
            File f = new File(imgPath);
            if (f != null && f.exists() && f.isFile()) {
                intent.setType("image/jpg");
                Uri u = Uri.fromFile(f);
                intent.putExtra(Intent.EXTRA_STREAM, u);
            }
        }
        intent.putExtra(Intent.EXTRA_SUBJECT, msgTitle);
        intent.putExtra(Intent.EXTRA_TEXT, msgText);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(Intent.createChooser(intent, activityTitle));
    }

    /**
     * 分享功能
     *
     * @param context  上下文
     * @param imgPaths 图片路径，不分享图片则传null
     */
    public static void shareImages(Context context, @NonNull List<String> imgPaths) {
        Intent intent = new Intent(Intent.ACTION_SEND_MULTIPLE);
        ArrayList<Uri> uris = new ArrayList<>();
        for (String imgPath : imgPaths) {
            File f = new File(imgPath);
            if (f != null && f.exists() && f.isFile()) {
                Uri u = Uri.fromFile(f);
                uris.add(u);
            }
        }
        intent.setType("image/*");
        intent.putParcelableArrayListExtra(Intent.EXTRA_STREAM, uris);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(Intent.createChooser(intent, "多图分享"));
    }
    public static void shareImages(Context context, @NonNull List<String> imgPaths,int requestCode) {
        Intent intent = new Intent(Intent.ACTION_SEND_MULTIPLE);
        ArrayList<Uri> uris = new ArrayList<>();
        for (String imgPath : imgPaths) {
            File f = new File(imgPath);
            if (f != null && f.exists() && f.isFile()) {
                Uri u = Uri.fromFile(f);
                uris.add(u);
            }
        }
        intent.setType("image/*");
        intent.putParcelableArrayListExtra(Intent.EXTRA_STREAM, uris);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        if(context instanceof Activity){
            ((Activity)context).startActivityForResult(Intent.createChooser(intent, "多图分享"),requestCode);
        }

    }

    public static boolean shareImagesWX(Context context,List<String> urls) {
        if(!APPThirdPartyUtils.isAvailable(context,APPThird.WX.getPackageName())){
            ToastUtils.showShort("您尚未安装微信");
            return false;
        }
        //解决android N（>=24）系统以上分享 路径为file://时的 android.os.FileUriExposedException异常
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            StrictMode.VmPolicy.Builder builder = new StrictMode.VmPolicy.Builder();
            StrictMode.setVmPolicy(builder.build());
            builder.detectFileUriExposure();
        }
        Intent shareIntent = new Intent();
        ComponentName comp = new ComponentName(APPThird.WX.getPackageName(), "com.tencent.mm.ui.tools.ShareImgUI");
        shareIntent.setComponent(comp);
        shareIntent.setAction(Intent.ACTION_SEND_MULTIPLE);
        shareIntent.setType("image/*");
        ArrayList<Uri> uris = new ArrayList<>();
        for (String imgPath : urls) {
            if(imgPath.startsWith("content")){
                uris.add(Uri.parse(imgPath));
            }else {
                File f = new File(imgPath);
                if (f.exists() && f.isFile()) {
                    Uri u = Uri.fromFile(f);
                    uris.add(u);
                    KLog.d(u::toString);
                }
            }
        }
        shareIntent.putExtra(Intent.EXTRA_STREAM, uris);
//        shareIntent.putParcelableArrayListExtra(Intent.EXTRA_STREAM,uris);
        shareIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

        context.startActivity(shareIntent);
        return true;
    }
    public static boolean shareImagesWX(Context context,ArrayList<Uri> uris) {
        if(!APPThirdPartyUtils.isAvailable(context,APPThird.WX.getPackageName())){
            ToastUtils.showShort("您尚未安装微信");
            return false;
        }
        //解决android N（>=24）系统以上分享 路径为file://时的 android.os.FileUriExposedException异常
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            StrictMode.VmPolicy.Builder builder = new StrictMode.VmPolicy.Builder();
            StrictMode.setVmPolicy(builder.build());
            builder.detectFileUriExposure();
        }
        Intent shareIntent = new Intent();
        ComponentName comp = new ComponentName(APPThird.WX.getPackageName(), "com.tencent.mm.ui.tools.ShareImgUI");
        shareIntent.setComponent(comp);
        shareIntent.setAction(Intent.ACTION_SEND_MULTIPLE);
        shareIntent.setType("image/*");
//        ArrayList<Uri> uris = new ArrayList<>();
//        for (String imgPath : urls) {
//            File f = new File(imgPath);
//            if (f.exists() && f.isFile()) {
//                Uri u = Uri.fromFile(f);
//                uris.add(u);
//                KLog.d(u::toString);
//            }
//        }
        shareIntent.putExtra(Intent.EXTRA_STREAM, uris);
//        shareIntent.putParcelableArrayListExtra(Intent.EXTRA_STREAM,uris);
        shareIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

        context.startActivity(shareIntent);
        return true;
    }
}
