package com.fry.base.utils;


import com.fry.base.func.Func0;
import com.fry.base.func.Func2;

/**
 * 2019-10-02 created
 *
 * @author KOM
 */
public class SymbolUtils {
    /**
     * t --> 非null时,执行run函数并将run函数的结果作为nonNullRun函数的结果返回
     * t --> null时，执行run函数并将run函数的结果作为nullRun函数的结果返回
     */
    public static <T, R> R nullRun(T t, Func2<T, R> nonNullRun, Func0<R> nullRun) {
        return t != null ? nonNullRun.run(t) : nullRun.run();
    }

    /**
     * t --> 非null时,执行run函数并将run函数的结果作为nonNullRun函数的结果返回
     * t --> null时，返回null
     */
    public static <T, R> R nonNullRun(T t, Func2<T, R> run) {
        if (t != null) {
            return run.run(t);
        }
        return null;
    }

    /**
     * 如果是true,则执行trueRun,并返回将其结果做为返回值,否则返回null
     * 如果是false,则执行falseRun,并返回将其结果做为返回值,否则返回null
     */
    public static <R> R takeIf(boolean isTrue, Func0<R> trueRun, Func0<R> falseRun) {
        if (isTrue && trueRun != null) {
            return trueRun.run();
        }
        if (!isTrue && falseRun != null) {
            return falseRun.run();
        }
        return null;
    }
}
