package com.fry.base.utils;

import android.graphics.Paint;
import android.graphics.Point;
import android.text.Layout;
import android.text.StaticLayout;
import android.text.TextPaint;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.coorchice.library.utils.LogUtils;

/**
 * 2019-10-02 created
 *
 * @author KOM
 */
public class ViewUtils {

    private ViewUtils() {
    }

    public static <T extends View> T findViewById(View v, int id) {
        return (T) v.findViewById(id);
    }


    /**
     * 设置View宽高为屏幕宽
     */
    public static void setViewWidthAndHeightForScreenWidth(View view) {
        int screenWidth = DensityUtils.getWindowWidthHeight().x;
        setViewWidthAndHeight(view, screenWidth, screenWidth);
    }

    /**
     * 设置View的宽高
     */
    public static void setViewWidthAndHeight(View view, int width, int height) {
        if (view == null) return;
        ViewGroup.LayoutParams layoutParams = view.getLayoutParams();
        layoutParams.width = width;
        layoutParams.height = height;
    }

    /**
     * 检查 View 在 ViewGroup 中是否可见
     *
     * @param parent ViewGroup
     * @param target View
     * @return true 可见 false 不可见
     */
    public static boolean checkViewVisibilityInViewGroup(ViewGroup parent, View target) {
        if (parent == null || target == null) return false;
        int[] parentLocationInWindows = new int[2];
        parent.getLocationInWindow(parentLocationInWindows);
        int[] targetLocationInWindows = new int[2];
        target.getLocationInWindow(targetLocationInWindows);

        boolean isHorizontalVisibility;
        int targetWindowsLeft = targetLocationInWindows[0];
        if (targetWindowsLeft < 0) {
            isHorizontalVisibility = Math.abs(parentLocationInWindows[0] - targetWindowsLeft) < target.getMeasuredWidth();
        } else {
            isHorizontalVisibility = Math.abs(parentLocationInWindows[0] - targetWindowsLeft) < parent.getMeasuredWidth();
        }
        boolean isVerticalVisibility;
        int targetWindowsTop = targetLocationInWindows[1];
        if (targetWindowsTop < 0) {
            isVerticalVisibility = Math.abs(parentLocationInWindows[1] - targetWindowsTop) < target.getMeasuredHeight();
        } else {
            isVerticalVisibility = Math.abs(parentLocationInWindows[1] - targetWindowsTop) < parent.getMeasuredHeight();
        }

        return isHorizontalVisibility && isVerticalVisibility;
    }

    public static boolean inTheCenterPointForScreen(View target, int offsetX) {
        Point screenWH = DensityUtils.getWindowWidthHeight();
        int[] targetLocationInWindows = new int[2];
        target.getLocationInWindow(targetLocationInWindows);

        int screenCenterPointX = screenWH.x / 2;
        int screenCenterPointY = screenWH.y / 2;

        int targetCenterPointX = targetLocationInWindows[0] + target.getMeasuredWidth() / 2;
        int targetCenterPointY = targetLocationInWindows[1] + target.getMeasuredHeight() / 2;

        boolean isCenterX = targetCenterPointX >= screenCenterPointX - offsetX
                && targetCenterPointX <= screenCenterPointX + offsetX;
        boolean isCenterY = targetCenterPointY >= screenCenterPointY - offsetX
                && targetCenterPointY <= screenCenterPointY + offsetX;

        LogUtils.d("-------------------");
        LogUtils.d("targetCenterPointX:" + targetCenterPointX);
        LogUtils.d("targetCenterPointY:" + targetCenterPointY);
        LogUtils.d("screenCenterPointX:" + screenCenterPointX);
        LogUtils.d("screenCenterPointY:" + screenCenterPointY);
        LogUtils.d("isCenterX:" + isCenterX);
        LogUtils.d("isCenterY:" + isCenterY);
        LogUtils.d("inTheCenterPointForScreen:" + (isCenterX && isCenterY));
        LogUtils.d("-------------------");
        return isCenterX && isCenterY;
    }

    public static int lineCountTextView(String content, int limitWidth, TextPaint textPaint) {
        StaticLayout staticLayout = new StaticLayout(content, textPaint, limitWidth,
                Layout.Alignment.ALIGN_NORMAL, 1.1f, 0, true);
        return staticLayout.getLineCount();
    }


    /**
     * @param textView
     * @param isCenterLine true 中划线 或 false 下划线
     */
    public static void textViewDrawLine(TextView textView, boolean isCenterLine) {
        textView.getPaint().setFlags(isCenterLine
                ? Paint.STRIKE_THRU_TEXT_FLAG //中划线
                : Paint.UNDERLINE_TEXT_FLAG); //下划线
    }

    /**
     * 当所有View都不可见时，返回false，有一个可见则返回true
     * @param views
     * @return
     */
    public static boolean checkAllViewVisibility(View... views) {
        if (views != null && views.length > 0) {
            for (int i = 0, size = views.length; i < size; i++) {
                if(views[i].getVisibility() == View.VISIBLE){
                    return true;
                }
            }
        }
        return false;
    }
}
