package me.goldze.mvvmhabit._enum;

import androidx.annotation.IntDef;

import com.trello.rxlifecycle2.android.ActivityEvent;
import com.trello.rxlifecycle2.android.FragmentEvent;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

/**
 * @author jeme
 * @date 2020/2/16
 */
public class LifeViewEvent {

    @IntDef({LifeViewEnum.A_CREATE, LifeViewEnum.A_START, LifeViewEnum.A_RESUME, LifeViewEnum.A_PAUSE,
            LifeViewEnum.A_STOP, LifeViewEnum.A_DESTROY, LifeViewEnum.F_ATTACH, LifeViewEnum.F_CREATE_VIEW,
            LifeViewEnum.F_CREATE, LifeViewEnum.F_START, LifeViewEnum.F_RESUME, LifeViewEnum.F_STOP,
            LifeViewEnum.F_PAUSE, LifeViewEnum.F_DESTROY_VIEW, LifeViewEnum.F_DESTROY, LifeViewEnum.F_DETACH})
    @Retention(RetentionPolicy.SOURCE)
    public @interface LifeViewEnum {

        /***
         CREATE,
         START,
         RESUME,
         PAUSE,
         STOP,
         DESTROY
         */
        int A_CREATE = 0;
        int A_START = 1;
        int A_RESUME = 2;
        int A_PAUSE = 3;
        int A_STOP = 4;
        int A_DESTROY = 5;

        /***
         ATTACH,
         CREATE,
         CREATE_VIEW,
         START,
         RESUME,
         PAUSE,
         STOP,
         DESTROY_VIEW,
         DESTROY,
         DETACH
         */
        int F_ATTACH = 11;
        int F_CREATE = 12;
        int F_CREATE_VIEW = 13;
        int F_START = 14;
        int F_RESUME = 15;
        int F_PAUSE = 16;
        int F_STOP = 17;
        int F_DESTROY_VIEW = 18;
        int F_DESTROY = 19;
        int F_DETACH = 20;
    }


    public static Object getEvent(@LifeViewEnum int eventId) {
        switch (eventId) {

            case LifeViewEnum.A_CREATE:
                return ActivityEvent.CREATE;
            case LifeViewEnum.A_DESTROY:
                return ActivityEvent.DESTROY;
            case LifeViewEnum.A_PAUSE:
                return ActivityEvent.PAUSE;
            case LifeViewEnum.A_RESUME:
                return ActivityEvent.RESUME;
            case LifeViewEnum.A_START:
                return ActivityEvent.START;
            case LifeViewEnum.A_STOP:
                return ActivityEvent.STOP;

            case LifeViewEnum.F_ATTACH:
                return FragmentEvent.ATTACH;
            case LifeViewEnum.F_CREATE_VIEW:
                return FragmentEvent.CREATE_VIEW;
            case LifeViewEnum.F_CREATE:
                return FragmentEvent.CREATE;
            case LifeViewEnum.F_DESTROY_VIEW:
                return FragmentEvent.DESTROY_VIEW;
            case LifeViewEnum.F_DESTROY:
                return FragmentEvent.DESTROY;
            case LifeViewEnum.F_DETACH:
                return FragmentEvent.DETACH;
            case LifeViewEnum.F_PAUSE:
                return FragmentEvent.PAUSE;
            case LifeViewEnum.F_RESUME:
                return FragmentEvent.RESUME;
            case LifeViewEnum.F_START:
                return FragmentEvent.START;
            case LifeViewEnum.F_STOP:
                return FragmentEvent.STOP;
            default:
                return ActivityEvent.DESTROY;
        }
    }
}
