package me.goldze.mvvmhabit.binding.viewadapter.edittext;

import android.content.Context;
import androidx.databinding.BindingAdapter;
import android.graphics.Paint;
import android.text.Editable;
import android.text.InputFilter;
import android.text.Spanned;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.Log;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import me.goldze.mvvmhabit.binding.command.BindingCommand;
import me.goldze.mvvmhabit.utils.KLog;
import me.goldze.mvvmhabit.utils.ToastUtils;

/**
 * Created by goldze on 2017/6/16.
 */

public class ViewAdapter {
    /**
     * EditText重新获取焦点的事件绑定
     */
    @BindingAdapter(value = {"requestFocus","needSoftInput"}, requireAll = false)
    public static void requestFocusCommand(EditText editText, final boolean needRequestFocus,boolean needSoftInput) {
        KLog.d("needRequestFocus:" + needRequestFocus);
        if (needRequestFocus) {
            editText.setSelection(editText.getText().length());
            editText.requestFocus();
            if(needSoftInput) {
                InputMethodManager imm = (InputMethodManager) editText.getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.showSoftInput(editText, InputMethodManager.SHOW_IMPLICIT);
            }
        }
        editText.setFocusableInTouchMode(needRequestFocus);
    }
    @BindingAdapter(value = {"clearFocus"})
    public static void clearFocusCommand(EditText editText, final boolean clear) {

        /*if (needFocus) {
            editText.setSelection(editText.getText().length());
            editText.requestFocus();
            InputMethodManager imm = (InputMethodManager) editText.getContext().getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.showSoftInput(editText, InputMethodManager.SHOW_IMPLICIT);
        }*/
        if(clear) {
            editText.clearFocus();
        }
    }
    /**
     * 添加下划线
     * 添加中划线
     */
    @BindingAdapter(value = {"bottomLine","middleLine"}, requireAll = false)
    public static void setBottomLine(TextView editText, final Boolean needBottomLine, final Boolean needMiddleLine) {
        Paint paint = editText.getPaint();
        paint.setAntiAlias(true);//抗锯齿

        if(needBottomLine != null && needBottomLine){
            paint.setFlags(Paint. UNDERLINE_TEXT_FLAG ); //下划线
        }
        if(needMiddleLine != null && needMiddleLine){
            paint.setFlags(Paint. STRIKE_THRU_TEXT_FLAG ); //中划线
        }
    }



    /**
     * EditText输入文字改变的监听
     */
    @BindingAdapter(value = {"textChanged"}, requireAll = false)
    public static void addTextChangedListener(EditText editText, final BindingCommand<String> textChanged) {
        editText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence text, int i, int i1, int i2) {
                if (textChanged != null) {
                    textChanged.execute(text.toString());
                }
            }

            @Override
            public void afterTextChanged(Editable editable) {

            }
        });
    }

    @BindingAdapter(value = {"outLengthToast"}, requireAll = false)
    public static void setMaxLength(EditText editText, final String outLengthToast) {
        if (TextUtils.isEmpty(outLengthToast)) {
            return;
        }
        int maxLength = 0;
        List<InputFilter> filters = new ArrayList<>(Arrays.asList(editText.getFilters()));
        for (Iterator<InputFilter> it = filters.iterator(); it.hasNext(); ) {
            InputFilter filter = it.next();
            if (filter instanceof InputFilter.LengthFilter) {
                maxLength = ((InputFilter.LengthFilter) filter).getMax();
                it.remove();
            }
        }
        if (maxLength <= 0) {
            return;
        }
        filters.add(new InputFilter.LengthFilter(maxLength) {
            @Override
            public CharSequence filter(CharSequence source, int start, int end, Spanned dest, int dstart, int dend) {
                CharSequence superFilter = super.filter(source, start, end, dest, dstart, dend);
                if (superFilter != null && !superFilter.equals(source)) {
                    ToastUtils.showShort(outLengthToast);
                }
                return superFilter;
            }
        });
        editText.setFilters(filters.toArray(new InputFilter[]{}));
    }
}
