package me.goldze.mvvmhabit.binding.viewadapter.image;


import androidx.databinding.BindingAdapter;
import android.widget.ImageView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.RequestBuilder;
import com.bumptech.glide.RequestManager;
import com.bumptech.glide.load.MultiTransformation;
import com.bumptech.glide.load.Transformation;
import com.bumptech.glide.load.resource.bitmap.CenterCrop;
import com.bumptech.glide.load.resource.bitmap.FitCenter;
import com.bumptech.glide.request.RequestOptions;

import me.goldze.mvvmhabit.utils.AppUtils;
import me.goldze.mvvmhabit.utils.CornerTransform;

/**
 * Created by goldze on 2017/6/18.
 */
public final class ViewAdapter {
    @BindingAdapter(value = {"url", "placeholderRes", "viewCorner","startTopCorner","endTopCorner","startBottomCorner","endBottomCorner"}, requireAll = false)
    public static void setImageUri(ImageView imageView, String url, int placeholderRes, int viewCorner,
                                   Boolean startTopCorner,Boolean endTopCorner,Boolean startBottomCorner,Boolean endBottomCorner) {
        RequestOptions options = new RequestOptions();
        Transformation tormation = null;
        if(startTopCorner == null){
            startTopCorner = true;
        }
        if(endTopCorner == null){
            endTopCorner = true;
        }
        if(startBottomCorner == null){
            startBottomCorner = true;
        }
        if(endBottomCorner == null){
            endBottomCorner = true;
        }

        if(ImageView.ScaleType.CENTER_CROP == imageView.getScaleType() && viewCorner > 0){
            CornerTransform cornerTransform = new CornerTransform(imageView.getContext(),AppUtils.dp2Px(imageView.getContext(), viewCorner));
            cornerTransform.setExceptCorner(!startTopCorner,!endTopCorner,!startBottomCorner,!endBottomCorner);
            tormation = new MultiTransformation<>(new CenterCrop(),cornerTransform);
        }else{
            if(viewCorner > 0) {
                CornerTransform cornerTransform = new CornerTransform(imageView.getContext(),AppUtils.dp2Px(imageView.getContext(), viewCorner));
                cornerTransform.setExceptCorner(!startTopCorner,!endTopCorner,!startBottomCorner,!endBottomCorner);
                if(ImageView.ScaleType.FIT_CENTER == imageView.getScaleType()){
                    tormation = new MultiTransformation<>( new FitCenter(),cornerTransform);
                }else{
                    tormation = new MultiTransformation<>(cornerTransform);
                }

            }
        }

        options = options.placeholder(placeholderRes);
        if(url != null && !url.startsWith("http")){
            url = String.format("https:%s",url);
        }
        RequestManager requestManager = Glide.with(imageView);
        RequestBuilder requestBuilder = requestManager
                .load(url)
                .apply(options);
        if(tormation != null){
            requestBuilder.transform(tormation);
        }

        requestBuilder.into(imageView);

    }
}

