package me.goldze.mvvmhabit.utils;

import android.Manifest;
import android.app.Activity;
import android.app.ActivityManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.content.res.TypedArray;
import android.net.Uri;
import android.os.Build;
import android.provider.Settings;
import android.telephony.TelephonyManager;
import android.util.Log;

import androidx.core.content.ContextCompat;
import androidx.core.content.FileProvider;

import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.List;
import java.util.UUID;

import me.goldze.mvvmhabit.R;
import me.goldze.mvvmhabit.base.AppManager;

import static android.content.Context.ACTIVITY_SERVICE;

/**
 * Created by jeme on 2019/2/15
 */
public class AppUtils {

    /***
     * 修复8.0 问题  如果是透明页面，禁止设置屏幕方向
     */
    public static void fixAndroidTransWindow(Activity activity) {
        if (Build.VERSION.SDK_INT == Build.VERSION_CODES.O && isTranslucentOrFloating(activity)) {
            fixOrientation(activity);
        }
    }

    public static boolean isTranslucentOrFloating(Activity activity) {
        boolean isTranslucentOrFloating = false;
        try {
            int[] styleableRes = (int[]) Class.forName("com.android.internal.R$styleable").getField("Window").get(null);
            final TypedArray ta = activity.obtainStyledAttributes(styleableRes);
            Method m = ActivityInfo.class.getMethod("isTranslucentOrFloating", TypedArray.class);
            m.setAccessible(true);
            isTranslucentOrFloating = (boolean) m.invoke(null, ta);
            m.setAccessible(false);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return isTranslucentOrFloating;
    }

    private static boolean fixOrientation(Activity activity) {
        try {
            Field field = Activity.class.getDeclaredField("mActivityInfo");
            field.setAccessible(true);
            ActivityInfo o = (ActivityInfo) field.get(activity);
            o.screenOrientation = -1;
            field.setAccessible(false);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    public static boolean isTranslucentStatus(Context context) {
        int[] attrsArray = {android.R.attr.windowTranslucentStatus};
        TypedArray typedArray = context.obtainStyledAttributes(attrsArray);
        boolean translucentStatus = typedArray.getBoolean(0, false);
        Log.i("THEME", "translucentStatus: " + translucentStatus);
        typedArray.recycle();
        return translucentStatus;
    }

    public static int getStatusBarHeight(Context context) {
        Class<?> c = null;
        Object obj = null;
        Field field = null;
        int x = 0;
        try {
            c = Class.forName("com.android.internal.R$dimen");
            obj = c.newInstance();
            field = c.getField("status_bar_height");
            x = Integer.parseInt(field.get(obj).toString());
            return context.getResources().getDimensionPixelSize(x);
        } catch (Exception e) {
            Log.e("APP", "get status bar height fail: " + e);
            e.printStackTrace();
        }
        return context.getResources().getDimensionPixelSize(R.dimen.statusBar_height);
    }

    public static int dp2Px(Context context, float dp) {
        final float scale = context.getResources().getDisplayMetrics().density; //当前屏幕密度因子
        return (int) (dp * scale + 0.5f);
    }

    public static int px2Dp(Context context, float px) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (px / scale + 0.5f);
    }

    public static int sp2px(Context context, float spValue) {
        final float fontScale = context.getResources().getDisplayMetrics().scaledDensity;
        return (int) (spValue * fontScale + 0.5f);
    }


    /***
     * 调用此方法前需要判断8.0的 REQUEST_INSTALL_PACKAGES 权限
     */
    public static void installApk(Context context, String downloadApk) {
        Intent intent = new Intent(Intent.ACTION_VIEW);

        File file = new File(downloadApk);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            //适配Android Q,注意mFilePath是通过ContentResolver得到的，上述有相关代码
            Uri contentUri = FileProvider.getUriForFile(context, "com.mints.street.fileprovider", file);
            intent.setDataAndType(contentUri, "application/vnd.android.package-archive");
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            //  7.0
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            //第二个参数是你的包名+.FileProvider（你也可以直接去AndroidMenifest中把你刚才写的provider的authorities拷贝过来，Fileprovider开头大小写不限）
            Uri contentUri = FileProvider.getUriForFile(context, "com.mints.street.fileprovider", file);
            intent.setDataAndType(contentUri, "application/vnd.android.package-archive");
        } else {//7.0以下
            intent.setDataAndType(Uri.fromFile(file), "application/vnd.android.package-archive");
        }
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(intent);
    }

    /**
     * Return pseudo unique ID
     * @return ID
     */
    public static String getUniquePsuedoID() {
        // If all else fails, if the user does have lower than API 9 (lower
        // than Gingerbread), has reset their device or 'Secure.ANDROID_ID'
        // returns 'null', then simply the ID returned will be solely based
        // off their Android device information. This is where the collisions
        // can happen.
        // Thanks http://www.pocketmagic.net/?p=1662!
        // Try not to use DISPLAY, HOST or ID - these items could change.
        // If there are collisions, there will be overlapping data
        String m_szDevIDShort = "35" + (Build.BOARD.length() % 10) + (Build.BRAND.length() % 10) + (Build.CPU_ABI.length() % 10) + (Build.DEVICE.length() % 10) + (Build.MANUFACTURER.length() % 10) + (Build.MODEL.length() % 10) + (Build.PRODUCT.length() % 10);

        // Thanks to @Roman SL!
        // http://stackoverflow.com/a/4789483/950427
        // Only devices with API >= 9 have android.os.Build.SERIAL
        // http://developer.android.com/reference/android/os/Build.html#SERIAL
        // If a user upgrades software or roots their device, there will be a duplicate entry
        String serial = null;
        try {
            serial = android.os.Build.class.getField("SERIAL").get(null).toString();

            // Go ahead and return the serial for api => 9
            return new UUID(m_szDevIDShort.hashCode(), serial.hashCode()).toString();
        } catch (Exception exception) {
            // String needs to be initialized
            serial = "serial"; // some value
        }

        // Finally, combine the values we have found by using the UUID class to create a unique identifier
        return new UUID(m_szDevIDShort.hashCode(), serial.hashCode()).toString();
    }

    /***
     * 获取activity是否存在栈中
     */
    public static boolean isExistMainActivity(Class<?> activity) {
        Intent intent = new Intent(Utils.getContext(), activity);
        ComponentName cmpName = intent.resolveActivity(Utils.getContext().getPackageManager());
        boolean flag = false;
        // 说明系统中存在这个activity    
        if (cmpName != null) {
            ActivityManager am = (ActivityManager) Utils.getContext().getSystemService(ACTIVITY_SERVICE);
            List<ActivityManager.RunningTaskInfo> taskInfoList = am.getRunningTasks(10);
            // 只是拿当前运行的栈
            ActivityManager.RunningTaskInfo runningTaskInfo = taskInfoList.get(0);
            int numActivities = taskInfoList.get(0).numActivities;

            for (ActivityManager.RunningTaskInfo taskInfo : taskInfoList) {
                // 说明它已经启动了
                if (taskInfo.baseActivity.equals(cmpName)) {
                    flag = true;
                    //跳出循环，优化效率
                    break;
                }
            }
        }
        return flag;

    }

    /**
     * 包名判断是否为主进程
     *
     * @param context
     * @return
     */
    public static boolean isMainProcess(Context context) {
        return context.getPackageName().equals(getProcessName(context));
    }

    /**
     * 获取进程名称
     *
     * @param context
     * @return
     */
    public static String getProcessName(Context context) {
        ActivityManager am = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        List<ActivityManager.RunningAppProcessInfo> runningApps = am.getRunningAppProcesses();
        if (runningApps == null) {
            return null;
        }
        for (ActivityManager.RunningAppProcessInfo proInfo : runningApps) {
            if (proInfo.pid == android.os.Process.myPid()) {
                if (proInfo.processName != null) {
                    return proInfo.processName;
                }
            }
        }
        return null;
    }

    public static String getImei(Context context) {
        if (ContextCompat.checkSelfPermission(context,Manifest.permission.READ_PHONE_STATE) == PackageManager.PERMISSION_GRANTED) {
            TelephonyManager telephonyManager = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
            if(telephonyManager == null){
                return null;
            }
            if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
               return null;
            }else if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O){
                try {
                    return telephonyManager.getImei();
                }catch (Exception e){
                    e.printStackTrace();
                }
                return null;
            }else{
                return telephonyManager.getDeviceId();
            }
        }
        return null;
    }

    public static String getAndroidId(Context context){
        return Settings.System.getString(context.getContentResolver(),
                Settings.Secure.ANDROID_ID);
    }
    /**
     * 添加图片http前缀地址
     *
     * @return String
     */
    public static String isAddHttp(String imageUrl){
        if (imageUrl == null)return null;
        if (imageUrl.contains("http")){
            return imageUrl;
        }else {
            return "https:"+imageUrl;
        }

    }

    /***
     * 跳转到拨号页面，并传入电话号码
     * @param mobile 号码
     */
    public static void call(String mobile){
        Activity activity = AppManager.getAppManager().getCurrentActivity();
        if(activity != null){
            Intent intent = new Intent(Intent.ACTION_DIAL);
            Uri data = Uri.parse(String.format("tel:%s",mobile));
            intent.setData(data);
            activity.startActivity(intent);
        }
    }

}


