package me.goldze.mvvmhabit.utils

import android.content.Context
import android.text.Html
import com.lxj.xpopup.XPopup
import com.lxj.xpopup.animator.EmptyAnimator
import com.lxj.xpopup.core.BasePopupView
import com.lxj.xpopup.enums.PopupType
import com.lxj.xpopup.impl.ConfirmPopupView
import com.lxj.xpopup.interfaces.OnCancelListener
import com.lxj.xpopup.interfaces.OnConfirmListener
import com.lxj.xpopup.interfaces.OnInputConfirmListener
import me.goldze.mvvmhabit.R
import me.goldze.mvvmhabit.dialog.InputDialog
import me.goldze.mvvmhabit.dialog.MsgDialog

/**
 * 通用对话框
 * @author jeme
 * @date 2020/2/24
 */
class CommonDialogUtils {
    companion object {

        @JvmStatic
        fun showLoading(context: Context, loadingTip: String?): BasePopupView {
            return XPopup.Builder(context)
                    .customAnimator(EmptyAnimator())
                    .asLoading(loadingTip)
                    .bindLayout(R.layout.dialog_loading_layout)
                    .show()
        }

        @JvmStatic
        fun showMessage(context: Context,
                        title: String? = "",
                        content: String,
                        confirm: String = "确认",
                        cancel: String = "取消",
                        confirmListener: OnConfirmListener,
                        cancelListener: OnCancelListener? = null,
                        isCanCancel: Boolean = true): BasePopupView {
            return XPopup.Builder(context)
                    .dismissOnBackPressed(isCanCancel)
                    .dismissOnTouchOutside(isCanCancel)
                    .enableDrag(false)
                    .asConfirm(title, content, cancel, confirm, confirmListener, cancelListener, false)
                    .bindLayout(R.layout.dialog_msg_tip_layout)
                    .show()
        }

        @JvmStatic
        fun showTip(context: Context,
                    title: String = "",
                    content: String,
                    confirm: String = "确认",
                    confirmListener: OnConfirmListener,
                    isCanCancel: Boolean = true): BasePopupView {
            return XPopup.Builder(context)
                    .dismissOnBackPressed(isCanCancel)
                    .dismissOnTouchOutside(isCanCancel)
                    .enableDrag(false)
                    .asCustom(asConfirm(context,title, content, null, confirm, confirmListener, null, true))
                    .show()
        }

        /***
         * 弹出带有输入法的弹窗
         */
        @JvmStatic
        fun showInput(context: Context,
                      title: String = "",
                      content: String? = "",
                      hint: String = "",
                      confirm: String = "确认",
                      confirmListener: OnInputConfirmListener): InputDialog {
            return XPopup.Builder(context)
                    .enableDrag(false)
                    .autoOpenSoftInput(true)
                    .moveUpToKeyboard(true)
                    .asCustom(InputDialog(context, title, content, hint, confirm, confirmListener))
                    .show() as InputDialog
        }


        private fun asConfirm(context: Context,title: String?, content: String?,
                      cancelBtnText: String?, confirmBtnText: String?,
                      confirmListener: OnConfirmListener?, cancelListener: OnCancelListener?, isHideCancel: Boolean): MsgDialog? {
            val popupView = MsgDialog(context,isHideCancel)
            popupView.setTitleContent(title, content, null as String?)
            popupView.setCancelText(cancelBtnText)
            popupView.setConfirmText(confirmBtnText)
            popupView.setListener(confirmListener, cancelListener)
            popupView.bindLayout(R.layout.dialog_msg_tip_layout)
            if (isHideCancel) {
                popupView.hideCancelBtn()
            }
            return popupView
        }
    }
}