package me.goldze.mvvmhabit.utils;

import android.text.format.DateUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.concurrent.TimeUnit;

public class DateUtil {

    public static String add(String startDate, int days) {
        String dt = "";
        try {
            Date date = parseDate(startDate, "yyyy-MM-dd");
            Calendar c = Calendar.getInstance();
            c.setTime(date);
            c.add(Calendar.DATE, days);  // number of days to add
            dt = formatDate(c.getTimeInMillis());
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return dt;
    }

    public static String getTimestampString(long timestamp) {
        String format;
        if (DateUtils.isToday(timestamp)) {
            format = "HH:mm";
        } else {
            format = "yyyy/MM/dd";
        }
        return new SimpleDateFormat(format, Locale.CHINESE).format(timestamp);
    }

    public static String formatDate(int year, int month, int dayOfMonth) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(year,month,dayOfMonth);
        return formatDate(calendar.getTimeInMillis());
    }

    public static String formatDate(long timestamp) {
        String format = "yyyy-MM-dd";
        return formatDate(timestamp, format);
    }

    public static String formatDate(long timestamp, String format) {
        return new SimpleDateFormat(format, Locale.CHINESE).format(timestamp);
    }

    public static Date parseDate(String date, String format) throws ParseException {
        SimpleDateFormat formatter = new SimpleDateFormat(format,Locale.CHINESE);
        return formatter.parse(date);
    }

    public static String convertDate(String oldDate, String oldFormat, String newFormat) {
        try {
            return formatDate(parseDate(oldDate,oldFormat).getTime(),newFormat);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return "";
    }

    public static boolean compairDate(String startDate, String endDate) throws ParseException {
        return DateUtil.parseDate(startDate,"yyyy-MM-dd").after(DateUtil.parseDate(endDate, "yyyy-MM-dd"));
    }

    public static int differenceInMonths(Date d1, Date d2) {
        Calendar c1 = Calendar.getInstance();
        c1.setTime(d1);
        Calendar c2 = Calendar.getInstance();
        c2.setTime(d2);
        int diff = 0;
        if (c2.after(c1)) {
            while (c2.after(c1)) {
                c1.add(Calendar.MONTH, 1);
                if (c2.after(c1)) {
                    diff++;
                }
            }
        } else if (c2.before(c1)) {
            while (c2.before(c1)) {
                c1.add(Calendar.MONTH, -1);
                if (c1.before(c2)) {
                    diff--;
                }
            }
        }
        return diff + 1;
    }

    /**
     * 计算两个日期相差的天数
     * @param d1
     * @param d2
     * @return
     */
    public static long getDifferenceDays(Date d1, Date d2) {
        long diff = d2.getTime() - d1.getTime();
        return TimeUnit.DAYS.convert(diff, TimeUnit.MILLISECONDS);
    }

    /**
     * date是否是在当前日期往后推days天内
     * @param date
     * @param days
     * @return
     */
    public static boolean inDays(Date date, int days) {
        Calendar c1 = Calendar.getInstance();
        c1.setTime(date);

        Calendar c2 = Calendar.getInstance();
        c2.setTimeInMillis(System.currentTimeMillis());

        c2.add(Calendar.DAY_OF_MONTH,days);
        return c2.after(c1);
    }

    /**
     * 获取当前时间
     * @return
     */
    public static Date getCurrentDate() {
        Date date = new Date();
        date.setTime(System.currentTimeMillis());
        return date;
    }

    public static Date getDate(int year, int month, int dayOfMonth) {
        try {
            return parseDate(formatDate(year,month,dayOfMonth),"yyyy-MM-dd");
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return getCurrentDate();
    }

    /**
     * 判断是否为今天(效率比较高)
     * @param day 传入的 时间戳
     * @return true今天 false不是
     * @throws ParseException
     */
    public static boolean isToday(long day){
        if(day == 0){
            return false;
        }
        Calendar pre = Calendar.getInstance();
        Date predate = new Date(System.currentTimeMillis());
        pre.setTime(predate);

        Calendar cal = Calendar.getInstance();
        Date date = new Date(day);
        cal.setTime(date);

        if (cal.get(Calendar.YEAR) == (pre.get(Calendar.YEAR))) {
            int diffDay = cal.get(Calendar.DAY_OF_YEAR)
                    - pre.get(Calendar.DAY_OF_YEAR);

            if (diffDay == 0) {
                return true;
            }
        }
        return false;
    }

    public static boolean isToday(String day) throws ParseException {

        Calendar pre = Calendar.getInstance();
        Date predate = new Date(System.currentTimeMillis());
        pre.setTime(predate);

        Calendar cal = Calendar.getInstance();
        Date date = getDateFormat().parse(day);
        cal.setTime(date);

        if (cal.get(Calendar.YEAR) == (pre.get(Calendar.YEAR))) {
            int diffDay = cal.get(Calendar.DAY_OF_YEAR)
                    - pre.get(Calendar.DAY_OF_YEAR);

            if (diffDay == 0) {
                return true;
            }
        }
        return false;
    }
    private static ThreadLocal<SimpleDateFormat> DateLocal = new ThreadLocal<>();
    public static SimpleDateFormat getDateFormat() {
        if (null == DateLocal.get()) {
            DateLocal.set(new SimpleDateFormat("yyyy-MM-dd", Locale.CHINA));
        }
        return DateLocal.get();
    }
}
