package me.goldze.mvvmhabit.utils;

import android.app.Activity;
import android.graphics.Rect;
import android.view.View;
import android.view.ViewTreeObserver;

import java.lang.ref.WeakReference;

/**
 * Created by jeme on 2018/1/19.
 */

public class IKeyboard {

    private int statusBarHeight = 0;//状态栏的高度
    private int virHeight = 0;//虚拟键的高度
    private boolean initVir;//有没有获取过虚拟键的高度
    private boolean keyBoardShow;

    private KeyboardListener keyboardListener;
    private ViewTreeObserver.OnGlobalLayoutListener globalLayoutListener;
    private View decorView;

    public void registerListener(final Activity activity, KeyboardListener listener) {
        setKeyboardListener(listener);
        decorView = activity.getWindow().getDecorView();
        globalLayoutListener = registerGlobalListener();
        decorView.getViewTreeObserver().addOnGlobalLayoutListener(globalLayoutListener);
    }

    public void setKeyboardListener(KeyboardListener keyboardListener) {
        this.keyboardListener = keyboardListener;
    }

    private ViewTreeObserver.OnGlobalLayoutListener registerGlobalListener(){
        return new ViewTreeObserver.OnGlobalLayoutListener() {
            @Override
            public void onGlobalLayout() {
                int screenHeight = decorView.getHeight();
                //Log.e("TEST", "screenHeight: " + screenHeight);

                Rect rect = new Rect();
                decorView.getWindowVisibleDisplayFrame(rect);
                //Log.e("TEST", "Visible Rect: " + rect.top + " " + rect.bottom);

                if (statusBarHeight == 0) {
                    statusBarHeight = rect.top;
                }

                if (!initVir) {
                    initVir = true;
                    virHeight = screenHeight - rect.bottom;
                }

                //获得键盘高度
                int keyboardHeight = screenHeight - rect.bottom - virHeight;
                if (keyboardHeight > screenHeight / 4) {
                    //Log.e("TEST", "keyboardHeight: " + keyboardHeight);
                    if(!keyBoardShow) {
                        keyBoardShow = true;
                        if (keyboardListener != null) {
                            keyboardListener.onKeyboardUp(keyboardHeight);
                        }
                    }
                } else if(keyBoardShow) {
                    keyBoardShow = false;
                    if (keyboardListener != null) {
                        keyboardListener.onKeyboardDown();
                    }
                }
            }
        };
    }

    public void removeListener(){
        decorView.getViewTreeObserver().removeOnGlobalLayoutListener(globalLayoutListener);
    }

    public interface KeyboardListener {
        void onKeyboardUp(int height);
        void onKeyboardDown();
    }

}
