package me.goldze.mvvmhabit.utils;

import android.annotation.SuppressLint;
import android.content.Context;
import android.view.View;

import androidx.fragment.app.Fragment;

import com.jakewharton.rxbinding2.view.RxView;
import com.trello.rxlifecycle2.LifecycleProvider;
import com.trello.rxlifecycle2.LifecycleTransformer;
import com.trello.rxlifecycle2.android.ActivityEvent;
import com.trello.rxlifecycle2.android.FragmentEvent;

import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.ObservableSource;
import io.reactivex.ObservableTransformer;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.annotations.NonNull;
import io.reactivex.functions.Consumer;
import io.reactivex.functions.Function;
import io.reactivex.schedulers.Schedulers;
import me.goldze.mvvmhabit._enum.LifeViewEvent;
import me.goldze.mvvmhabit.http.ExceptionHandle;

/**
 * Created by goldze on 2017/6/19.
 * 有关Rx的工具类
 */
public class RxUtils {
    /**
     * 生命周期绑定
     *
     * @param lifecycle Activity
     */
    public static <T> LifecycleTransformer<T> bindToLifecycle(@NonNull Context lifecycle) {
        if (lifecycle instanceof LifecycleProvider) {
            return ((LifecycleProvider<T>) lifecycle).bindToLifecycle();
        } else {
            throw new IllegalArgumentException("context not the LifecycleProvider type");
        }
    }

    public static <T>LifecycleTransformer<T> bindToLifecycle(LifecycleProvider lifecycleProvider,@LifeViewEvent.LifeViewEnum int eventId) {
        return lifecycleProvider.bindUntilEvent(LifeViewEvent.getEvent(eventId));
    }

    /**
     * 生命周期绑定
     *
     * @param lifecycle Fragment
     */
    public static <T>LifecycleTransformer<T> bindToLifecycle(@NonNull Fragment lifecycle) {
        if (lifecycle instanceof LifecycleProvider) {
            return ((LifecycleProvider<T>) lifecycle).bindToLifecycle();
        } else {
            throw new IllegalArgumentException("fragment not the LifecycleProvider type");
        }
    }

    /**
     * 生命周期绑定
     *
     * @param lifecycle Fragment
     */
    public static <T>LifecycleTransformer<T> bindToLifecycle(@NonNull LifecycleProvider<T> lifecycle) {
        return lifecycle.bindToLifecycle();
    }

    /**
     * 线程调度器
     */
    public static <T> ObservableTransformer<T,T> schedulersTransformer() {
        return new ObservableTransformer<T,T>() {
            @Override
            public ObservableSource<T> apply(Observable upstream) {
                return upstream.subscribeOn(Schedulers.io())
                        .observeOn(AndroidSchedulers.mainThread());
            }
        };
    }

    public static <T> ObservableTransformer<T,T> exceptionTransformer() {

        return new ObservableTransformer<T,T>() {
            @Override
            public ObservableSource<T> apply(Observable observable) {
                return observable
//                        .map(new HandleFuc<T>())  //这里可以取出BaseResponse中的Result
                        .onErrorResumeNext(new HttpResponseFunc());
            }
        };
    }

    private static class HttpResponseFunc<T> implements Function<Throwable, Observable<T>> {
        @Override
        public Observable<T> apply(Throwable t) {
            return Observable.error(ExceptionHandle.handleException(t));
        }
    }

    /***
     * 防止重复点击
     * @param view
     * @param listener
     */
    @SuppressLint("CheckResult")
    public static void onMultiClick(View view, View.OnClickListener listener){
        RxView.clicks(view)
                .throttleFirst(1, TimeUnit.SECONDS)//1秒钟内只允许点击1次
                .subscribe(new Consumer<Object>() {
                    @Override
                    public void accept(Object object) throws Exception {
                        if (listener != null) {
                            listener.onClick(view);
                        }
                    }
                });
    }
    @SuppressLint("CheckResult")
    public static void onLongClick(View view, View.OnClickListener listener){
        RxView.longClicks(view)
                .subscribe(new Consumer<Object>() {
                    @Override
                    public void accept(Object object) throws Exception {
                        if (listener != null) {
                            listener.onClick(view);
                        }
                    }
                });
    }

    /*private static class HandleFuc<T> implements Function<Response<BaseResponse<T>>, T> {
        @Override
        public T apply(Response<BaseResponse<T>> response) {
            if(response.code() >= 200 && response.code()<300){

            }
            if (!response.isOk()) {
                throw new RuntimeException(!"".equals(response.getCode() + "" + response.getMessage()) ? response.getMessage() : "");
            }
            return response.body().getResult();
        }
    }*/

}
