package me.goldze.mvvmhabit.utils;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Parcelable;
import android.text.TextUtils;

import com.tencent.mmkv.MMKV;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * Created by goldze on 2017/5/14.
 * SharedPreferences工具类
 */
public final class SPUtils {

    private static Map<String, SPUtils> sSPMap = new HashMap<>();
    private static final String DEFAULT_KEY = "spUtils";
    public static final String POSITIONING_MAP = "positioningMap";
    private MMKV mv;

    public static void init(Application application){
        MMKV.initialize(application);
    }
    /**
     * 获取SP实例
     *
     * @return {@link SPUtils}
     */
    public static SPUtils getInstance() {
        return getInstance("");
    }
    public static SPUtils getInstance(String name) {
        String key = name;
        if(TextUtils.isEmpty(key)) {
            key = DEFAULT_KEY;
        }
        SPUtils sp = sSPMap.get(key);
        if(sp == null) {
            synchronized (SPUtils.class) {
                if (sp == null) {
                    sp = new SPUtils(name);
                    sSPMap.put(key, sp);
                }
            }
        }
        return sp;
    }

    /***
     * 迁移所有sp,如果新应用，不需要调用
     */
    public void importFromSp(String key){
        String name = TextUtils.isEmpty(key) ? DEFAULT_KEY : key;
        SharedPreferences oldSp = Utils.getContext().getSharedPreferences(name, Context.MODE_PRIVATE);
        if(oldSp != null) {
            mv.importFromSharedPreferences(oldSp);
            oldSp.edit().clear().apply();
        }
    }

    /**
     * 获取SP实例
     *
     * @return {@link SPUtils}
     */

    private SPUtils(String key) {
        if(TextUtils.isEmpty(key)) {
            mv = MMKV.defaultMMKV();
        }else {
            mv = MMKV.mmkvWithID(key);
        }
        //importFromSp(key);
    }

    /**
     * 保存数据的方法，我们需要拿到保存数据的具体类型，然后根据类型调用不同的保存方法
     *
     * @param key
     * @param object
     */
    public void encode(String key, Object object) {
        if(mv == null){
            return;
        }
        if (object instanceof String) {
            mv.encode(key, (String) object);
        } else if (object instanceof Integer) {
            mv.encode(key, (Integer) object);
        } else if (object instanceof Boolean) {
            mv.encode(key, (Boolean) object);
        } else if (object instanceof Float) {
            mv.encode(key, (Float) object);
        } else if (object instanceof Long) {
            mv.encode(key, (Long) object);
        } else if (object instanceof Double) {
            mv.encode(key, (Double) object);
        } else if (object instanceof byte[] ) {
            mv.encode(key, (byte[]) object);
        } else {
            mv.encode(key, object.toString());
        }
    }

    public void encodeSet(String key,Set<String> sets) {
        mv.encode(key, sets);
    }

    public void encodeParcelable(String key,Parcelable obj) {
        mv.encode(key, obj);
    }


    /**
     * 得到保存数据的方法，我们根据默认值得到保存的数据的具体类型，然后调用相对于的方法获取值
     */
    public Integer decodeInt(String key,int defaultValue) {
        return mv.decodeInt(key, defaultValue);
    }
    public Double decodeDouble(String key) {
        return mv.decodeDouble(key, 0.00);
    }
    public Long decodeLong(String key) {
        return mv.decodeLong(key, 0L);
    }
    public Boolean decodeBoolean(String key,boolean defaultValue) {
        return mv.decodeBool(key, defaultValue);
    }
    public Float decodeFloat(String key) {
        return mv.decodeFloat(key, 0F);
    }
    public byte[] decodeBytes(String key) {
        return mv.decodeBytes(key);
    }
    public String decodeString(String key) {
        return mv.decodeString(key,"");
    }
    public String decodeString(String key,String defaultValue) {
        return mv.decodeString(key,defaultValue);
    }
    public Set<String> decodeStringSet(String key) {
        return mv.decodeStringSet(key, Collections.<String>emptySet());
    }
    public Parcelable decodeParcelable(String key) {
        return mv.decodeParcelable(key, null);
    }
    /**
     * 移除某个key对
     *
     * @param key
     */
    public void removeKey(String key) {
        mv.removeValueForKey(key);
    }
    /**
     * 清除所有key
     */
    public void clearAll() {
        mv.clearAll();
    }

}
